#! /usr/bin/perl -w

use POSIX;

my $chopEnd = 40000;
my $keepLen = 500000;

if ($#ARGV > 1) {
    $chopEnd = shift @ARGV;
    $keepLen = shift @ARGV;
}

$filename = $ARGV[0];
if ($filename =~ /\-(\d+)/) {
    $thrCount = $1;
} else {
    $thrCount = 0;
}

my @durations = ();
my @timestamps = ();
my @successes = ();
my $lastStamp = 0;
while (<>) {
    # Regular expression extracts relevant fields from the log entry. 
    if (/t=\"(\d+)\".*\ ts=\"(\d+)\".*\ s=\"([^\"]+)\"/) {
	my $duration = $1; # operation time in milliseconds
	my $timestamp = $2; # unix timestamp 
	my $success = $3; # boolean success indicator for this operation

	$timestamp = $lastStamp if ($timestamp == 0);
	$lastStamp = $timestamp;

	push @timestamps, $timestamp;
	push @durations, $duration;
	push @successes, $success;
    } else {
	next if (/^$/);
	next if ($_ eq "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	next if ($_ eq "<testResults version=\"1.2\">\n");
	next if ($_ eq "</testResults>\n");
	print "Unmatched: $_";
    }
}

my $minTimeStamp;
for (my $i = 0; $i < $#timestamps; $i++) {
    if ($timestamps[$i] > 0) {
	$minTimeStamp = $timestamps[$i];
	while (--$i >= 0) {
	    $timestamps[$i] = $minTimeStamp;
	}
	last;
    }
}

my $maxTimeStamp = $timestamps[$#timestamps - 1];
$maxTimeStamp -= $chopEnd;

if ($minTimeStamp < $maxTimeStamp - $keepLen) {
    $minTimeStamp = $maxTimeStamp - $keepLen;
} else {
    $keepLen = $maxTimeStamp - $minTimeStamp;
}


# my $startIdx;
# for (my $i = 0; $i < $#timestamps; $i++) {
#     if ($timestamps[$i] > $minTimeStamp) {
# 	$startIdx = $i;
# 	last;
#     }
# }
# if ($startIdx < $thrCount * 2) {
#     print STDERR "WARNING: startIdx i low. Not enough samples?";
# }

# my $endIdx;
# for (my $i = $#timestamps - 1; $startIdx < $i; $i--) {
#     if ($timestamps[$i] <= $maxTimeStamp) {
# 	$endIdx = $i + 1;
# 	last;
#     }
# }


&printResults(0, $#timestamps - 1, "ALL");

# Now try using 1-hour windows, from the start of the test.
my $startIdx = 0;
my $lastTimeStamp = $timestamps[0];
my $width = 60 * 60 * 1000;
my $firstStamp = $lastTimeStamp;
for (my $i = 0; $i < $#timestamps; $i++) {
    if ($timestamps[$i] >= $lastTimeStamp + $width) {
	&printResults($startIdx, $i, POSIX::floor(($lastTimeStamp - $firstStamp) / $width));
	$lastTimeStamp += $width;
	$startIdx = $i;
    }
}
&printResults($startIdx, $#timestamps, "tail");


sub printResults {
    my $startIdx = shift @_;
    my $endIdx = shift @_;
    my $title = shift @_;
# Throw away all disregarded samples:
#    my @selectedTimestamps = splice @timestamps, $startIdx, ($endIdx - $startIdx);
#    my @selectedDurations = splice @durations, $startIdx, ($endIdx - $startIdx);
#    my @selectedSuccesses = splice @successes, $startIdx, ($endIdx - $startIdx);

    my @selectedTimestamps = @timestamps[$startIdx .. $endIdx];
    my @selectedDurations =  @durations[$startIdx .. $endIdx];
    my @selectedSuccesses =  @successes[$startIdx .. $endIdx];
    my $windowWidth = $selectedTimestamps[$#selectedTimestamps - 1] - $selectedTimestamps[0];

# Calc stats on selected data
    my $cnt = $#selectedTimestamps;
    my $successCount = 0;
    my $totalDuration = 0;
    for (my $i = 0; $i < $#selectedTimestamps; $i++) {
	if ($selectedSuccesses[$i] eq 'true') {
	    $successCount++;
	    $totalDuration += $durations[$i];
	} 
    }
    
    my $errorCount = $cnt - $successCount;
    my @percentiles = sort { $a <=> $b } @selectedDurations;
    my $p50 = $percentiles[$errorCount + POSIX::floor(50.0 * $successCount / 100.0)];
    my $p90 = $percentiles[$errorCount + POSIX::floor(90.0 * $successCount / 100.0)];
    my $p95 = $percentiles[$errorCount + POSIX::floor(95.0 * $successCount / 100.0)];
    my $p98 = $percentiles[$errorCount + POSIX::floor(98.0 * $successCount / 100.0)];
    my $p99 = $percentiles[$errorCount + POSIX::floor(99.0 * $successCount / 100.0)];
    my $maximum = $percentiles[$#percentiles - 1];
    my $tph = POSIX::floor($successCount * (60 * 60 * 1000 / $windowWidth));
    my $average = POSIX::floor($totalDuration / $successCount);
# "title,windowsWidth,TPH,average,median,p90,p95,p98,p99,maximum,transactions,successes,errors"
    print "$title,$windowWidth,$tph,$average,$p50,$p90,$p95,$p98,$p99,$maximum,$cnt,$successCount,$errorCount\n";
}
