/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

import com.trifork.sosigw.api.Intercom;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
public class BootStrapRejectorFilter implements Filter {
    private static final Logger logger = Logger.getLogger(BootStrapRejectorFilter.class);

    private static final String ERR_404 = "Sending '404' as we are bootstrapping. Expected client behaviour is to try another server.";

    private static final Logger log = Logger.getLogger(BootStrapRejectorFilter.class); 

    private static boolean killed = false;

    public void init(FilterConfig filterconfig) throws ServletException {
    }

    public void destroy() {
    }

    public void doFilter(ServletRequest servletrequest, ServletResponse servletresponse, FilterChain filterchain) throws IOException, ServletException {
        if (requestDisallowed()) {
            HttpServletResponse resp = (HttpServletResponse)servletresponse;
            resp.setStatus(404);
            resp.addHeader("Retry-After", "5"); // In 5 seconds, or try other host. (Documented for 503, so not really for 404, but lets see...)
            resp.setContentType("text/xml");
            PrintWriter writer = resp.getWriter();    
            writer.write("<error>SOSI-GW is bootstrapping. Please try another server or retry later.</error>");
            log.warn(ERR_404);
            System.err.println(ERR_404);
            return;
        }
        filterchain.doFilter(servletrequest, servletresponse);
    }

    public static boolean requestDisallowed() {
        Intercom  intercom = Factory.getIntercom();
        boolean disallowed = (intercom == null || !intercom.isBootstrapped() || killed);
        logger.debug("disallowed: " + disallowed + ", intercom: " + intercom + ", killed: " + killed + ", filter: " + BootStrapRejectorFilter.class);
        return disallowed;
    }

    public static void kill() {
        killed = true;
    }

    public static void start() {
        killed = false;
    }
}
