/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.auditlog;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
public class AlertSender {
    private static final Logger log = Logger.getLogger(AlertSender.class);

    private static final long INTERVAL = 1000 * 60* 60; // ms to wait between sending the same alert.

    private static Map<String,Long> alreadySent = new HashMap<String, Long>();
    /**
     * Send an alert to the administrator of the system about a serious failure in need of attention, such as lack of ability to contact database.
     * Do not invoke from too many locations... 
     * Must not throw any kind of exception.
     * @param alert
     */
    public static void sendAlert(String alert) {
        try {
            log.debug("SendAlert invoked with " + alert);
            Long at = alreadySent.get(alert);
            if (at == null || at.longValue() + INTERVAL < System.currentTimeMillis()) {
                sendMessage(alert);
                alreadySent.put(alert, new Long(System.currentTimeMillis()));
                        }
        } catch (Exception any) {
            log.error("Unable to send alert ", any);
        }
    }

    private static void sendMessage(String alert) {
        //TODO: Extension point: Send alert to administrator through smtp or similar.
    }
}
