/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.auditlog;

import java.sql.Connection;
import java.sql.PreparedStatement;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.sql.DataSource;

import org.apache.log4j.Logger;

import com.trifork.sosigw.Factory;
import com.trifork.sosigw.config.StaticConfigImpl;

/**
 * 
 * @author Jeppe Sommer <jso@trifork.com>
 *
 */
public abstract class AuditLog2DB implements AuditLogger {
    public static final Logger log = Logger.getLogger(AuditLog2DB.class); 

    protected static final String SOSIGWLOG_SOSIGWLOG = "sosigwlog.SOSIGWLOG";
    private static AuditLogger instance;

    public static AuditLogger getInstance() {
        if (instance == null) {
            String jdbcDriver = StaticConfigImpl.getInstance().getJdbcDriver().toLowerCase();
            if (jdbcDriver.contains("mysql")) {
                instance = new AuditLogMysql();
            } else if (jdbcDriver.contains("postgresql")) {
                instance = new AuditLogPostgresql();
            } else if (jdbcDriver.contains("hsqldb")) {
                instance = new AuditLogHsqldb();
            } else {
                log.fatal("Could not determine type of database support from jdbc driver \"" + StaticConfigImpl.getInstance().getJdbcDriver() + "\"");
            }
        }
        return instance;

    }
    
    protected Connection getConnectionFromPool() throws Exception {
        String name = StaticConfigImpl.getInstance().getDataSourceName();
        Context ctx = new InitialContext();

        DataSource ds = (DataSource)ctx.lookup(name);
        if (ds == null) {
            throw new IllegalStateException("Unable to lookup Datasource in jndi from '" + name + "'");
        }
        
        return ds.getConnection();
    }
    
    abstract protected String getInsertLogSql();
    
    public void log(AuditLogItem item) throws Exception {
        Connection con = getConnectionFromPool();
        try {
            PreparedStatement stmt = con.prepareStatement(getInsertLogSql());
            stmt.setString(1, Factory.getIntercom().getUID());
            stmt.setString(2, item.operation);
            stmt.setString(3, item.cpr);
            stmt.setTimestamp(4, new java.sql.Timestamp(item.timestamp));
            stmt.setString(5, item.wsaTo);
            stmt.setString(6, item.wsaAction);
            stmt.setString(7, item.clientIP);
            stmt.executeUpdate();
            stmt.close();
        } finally {
            con.close();
        }
    }
}
