/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.auditlog;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.sql.Types;


/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
public class AuditLogGlobalHsqldb extends AuditLogGlobal {
    private static final String SOSIGWGLOBALLOG_PKSEQ = SOSIGWGLOBAL_PREFIX + ".PKSEQ";
    private static final String SOSIGWGLOBALLOG_STATICPKSEQ = SOSIGWGLOBAL_PREFIX + ".STATICPKSEQ";

    protected String getInsertIntoGlobalLogSql() {
        return "INSERT INTO " + SOSIGWGLOBALLOG_SOSIGWGLOBALLOG + " VALUES (NEXTVAL('" + SOSIGWGLOBALLOG_PKSEQ + "'), ?, ?, ?, ?, ?, ?)";
    }

    protected String getInsertIntoWsaSql() {
        return "INSERT INTO " + SOSIGWGLOBALLOG_WSA + " VALUES (NEXTVAL('" + SOSIGWGLOBALLOG_STATICPKSEQ + "'), ?, ?)";
    }

    protected String getInsertIntoOperation() {
        return "INSERT INTO " + SOSIGWGLOBALLOG_OPERATION + " VALUES (NEXTVAL('" + SOSIGWGLOBALLOG_STATICPKSEQ + "'), ?)";
    }

    protected String getInsertIntoClientIpSql() {
        return "INSERT INTO " + SOSIGWGLOBALLOG_CLIENTIP + " VALUES (NEXTVAL('" + SOSIGWGLOBALLOG_STATICPKSEQ + "'), ?)";
    }

    protected String getInsertIntoSystemIdSql() {
        return "INSERT INTO " + SOSIGWGLOBALLOG_SYSTEMID + " VALUES (NEXTVAL('" + SOSIGWGLOBALLOG_STATICPKSEQ + "'), ?)";
    }

    /*
     * This is the old implementation not using SELECT FOR UPDATE. This is not supported by HSQLDB
     * @see com.trifork.sosigw.auditlog.AuditLogGlobal#transferLogItems()
     */

    protected void transferLogItems() throws SQLException {
        PreparedStatement insertStmt = globalDB.prepareStatement("INSERT INTO " + SOSIGWGLOBALLOG_SOSIGWGLOBALLOG + " VALUES (NEXTVAL('" + SOSIGWGLOBALLOG_PKSEQ + "'), ?, ?, ?, ?, ?, ?)");
        Statement stmt = localDB.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_UPDATABLE);
        PreparedStatement delete = localDB.prepareStatement("DELETE FROM " + SOSIGWLOCALLOG_SOSIGWLOG + " WHERE lpk = ?");
        if (stmt.execute("SELECT * FROM " + SOSIGWLOCALLOG_SOSIGWLOG)) {
            ResultSet rs = stmt.getResultSet();
            while (rs.next()) {
                //long lpk = rs.getLong(1);
                String uid = rs.getString(2);
                String oper = rs.getString(3);
                String cpr = rs.getString(4);
                Timestamp timestamp = rs.getTimestamp(5);
                String wsaTo = rs.getString(6);
                String wsaAction = rs.getString(7);
                String clientIP = rs.getString(8);
                
                Integer systemIdState = systemidStates.get(uid);
                if (systemIdState == null) {
                    continue; // unknown systemid, we will find it next time round.
                }
                Integer operId = ensureOperation(oper);
                Integer wsaId = ensureWSA(wsaTo, wsaAction);
                Integer clienID = ensureClientIP(clientIP);

                insertStmt.setInt(1, systemIdState.intValue());
                insertStmt.setInt(2, operId.intValue());
                insertStmt.setString(3, cpr);
                insertStmt.setTimestamp(4, timestamp);
                if (wsaId != null) {
                    insertStmt.setInt(5, wsaId.intValue());
                } else {
                    insertStmt.setNull(5, Types.INTEGER);
                }
                insertStmt.setInt(6, clienID);
                insertStmt.addBatch();
                
                delete.setLong(1, rs.getLong("lpk"));
                delete.executeUpdate();
            }
            try { rs.close(); } catch (SQLException e) {}
        }
        insertStmt.executeBatch();
        insertStmt.close();
        delete.close();
    }


}
