/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.browser;

import com.trifork.sosigw.Factory;
import com.trifork.sosigw.api.Constants;
import com.trifork.sosigw.api.IDCardCache;
import com.trifork.sosigw.api.IDCardCacheEntry;
import com.trifork.sosigw.api.IDCardState;
import com.trifork.sosigw.cluster.IDCardClusterCacheEntry;
import com.trifork.sosigw.config.RuntimeConfig;
import com.trifork.sosigw.util.Util;
import org.apache.log4j.Logger;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Map;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
public class BrowserBasedSigning {
    private static final Logger log = Logger.getLogger(BrowserSigningServlet.class); 

    public static final String URL_ID = "ID";
    public static final String URL_CK = "CK";
    private static final int SECRET_BITLENGTH = 128; // Careful, it is quite expensive to make these bytes...

    
    public static String prepareBrowserSigning(IDCardCacheEntry cachedIDCard, byte[] bytesForSigning) {
        String signingURL = makeSigningURL(cachedIDCard);
        cachedIDCard.setBytesForSigning(bytesForSigning);
        return signingURL;
    }
    
    
    protected static String makeSigningURL(IDCardCacheEntry cachedIDCard) {
        String secret = Util.makeSecretKey(SECRET_BITLENGTH);
        cachedIDCard.setSecret(secret);
        
        String partition = null;
        if (RuntimeConfig.isCachePartitioningEnabled()) {
            // idCardCache.ensureIDCard(cacheKey) has checked the validity of this lookup
            partition = Util.getHttpServletRequest().getHeader(RuntimeConfig.getCachePartitioningHttpHeaderName());
        }
        
        StringBuffer url = new StringBuffer();
        url.append(RuntimeConfig.getBrowserSigningUrlBase(partition));
        try {
            url.append("?" + URL_ID + "=").append(URLEncoder.encode(cachedIDCard.getCacheKey(), Constants.UTF_8));
        } catch (UnsupportedEncodingException e) {
            // Cannot happen
        }
        url.append("&" + URL_CK + "=").append(secret);
        return url.toString();
    }

    public static IDCardClusterCacheEntry findAndCheckIDCardCacheEntry(Map<String, String[]> parameterMap) {
        String[] ids = parameterMap.get(URL_ID);
        String[] cks = parameterMap.get(URL_CK);
        if (ids == null || cks == null) {
            log.error("Missing arguments id/ck");
            return null;
        }
        
        String id = ids[0];
        String ck = cks[0];
        final IDCardCache idCardCache = Factory.getIDCardCache(); 
        final IDCardClusterCacheEntry cachedIDCardEntry = idCardCache.findIDCardOrNull(id);
        if (cachedIDCardEntry != null) {
            if (cachedIDCardEntry.getState().equals(IDCardState.SIGNED)) {
                return cachedIDCardEntry;
            } else if (!cachedIDCardEntry.validateSecret(ck)) {
                log.error("IDcard failed on ck for params id='" + id + "' ck='" + ck + "'");
            } else if (cachedIDCardEntry.isTimedOutWaitingForSigning()) {
                log.info("IDcard timed out while waiting for browsersigning for params id='" + id + "' ck='" + ck + "'");
            } else {
                return cachedIDCardEntry;
            }
        } else {
            log.error("No matching idcard in cache for param id='" + id + "'");
        }
        return null;
    }
}
