/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.cluster.intercom;

import java.io.IOException;

import org.apache.log4j.Logger;

import com.trifork.sosigw.config.RuntimeConfig;

public class ConfigSender {
    private static final Logger log = Logger.getLogger(ConfigSender.class);
    private final String myUID;
    private final NetHandler netHandler;
    
    public ConfigSender(String myUID, NetHandler netHandler) {
        this.myUID = myUID;
        this.netHandler = netHandler;
    }

    public void sendCurrentRuntimeConfigState() {
        try {
            byte[] byteArray = Protocol.encodeVersion1RuntimeConfigStateAsBytes(RuntimeConfig.getRuntimeConfigState(), myUID);
            netHandler.sendMessage(byteArray);
        } catch (IOException e) {
            log.error("Error encoding RuntimeConfigState to stream", e);
        }
    }

    public void requestConfig(long timestampOfKnownConfig) {
        try {
            byte[] byteArray = Protocol.encodeVersion1ConfigRequestAsBytes(timestampOfKnownConfig, myUID);
            netHandler.sendMessage(byteArray);
        } catch (IOException e) {
            log.error("Error encoding ConfigRequest to stream", e);
        }
    }
    

}
