/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.cluster.intercom;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.net.SocketAddress;
import java.util.Map;

import com.trifork.sosigw.api.IDCardCacheEntry;
import com.trifork.sosigw.config.RuntimeConfigState;

public class Protocol {
    
    // Constants:
    public static final byte PROTOCOL_VERSION_1 = 1;
    public static final byte PROTOCOL_V1_IDCARDENTRY = 1;
    public static final byte PROTOCOL_V1_ALIVE = 2;
    public static final byte PROTOCOL_V1_SYNCREQUEST = 3;
    public static final byte PROTOCOL_V1_CONFIGREQUEST = 4;
    public static final byte PROTOCOL_V1_CONFIGSTATE = 5;



    public static byte[] encodeVersion1ServerStatesAsBytes(String uid, SlidingWindow<?> sentWindow, Map<String, Long> lastSenderSeqno) throws IOException {
        final ByteArrayOutputStream baos = new ByteArrayOutputStream(2048);
        final DataOutputStream dataOutputStream = new DataOutputStream(baos);
        dataOutputStream.writeByte(PROTOCOL_VERSION_1);
        synchronized (lastSenderSeqno) {
            dataOutputStream.writeInt(lastSenderSeqno.size() + 1);
            dataOutputStream.writeUTF(uid);
            dataOutputStream.writeLong(sentWindow.getTop());
            for (String senderUID : lastSenderSeqno.keySet()) {
                dataOutputStream.writeUTF(senderUID);
                Long l = lastSenderSeqno.get(senderUID);
                dataOutputStream.writeLong(l == null ? 0 : l);
            }
        }
        dataOutputStream.flush();
        byte[] byteArray = baos.toByteArray();
        return byteArray;
    }

    public static byte[] encodeVersion1RuntimeConfigStateAsBytes(RuntimeConfigState runtimeConfigState, String uid) throws IOException {
        final ByteArrayOutputStream baos = new ByteArrayOutputStream(16000);
        final ObjectOutputStream objectOutputStream = new ObjectOutputStream(baos);
        objectOutputStream.writeByte(Protocol.PROTOCOL_VERSION_1);
        objectOutputStream.writeByte(Protocol.PROTOCOL_V1_CONFIGSTATE);
        objectOutputStream.writeUTF(uid);
        objectOutputStream.writeObject(runtimeConfigState);
        objectOutputStream.flush();
        byte[] byteArray = baos.toByteArray();
        return byteArray;
    }

    public static byte[] encodeVersion1ConfigRequestAsBytes(long timestampOfKnownConfig, String uid) throws IOException {
        final ByteArrayOutputStream baos = new ByteArrayOutputStream(100);
        final ObjectOutputStream objectOutputStream = new ObjectOutputStream(baos);
        objectOutputStream.writeByte(Protocol.PROTOCOL_VERSION_1);
        objectOutputStream.writeByte(Protocol.PROTOCOL_V1_CONFIGREQUEST);
        objectOutputStream.writeUTF(uid);
        objectOutputStream.writeLong(timestampOfKnownConfig);
        objectOutputStream.flush();
        byte[] byteArray = baos.toByteArray();
        return byteArray;
    }

    public static byte[] encodeVersion1AliveAsBytes(long seqNo, long configTimestamp, String uid) throws IOException {
        final ByteArrayOutputStream baos = new ByteArrayOutputStream(100);
        final ObjectOutputStream objectOutputStream = new ObjectOutputStream(baos);
        objectOutputStream.writeByte(PROTOCOL_VERSION_1);
        objectOutputStream.writeByte(PROTOCOL_V1_ALIVE);
        objectOutputStream.writeUTF(uid);
        objectOutputStream.writeLong(seqNo);
        objectOutputStream.writeLong(configTimestamp);
        objectOutputStream.writeLong(System.currentTimeMillis());
        objectOutputStream.flush();
        byte[] byteArray = baos.toByteArray();
        return byteArray;
    }

    
    public static byte[] encodeVersion1SyncRequestAsBytes(SyncRequest syncReq, SocketAddress socketAddress) throws IOException {
        final ByteArrayOutputStream baos = new ByteArrayOutputStream(512);
        final ObjectOutputStream objectOutputStream = new ObjectOutputStream(baos);
        objectOutputStream.writeByte(PROTOCOL_VERSION_1);
        objectOutputStream.writeByte(PROTOCOL_V1_SYNCREQUEST);
        objectOutputStream.writeObject(syncReq);
        objectOutputStream.writeObject(socketAddress);
        objectOutputStream.flush();
        byte[] byteArray = baos.toByteArray();
        return byteArray;
    }

    public static byte[] encodeVersion1IDCardEntryAsBytes(IDCardCacheEntry entry, long seqNo, String myUID) throws IOException {
        final ByteArrayOutputStream baos = new ByteArrayOutputStream(20000);
        final ObjectOutputStream objectOutputStream = new ObjectOutputStream(baos);
        objectOutputStream.writeByte(PROTOCOL_VERSION_1);
        objectOutputStream.writeByte(PROTOCOL_V1_IDCARDENTRY);
        objectOutputStream.writeUTF(myUID);
        objectOutputStream.writeLong(seqNo);
        synchronized (entry) {
            objectOutputStream.writeObject(entry);   
        }
        objectOutputStream.flush();
        byte[] byteArray = baos.toByteArray();
        return byteArray;
    }

}
