/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.cluster.intercom;

import org.apache.log4j.Logger;

public abstract class Worker implements Runnable {
    private static final Logger log = Logger.getLogger(Worker.class);
    private final ClusterState state;
    
    public Worker(ClusterState state) {
        this.state = state;
    }
    
    public void run() {
        if (log.isDebugEnabled()) {
            log.debug(Thread.currentThread().getName() + ": starting");
        }
        while (!state.isKilled()) {
            //                if (log.isDebugEnabled()) {
            //                    log.debug(Thread.currentThread().getName() + ": run0");
            //                }
            try {
                run0();
            } catch (InterruptedException e) {
                if (!state.isKilled() && log.isDebugEnabled()) {
                    log.debug(Thread.currentThread().getName() + ": interrupted");
                }
                if (!state.isKilled()) {
                    log.warn("Ignored exception", e);
                }
            } catch (Exception e) {
                if (!state.isKilled()) {
                    log.warn("Ignored exception", e);
                }
            }
        }
        if (log.isDebugEnabled()) {
            log.debug(Thread.currentThread().getName() + ": Shutting down");
        }
    }

    protected abstract void run0() throws Exception;
}
