/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.config;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;


/**
 * Modifiable version of current state, meant for editing through the gui.
 * When done, commit changes something like:
 * 1) RuntimeConfigState newState = this.getResult();
 * 2) RuntimeConfig.setRuntimeConfigState(newState);
 * 3) Factory.getIntercom(). something... to be done...
 * @author Morten Grouleff <mgr@trifork.com>
 *
 */public class RuntimeConfigEditState extends RuntimeConfigState {

    RuntimeConfigEditState(RuntimeConfigItem[] state) {
        super(0, state);
    }

    public void replaceCategoryMap(String category, Map<String, String[]> catMap) {
        cacheByCategory.put(category, catMap);
    }
    
    public void addEntry(String category, String key, String[] values) {
        super.addEntry(category, key, values);
    }
    
    public RuntimeConfigState getResult() {
        List<RuntimeConfigItem> l = new ArrayList<RuntimeConfigItem>();
        for (String category: cacheByCategory.keySet()) {
            Map<String, String[]> catMap = cacheByCategory.get(category);
            for (String key: catMap.keySet()) {
                l.add(new RuntimeConfigItem(category, key, catMap.get(key)));
            }
            
        }
        return new RuntimeConfigState(System.currentTimeMillis(), l.toArray(new RuntimeConfigItem[l.size()]));
    }

    public void removeCategoryMap(String category) {
        cacheByCategory.remove(category);
    }
}
