/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.config;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.apache.log4j.Logger;

public class RuntimeConfigMapperProperties extends RuntimeConfigMapper {

    private static final Logger logger = Logger.getLogger(RuntimeConfigMapperProperties.class);

    private static Properties properties;
    public static final String RUNTIMECONFIG_KEY_PREFIX = "runtimeconfig.";
    private static Map<String, String> longCatagoryNames = new HashMap<String, String>() {
        {
            put("general", RuntimeConfig.CATEGORY_PROPERTIES);
            put("clientaccesswhitelist", RuntimeConfig.CATEGORY_CLIENTACCESSWHITELIST);
            put("serviceaccesswhitelist", RuntimeConfig.CATEGORY_SERVICEACCESSWHITELIST);
            put("servicepreservewsa", RuntimeConfig.CATEGORY_SERVICEPRESERVE_WSA);
            put("storage", "Dummy storage field, not used by program");
        }
    };

    protected RuntimeConfigMapperProperties(Properties properties) {
        RuntimeConfigMapperProperties.properties = properties;
    }

    public RuntimeConfigState loadLatestConfig() {
        ArrayList<RuntimeConfigItem> items = new ArrayList<RuntimeConfigItem>();

        for (Object propertyKey : properties.keySet()) {
            String propertyName = (String)propertyKey;
            if (propertyName.startsWith(RUNTIMECONFIG_KEY_PREFIX)) {
                checkPropertyKeyContainsAtLeastThreeComponents(propertyName);

                // remove "runtimeconfig."
                String propertyNameWithoutRuntimeconfig = stripFirstDottedComponent(propertyName);
                // remove category short name
                String propertyNameWithoutCategory = stripFirstDottedComponent(propertyNameWithoutRuntimeconfig);

                String category = getLongCategory(propertyName);

                String key = propertyNameWithoutCategory;
                String[] value = properties.getProperty(propertyName).split(",");
                items.add(new RuntimeConfigItem(category, key, value));
            }
        }

        logger.info("Loaded properties: " + items);

        return new RuntimeConfigState(Long.MAX_VALUE, items.toArray(new RuntimeConfigItem[items.size()]));
    }

    private void checkPropertyKeyContainsAtLeastThreeComponents(String s) {
        if (s.split("\\.").length < 3) {
            throw new ConfigurationException("Runetime configuration key " + s + " not parseable. Must contain at least three components.");
        }
    }

    private String getLongCategory(String propertyName) {
        String propertyNameWithoutRuntimeconfig = stripFirstDottedComponent(propertyName); // remove "runtimeconfig."
        String shortCategory = getFirstDottedComponent(propertyNameWithoutRuntimeconfig);
        String category = longCatagoryNames.get(shortCategory);
        if (category == null) {
            throw new ConfigurationException("Unknown category name " + shortCategory
                    + " specified as part of configuration property " + propertyName);
        }
        return category;
    }

    private String getFirstDottedComponent(String stringWithDots) {
        return stringWithDots.substring(0, stringWithDots.indexOf('.'));
    }

    private String stripFirstDottedComponent(String stringWithDots) {
        return stringWithDots.substring(stringWithDots.indexOf('.') + 1);
    }

    public void saveConfig(RuntimeConfigState state) throws UnsupportedOperationException {
        throw new UnsupportedOperationException("saveConfig");
    }
}
