/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.client;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.Widget;
import com.trifork.sosigw.console.client.util.DefaultCallback;
import com.trifork.sosigw.console.service.ConsoleFacade;
import com.trifork.sosigw.console.service.ConsoleFacadeAsync;

/**
 * Entry point classes define <code>onModuleLoad()</code>.
 */
// /CLOVER:OFF
public class ConsoleApp implements EntryPoint {

    /**
     * This is the entry point method.
     */
    public void onModuleLoad() {
        ConsoleFacadeAsync async = initFacade();
        async.runtimeConfigIsInPropertiesMode(new DefaultCallback() {
            public void onSuccess(Object result) {
                boolean inPropertiesMode = ((Boolean)result).booleanValue();
                setupGui(inPropertiesMode);
            }
        });
    }

    ConsoleFacadeAsync initFacade() {
        final ConsoleFacadeAsync consoleFacadeService = (ConsoleFacadeAsync)GWT.create(ConsoleFacade.class);

        // (2) Specify the URL at which our service implementation is running.
        // Note that the target URL must reside on the same domain and port from
        // which the host page was served.
        //
        ServiceDefTarget endpoint = (ServiceDefTarget)consoleFacadeService;
        String moduleRelativeURL = GWT.getModuleBaseURL() + "console/facade";
        endpoint.setServiceEntryPoint(moduleRelativeURL);

        return consoleFacadeService;
    }

    private void setupGui(boolean inPropertiesMode) {
        Widget widget = null;

        if (inPropertiesMode) {
            widget = new ConsoleIsDisabledWidget();
        } else {
            MainComposite mainWindow = new MainComposite();
            Window.enableScrolling(false);
            widget = mainWindow;
        }

        RootPanel rootPanel = RootPanel.get("ConsoleAppHook");
        if (rootPanel != null) {
            rootPanel.add(widget);
        }
    }
}
