/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.client;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.WindowResizeListener;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.Tree;
import com.google.gwt.user.client.ui.TreeItem;
import com.google.gwt.user.client.ui.TreeListener;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.trifork.sosigw.console.client.panels.AuditLogSearchPanel;
import com.trifork.sosigw.console.client.panels.ConfigFolder;
import com.trifork.sosigw.console.client.panels.DefaultFolder;
import com.trifork.sosigw.console.client.panels.IDCardSearchPanel;
import com.trifork.sosigw.console.service.ConsoleFacade;
import com.trifork.sosigw.console.service.ConsoleFacadeAsync;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
///CLOVER:OFF
public class NavTreeComposite extends Composite {
    
    private final MainComposite mainComposite;
    private ScrollPanel scrollPanel;

    NavTreeComposite(MainComposite mainComposite) {
        
        this.mainComposite = mainComposite;

        VerticalPanel panel = new VerticalPanel();
        panel.setWidth("100%");
        
        HTML div = new HTML("<table cellpadding=\"0\" cellspacing=\"0\" width=\"100%\" border=\"0\" class=\"BcmGryTbl\"><tr><td><div class=\"BcmGryDiv\"><span class=\"BcmTxt\">Navigation</span></div></td></tr></table>");

        panel.add(div);
        
        Tree navTree = createNavTree();

        scrollPanel = new ScrollPanel(navTree);
        scrollPanel.setWidth("100%");
        
        Window.addWindowResizeListener(new WindowResizeListener() {
            public void onWindowResized(int width, int height) {
                resetScrollPanel(width, height);
            }
        });

//        int clientHeight = Window.getClientHeight();
//
//        int top = panel.getAbsoluteTop();
//        scrollPanel.setHeight(clientHeight + "px");
//        scrollPanel.setHeight("100%");
//        scrollPanel.setHeight("300px");
        panel.add(scrollPanel);
        
        
        initWidget(panel);
    }
    

    Tree createNavTree() {
        Tree navTree = new Tree();
//        navTree.setStyleName("DefBdy");

        final ConsoleFacadeAsync consoleFacadeService = (ConsoleFacadeAsync)GWT.create(ConsoleFacade.class); 

        // (2) Specify the URL at which our service implementation is running.
        // Note that the target URL must reside on the same domain and port from
        // which the host page was served.
        //
        ServiceDefTarget endpoint = (ServiceDefTarget) consoleFacadeService;
        String moduleRelativeURL = GWT.getModuleBaseURL() + "console/facade";
        endpoint.setServiceEntryPoint(moduleRelativeURL);

        Context context = new Context();
        context.setNavTree(navTree);
        context.setConsoleService(consoleFacadeService);

        context.setTabbedEditor(mainComposite.getTabbedEditor());
        
        DefaultFolder server = new DefaultFolder(context, null, "SOSI-GW");
        
        // Search ID Card
        IDCardSearchPanel idSearch = new IDCardSearchPanel(context, server, "Search ID Card");
        server.addSubNode(idSearch);
        
        // Browse Audit Log
        AuditLogSearchPanel auditLogSearch = new AuditLogSearchPanel(context, server, "Browse Audit Log");
        server.addSubNode(auditLogSearch);

        // Cluster members
//        DefaultFolder servers = new DefaultFolder(context, server, "Cluster Members");
//        server.addSubNode(servers);
        
        DefaultFolder configuration = new ConfigFolder(context, server);
        server.addSubNode(configuration);

        
        
        server.addToNavTree();

        navTree.addTreeListener(new TreeListener() {

            public void onTreeItemSelected(TreeItem item) {
                // TODO Auto-generated method stub
                
            }

            public void onTreeItemStateChanged(TreeItem item) {
            }
            
        });
        
        return navTree;
    }

    public void resetScrollPanel(int windowWidth, int windowHeight) {
        int scrollPanelTop = scrollPanel.getAbsoluteTop();
        scrollPanel.setHeight(windowHeight - scrollPanelTop + "px");
    }

}
