/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.client;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.SourcesTabEvents;
import com.google.gwt.user.client.ui.TabListener;
import com.google.gwt.user.client.ui.TabPanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.trifork.sosigw.console.client.panels.AbstractNode;
import com.trifork.sosigw.console.client.util.Queue;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
///CLOVER:OFF
public class TabbedEditor extends Composite {
    
    /**
     * The listener interface for receiving tabClosed events.
     * The class that is interested in processing a tabClosed
     * event implements this interface, and the object created
     * with that class is registered with a component using the
     * component's <code>addTabClosedListener<code> method. When
     * the tabClosed event occurs, that object's appropriate
     * method is invoked.
     * 
     * @see TabClosedEvent
     */
    public interface TabClosedListener {
        public void tabClosed();
    }

    public class TabWidget extends Panel {

        private ArrayList widgets = new ArrayList();
        private String icon;
        private String name;
        private final TabClosedListener listener;
        
        public TabWidget(String icon, String name, final TabClosedListener listener) {
            this.icon = icon;
            this.name = name;
            this.listener = listener;
            
            init();
        }
        
        private void init() {
            Element top = DOM.createDiv();
            setElement(top);

            String html = "<table style=\"width:100%\"><tr><td align=\"left\"> <img height=\"16px\" width=\"16px\" src=\""
                + icon + "\"/>&nbsp;" + name.replaceAll(" ", "&nbsp;")
                + "&nbsp;</td><td align=\"right\"></tr></table>";

            DOM.setInnerHTML(getElement(), html);
            
            Element tableElm = DOM.getChild(top, 0);
            Element tbodyElm = DOM.getChild(tableElm, 0);
            Element trElm = DOM.getChild(tbodyElm, 0);
            Element tdElm = DOM.getChild(trElm, 1);
            
            Image closeTabIcon = new Image("images/icons/cross.png");
            
            closeTabIcon.setSize("16px", "16px");

            widgets.add(closeTabIcon);
            adopt(closeTabIcon, tdElm);

            closeTabIcon.addClickListener(new ClickListener() {
                public void onClick(Widget sender) {
                    listener.tabClosed();
                }
            });
        }

        public Iterator iterator() {
            return widgets.iterator();
        }

        public boolean remove(Widget w) {
            disown(w);
            return widgets.remove(w);
        }

        public void setModified(boolean modified) {
            Element top = getElement();
            Element tableElm = DOM.getChild(top, 0);
            Element tbodyElm = DOM.getChild(tableElm, 0);
            Element trElm = DOM.getChild(tbodyElm, 0);
            Element tdElm = DOM.getChild(trElm, 0);

            String tdHtml;
            
            if (modified) {
                tdHtml = "<img height=\"16px\" width=\"16px\" src=\""
                    + icon + "\"/>&nbsp;*" + name.replaceAll(" ", "&nbsp;")
                    + "&nbsp";
            } else {
                tdHtml = "<img height=\"16px\" width=\"16px\" src=\""
                    + icon + "\"/>&nbsp;" + name.replaceAll(" ", "&nbsp;")
                    + "&nbsp";
            }
            DOM.setInnerHTML(tdElm, tdHtml);
        }
    }

    static class MyTabPanel extends TabPanel {
        protected Widget getWidget() {
            return super.getWidget();
        }
    }
    
    private static final int MAX_TABS = 5;
    private MyTabPanel tabPanel;
    private Queue tabQueue;
//    private HorizontalPanel statusBar;
    private HashMap panel2TabWidget = new HashMap();
    
    
    TabbedEditor() {
        VerticalPanel panel = new VerticalPanel();
        
        panel.setWidth("100%");
        
        tabPanel = new MyTabPanel();
        tabPanel.setWidth("100%");

        // TODO: Extension point: Let the maximum number of tabs depend of the actual window size
        tabQueue = new Queue(MAX_TABS);
        tabPanel.addTabListener(new TabListener() {

            public boolean onBeforeTabSelected(SourcesTabEvents sender, int tabIndex) {
                return true;
            }

            public void onTabSelected(SourcesTabEvents sender, int tabIndex) {
                tabQueue.refresh(tabPanel.getWidget(tabIndex));
            }
        });
        panel.add(tabPanel);

//        statusBar = new HorizontalPanel();
//        statusBar.setVisible(false);
//        
//        ((VerticalPanel)tabPanel.getWidget()).insert(statusBar, 1);
//
//        Label label = new Label("Some editors contain unsaved changes. Press ");
//        statusBar.add(label);
//        statusBar.addStyleName("statusbar-highlight");
//        statusBar.setWidth("100%");

//        Button b = new Button("Save");
//        b.addClickListener(new ClickListener() {
//            public void onClick(Widget sender) {
//                doSaveAll();
//                updateNeedsSaving();
//            }
//        });
//        statusBar.add(b);
        
        initWidget(panel);

    }
    
    public void openTab(final String name, String icon, final AbstractNode panel) {

        // TabPanel.getWidgetIndex(Widget) searches by object identity - not equality
        int idx = findWidget(panel);
        if (idx != -1) {
            tabPanel.selectTab(idx);
        } else {

            if (tabQueue.isFull()) {
                // Remove least recently used tab
                AbstractNode lru = (AbstractNode) tabQueue.get();
                tabPanel.remove(lru);
                panel2TabWidget.remove(lru);
            }

//            String html = imageItemHTML(icon, name);

            TabWidget tabWidget = new TabWidget(icon, name, new TabClosedListener() {
                public void tabClosed() {
                    closeTab(panel);
                }
            });
            
            panel2TabWidget.put(panel, tabWidget);

            tabPanel.add(panel, tabWidget);
            tabQueue.put(panel);
            tabPanel.selectTab(tabPanel.getWidgetCount() - 1);
        }

        panel.resetScrollPanel(Window.getClientWidth(), Window.getClientHeight());
    }

    public void closeTab(AbstractNode panel) {
        int idx = findWidget(panel);
        if (idx != -1) {
            tabPanel.remove(idx);
            tabQueue.remove(panel);
            panel2TabWidget.remove(panel);
            
            AbstractNode firstInQueue = (AbstractNode) tabQueue.peep();
            if (firstInQueue != null) {
                idx = findWidget(firstInQueue);
                if (idx != -1) {
                    tabPanel.selectTab(idx);
                }
            }
        }
    }

//    public void updateNeedsSaving() {
//        boolean needsSaving = false;
//        for (int i = 0; i < tabPanel.getWidgetCount(); i++) {
//            Widget widget = tabPanel.getWidget(i);
//            if (widget instanceof AbstractResource) {
//                AbstractResource res = (AbstractResource)widget;
//                needsSaving |= res.isModified();
//            }
//        }
//        updateStatusBar(needsSaving);
//    }

//    private void updateStatusBar(boolean show) {
//        statusBar.setVisible(show);
//    }

    
//    protected void doSaveAll() {
//        for (int i = 0; i < tabPanel.getWidgetCount(); i++) {
//            Widget widget = tabPanel.getWidget(i);
//            if (widget instanceof AbstractResource) {
//                AbstractResource res = (AbstractResource)widget;
//                res.save();
//            }
//        }
//    }

    private int findWidget(AbstractNode panel) {
        for (int i = 0; i < tabPanel.getWidgetCount(); i++) {
            Widget widget = tabPanel.getWidget(i);
            if (widget.equals(panel)) {
                return i;
            }
        }

        return -1;
    }

    public void setStatus(AbstractNode panel, boolean modified) {
        TabWidget tabWidget = (TabWidget)panel2TabWidget.get(panel);

        if (tabWidget != null) {
            tabWidget.setModified(modified);
        }
            
    }

//    private String imageItemHTML(String imageUrl, String title) {
//        return "<span><img style='margin-right:4px; margin-top:0px; margin-bottom: 0px; border-top: 0px border-bottom: 0px; padding-top: 0px; padding-bottom: 0px' src='" + imageUrl.toLowerCase()
//        + "'>" + title + "</span>";
//    }

}
