/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.client.panels;

import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.WindowResizeListener;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.Hyperlink;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.TreeItem;
import com.google.gwt.user.client.ui.Widget;
import com.trifork.sosigw.console.client.Context;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
///CLOVER:OFF
public abstract class AbstractNode extends Composite implements Node {
    protected String nodeType;
    protected ScrollPanel scrollPanel;
    protected Folder parentFolder;
    protected final Context context;
    protected String name;
    protected String icon;
    protected TreeItem treeItem;
    

    public AbstractNode(Context context, Folder parent, String nodeType, String name) {
        this.context = context;
        this.nodeType = nodeType;
        this.name = name;
        this.parentFolder = parent;
    }

    public String getName() {
        return name;
    }
    
    public void addToNavTree() {
        ClickListener listener = new ClickListener(){
            public void onClick(Widget sender) {
                treeItemClicked();
            }
        };
        
        Hyperlink link = new Hyperlink(imageItemHTML(icon, name), true, "");
        treeItem = new TreeItem(link);
        link.addClickListener(listener);

        if (parentFolder != null) {
            parentFolder.getTreeItem().addItem(treeItem);
        } else {
            context.getNavTree().addItem(treeItem);
        }
    }
    
    public TreeItem getTreeItem() {
        return treeItem;
    }

    public static String imageItemHTML(String imageUrl, String title) {
        return "<span><img class='TreeIconImg' style=\"border: 0px\" src='" + imageUrl
        + "'>" + title + "</span>";
    }
    
    protected void showError(String message, Throwable reason) {
        new ErrorDialog(message, reason).show();
    }

//    private TreeItem addImageItem(TreeItem root, String title, String image) {
//        TreeItem item = new TreeItem(imageItemHTML(image, title));
//        root.addItem(item); 
//        return item;
//    }
//
//    protected TreeItem addImageItem(Tree root, String title, String image) {
//        TreeItem item = new TreeItem(imageItemHTML(image, title));
//        root.addItem(item);
//        return item;
//    }
//
//    private TreeItem addImageItem(TreeItem root, String title, String image, ClickListener listener) {
//        Hyperlink link = new Hyperlink(imageItemHTML(image, title), true, "");
//        TreeItem item = new TreeItem(link);
//        link.addClickListener(listener);
//        root.addItem(item); 
//        return item;
//    }

    protected void initWidget(Widget widget) {
        // Wrap widget in a scrollbar
        
        scrollPanel = new ScrollPanel(widget);
        
        scrollPanel.setWidth("100%");
        scrollPanel.setHeight("100%");
        
        Window.addWindowResizeListener(new WindowResizeListener() {
            public void onWindowResized(int width, int height) {
                resetScrollPanel(width, height);
            }
        });

        super.initWidget(scrollPanel);
    }
    
    public void resetScrollPanel(int windowWidth, int windowHeight) {
        int scrollPanelTop = scrollPanel.getAbsoluteTop();
        scrollPanel.setHeight(windowHeight - scrollPanelTop + "px");
    }

    protected void treeItemClicked() {
        openInEditor();
    }

    public void openInEditor() {
        context.getTabbedEditor().openTab(name, icon, this);
    }

    public void closeInEditor() {
        context.getTabbedEditor().closeTab(this);
    }
    
    public boolean equals(Object o) {
        AbstractNode other = (AbstractNode)o;
        if (other.nodeType.equals(nodeType)) {
            // Same type of node
            if (name.equals(other.getName())) {
                return true;
            }
        }
        
        return false;
    }

    protected void setParentFolder(Folder parentFolder) {
        this.parentFolder = parentFolder;
    }

    protected Folder getParentFolder() {
        return parentFolder;
    }

}
