/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.client.panels;

import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.CheckBox;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.trifork.sosigw.console.client.Context;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
///CLOVER:OFF
public abstract class AbstractResource extends AbstractNode implements Resource {

    protected Grid settingsTbl;
    protected ButtonPanel buttonPanel;
    protected VerticalPanel panel = new VerticalPanel();
    protected boolean create;
    protected TextBox nameField;
    protected TextBox descriptionField;
    protected int fieldCount;
    protected ResourceFolder resourceFolder;
    protected Button createButton;
    protected Button cancelButton;
    protected Button saveButton;
    protected Button deleteButton;
    protected boolean modified;
    
    public AbstractResource(Context context, Folder parentFolder, String type, String name, final boolean create, int fieldCount) {
        super(context, parentFolder, type, name);

        this.create = create;
        this.fieldCount = fieldCount;
        this.resourceFolder = (ResourceFolder)parentFolder;
        
        buttonPanel = new ButtonPanel();

        panel.add(buttonPanel);

        if (create) {
            createButton = buttonPanel.addButton("Create", new ClickListener() {
                public void onClick(Widget sender) {
                    AbstractResource.this.name = nameField.getText();

                    doCreate();

                    // Succesfully created - add to tree view
                    //AbstractResource.this.parent.addSubNode(AbstractResource.this);
                }
            });

            cancelButton = buttonPanel.addButton("Cancel", new ClickListener() {
                public void onClick(Widget sender) {
                    closeInEditor();
                }
            });
        } else {
            saveButton = buttonPanel.addButton("Save", new ClickListener() {
                public void onClick(Widget sender) {
                    doSave();
                }
            });
            saveButton.setEnabled(false);

            deleteButton = buttonPanel.addButton("Delete", new ClickListener() {
                public void onClick(Widget sender) {
                    doDelete();
                }
            });
        }

        initSettingsTbl();
        if (!create) {
            loadAndInit();
        }
        
        panel.setWidth("100%");
//        panel.setHeight("100%");

        initWidget(panel);
    }

    protected abstract void doCreate();

    protected void initSettingsTbl() {
        if (settingsTbl == null) {
            settingsTbl = new Grid(fieldCount, 2);
            settingsTbl.setStyleName("CntntTbl");
            panel.add(settingsTbl);
        }

        nameField = addTextField(0, "Name");

        nameField.setText(name);

        if (!create) {
            nameField.setEnabled(false);
        }

        descriptionField = addTextField(1, "Description");
    }

    protected abstract void loadAndInit();
    
    abstract protected void doSave();
    
    public abstract void doDelete();

    public abstract String[] getColumnText();
    
    protected TextBox addTextField(int r, String name) {
        
        settingsTbl.setHTML(r, 0, name);
        settingsTbl.getCellFormatter().setStyleName(1, 0, "TblActLbl");
        TextBox field = new TextBox();
        settingsTbl.setWidget(r, 1, field);
        field.setWidth("50em");

        return field;
    }

    protected ListBox addListBoxField(int r, String name) {
        
        settingsTbl.setHTML(r, 0, name);
        settingsTbl.getCellFormatter().setStyleName(r, 0, "TblActLbl");
        ListBox field = new ListBox();
        field.setWidth("50em");
        settingsTbl.setWidget(r, 1, field);
        
        return field;
    }

    protected CheckBox addCheckBoxField(int r, String name) {
        settingsTbl.setHTML(r, 0, name);
        settingsTbl.getCellFormatter().setStyleName(r, 0, "TblActLbl");
        CheckBox field = new CheckBox();
        settingsTbl.setWidget(r, 1, field);
        
        return field;
    }



    protected void resourceCreated() {
        // Remove from tabbed editor.
        closeInEditor();
        
        // Add to tree view
        Resource resource = resourceFolder.createResource(context, name, false);
        resource.setParentFolder(resourceFolder);
        resource.addToNavTree();
                
        //  (optionally?) reopen in tabbed editor.
        resource.openInEditor();
    }
    
    protected void resourceInitialized() {
        if (resourceFolder != null) {
            resourceFolder.addResource(this);
        }
    }

    protected void resourceDeleted() {

        if (resourceFolder != null) {
            // Remove from Folder panel.. 
            resourceFolder.removeResource(this);
            
            // .. and tree view
            resourceFolder.removeSubNode(this);
        }

        closeInEditor();
    }

    public void setParentFolder(ResourceFolder parentFolder) {
        super.setParentFolder(parentFolder);
        this.resourceFolder = (ResourceFolder)parentFolder;
    }

    public void openInEditor() {
        super.openInEditor();
        
        // If creating, the specified name is probably not what the user wants. Make it selected and in focus
        if (create) {
            nameField.setSelectionRange(0, name.length());
            nameField.setFocus(true);
        }
    }

    public void setModified(boolean modified) {
        if (modified != this.modified) {
            saveButton.setEnabled(modified);
            context.getTabbedEditor().setStatus(this, modified);
            this.modified = modified;
        }
    }
    
    public boolean isModified() {
        return modified;
    }
}
