/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.client.panels;

import java.util.HashSet;
import java.util.Iterator;

import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.trifork.sosigw.console.client.Context;
import com.trifork.sosigw.console.client.widget.ResourceTable;


/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
///CLOVER:OFF
public abstract class AbstractResourceFolder extends DefaultFolder implements ResourceFolder {

    protected ButtonPanel buttonPanel;
    protected VerticalPanel panel = new VerticalPanel();
    protected String title;
    protected String guide;
    protected ResourceTable resTbl;
    
    HashSet subNodeSet = new HashSet();
    boolean initialized = false;
    private HTML guideField;
    
    public AbstractResourceFolder(Context context, Folder parentFolder, String name) {
        super(context, parentFolder, name);

        panel.setWidth("100%");

        initWidget(panel);
    }

    protected void init() {
        if (!initialized ) {
            
            panel.add(new HTML("<h1>" + title + "</h1>"));
            if (guide != null && guide.length() > 0) {
                panel.add(new HTML("<hr>"));
                guideField = new HTML(guide);
                panel.add(guideField);
            }
            panel.add(new HTML("<hr>"));
            
            doInit();

            initialized = true;
        }
    }

    public void setGuide(String guide) {
        guideField.setHTML(guide);
    }
    
    protected abstract void doInit();

//    protected abstract String getObjectName();

    protected abstract void fetch();

    public void addSubNode(AbstractNode node) {
        
        if (subNodeSet.add(node)) {
            // node is not already a sub node so really add it.
            super.addSubNode(node);
            node.setParentFolder(this);
            node.addToNavTree();
        }
        
    }


    public void addToNavTree() {
        super.addToNavTree();
        
        init();
        fetch();

        Iterator i = subNodes.iterator();
        while (i.hasNext()) {
            ((DefaultFolder)i.next()).addToNavTree();
        }
    }

    public void openInEditor() {

        init();
        
        super.openInEditor();
    }

    public void updateResource(Resource resource) {
        resTbl.updateResource(resource);
    }

    public void removeSubNode(Resource resource) {
        subNodeSet.remove(resource);

        super.removeSubNode(resource);
    }

    public void addResource(Resource resource) {
        init();
        
        if (resTbl != null) {
            resTbl.addResource(resource);
        }
    }

    public void removeResource(Resource resource) {
        resTbl.removeResource(resource);
    }

    protected void treeItemClicked() {
        openInEditor();
    }
}
