/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.client.panels;


import java.util.Date;

import com.google.gwt.user.client.ui.TextBox;
import com.trifork.sosigw.console.client.Context;
import com.trifork.sosigw.console.datamodel.AuditLogEntry;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
///CLOVER:OFF
public class AuditLogEntryPanel extends AbstractResource {

    private static final String LOGENTRY_TYPE = "AuditLogEntry";

//    private TextBox timestampField;
    private TextBox cprField;
    private TextBox systemIDField;
    private TextBox clientIPField;
    private TextBox operationField;
    private TextBox wsaActionField;
    private TextBox wsaToField;

    private final AuditLogEntry logEntry;

    public AuditLogEntryPanel(Context context, Folder parentFolder, AuditLogEntry logEntry, boolean create) {

        super(context, parentFolder, LOGENTRY_TYPE, new Date(logEntry.getTimeStamp()).toLocaleString(), create, 8);
        this.logEntry = logEntry;

        cprField.setText(logEntry.getCpr()); 
        systemIDField.setText(logEntry.getSystemID());
        clientIPField.setText(logEntry.getClientIP()); 
        operationField.setText(logEntry.getOperation()); 
        wsaActionField.setText(logEntry.getWsaAction());
        wsaToField.setText(logEntry.getWsaTo()); 
        
        icon = "images/icons/book_open.png";
    }


    protected void loadAndInit() {
        // Preloaded in constructor
    }

    protected void initSettingsTbl() {
        super.initSettingsTbl();
        
        cprField = addTextField(2, "CPR");
        cprField.setEnabled(false);
        
        systemIDField = addTextField(3, "System ID");
        systemIDField.setEnabled(false);
        
        clientIPField = addTextField(4, "Client IP");
        clientIPField.setEnabled(false);
        
        operationField = addTextField(5, "Operation Field");
        operationField.setEnabled(false);
        
        wsaActionField = addTextField(6, "WSA:Action");
        wsaActionField.setEnabled(false);
        
        wsaToField = addTextField(7, "WSA:To");
        wsaToField.setEnabled(false);
        
        
        settingsTbl.setWidth("100%");
    }

    public String[] getColumnText() {
        return new String[] { name, logEntry.getCpr(), logEntry.getWsaAction(), logEntry.getWsaTo() };
    }

    public void doDelete() {
    }

    protected void doCreate() {
        // TODO Auto-generated method stub
        
    }

    protected void doSave() {
        // TODO Auto-generated method stub
        
    }
}
