/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.client.panels;

import java.util.Date;

import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ChangeListener;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.widgetideas.datepicker.client.DatePicker;
import com.google.gwt.widgetideas.datepicker.client.impl.DatePickerCell;
import com.trifork.sosigw.console.client.Context;
import com.trifork.sosigw.console.client.util.DefaultCallback;
import com.trifork.sosigw.console.client.widget.ResourceTable;
import com.trifork.sosigw.console.datamodel.AuditLogEntry;
import com.trifork.sosigw.console.datamodel.AuditLogQuery;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
///CLOVER:OFF
public class AuditLogSearchPanel extends AbstractNode {
    private static final String AUDIT_LOG_SEARCH_TYPE = "IDCardSearch";
    static final DateTimeFormat dateFormat = DateTimeFormat.getFormat("dd/MM-yyyy");
    protected VerticalPanel panel = new VerticalPanel();
    private boolean initialized = false;
    protected String title;
    protected String guide;

    ResourceTable resTbl;
    
    public AuditLogSearchPanel(Context context, Folder parentFolder, String name) {
        super(context, parentFolder, AUDIT_LOG_SEARCH_TYPE, name);

        panel.setWidth("100%");
        icon = "images/icons/book_open.png";

        initWidget(panel);
    }

    protected void init() {
        if (!initialized ) {
            
            title = "Browse Audit Log";
            guide = "Fill out the relevant fields and press the Search button";
            
            panel.add(new HTML("<h1>" + title + "</h1>"));
            if (guide != null && guide.length() > 0) {
                panel.add(new HTML("<hr>" + guide));
            }
            panel.add(new HTML("<hr>"));
            
            doInit();

            panel.add(new HTML("<hr>"));

            initialized = true;
        }
    }

    public void doInit() {
        
        final AuditLogQuery query = new AuditLogQuery();
        
        final TextBox fromDateField = createPickableDateField();
        final TextBox toDateField = createPickableDateField();

        final TextBox fromTimeField = new TextBox();
        final TextBox toTimeField = new TextBox();
        
        final TextBox cprField = new TextBox();
        
        Button searchButton = new Button("Search");
        searchButton.addClickListener(new ClickListener() {
            public void onClick(Widget sender) {
                
                Long from = combineDateTime(fromDateField.getText(), fromTimeField.getText());
                Long to = combineDateTime(toDateField.getText(), toTimeField.getText());

                query.setMinTimeStamp(from);
                query.setMaxTimeStamp(to);
                
                String cpr = cprField.getText().trim(); 
                if (cpr.length() > 0) {
                    query.setCpr(cpr);
                }
                
                context.getConsoleService().searchAuditLog(query, 0, -1, new DefaultCallback() {

                    public void onSuccess(Object result) {
                        AuditLogEntry[] entries = (AuditLogEntry[])result;

                        presentResult(entries);
                    }
                    
                });
            }
        });
        
        Grid grid = new Grid(4, 4);

        grid.setWidget(0, 0, new HTML("CPR"));
        grid.setWidget(0, 1, cprField);
        grid.setWidget(1, 0, new HTML("From Date"));
        grid.setWidget(1, 1, fromDateField);
        grid.setWidget(1, 2, new HTML("At (hh:mm:ss)"));
        grid.setWidget(1, 3, fromTimeField);
        grid.setWidget(2, 0, new HTML("To Date"));
        grid.setWidget(2, 1, toDateField);
        grid.setWidget(2, 2, new HTML("At (hh:mm:ss)"));
        grid.setWidget(2, 3, toTimeField);
        grid.setWidget(3, 3, searchButton);
        
        panel.add(grid);
    }
    
    protected Long combineDateTime(String dateString, String timeString) {
        String d = dateString.trim();
        String t = timeString.trim();
        
        if ((d.length() == 0) && (t.length() == 0)) {
            // Neither date nor time is specified
            return null;
        }

        int year, month, dayInMonth;
        int hour, min, sec; 
        
        Date date;
        
        if (d.length() == 0) {
            // Assume today if time but not date is specified
            date = new Date();
        } else {
            date = dateFormat.parse(d);
        }

        year = date.getYear();
        month = date.getMonth();
        dayInMonth = date.getDate();

        if (t.length() == 0) {
            // Assume midnight if date but not time is specified
            hour = min = sec = 0;
        } else {
            int i1 = t.indexOf(':');
            hour = Integer.parseInt(t.substring(0, i1));
            int i2 = t.indexOf(':', i1 + 1);
            min = Integer.parseInt(t.substring(i1 + 1, i2));
            sec = Integer.parseInt(t.substring(i2 + 1));
        }

        return new Long(new Date(year, month, dayInMonth, hour, min, sec).getTime());
    }

    protected void presentResult(AuditLogEntry[] entries) {
        if (resTbl != null) {
            panel.remove(resTbl);
        }

        resTbl = new ResourceTable(new String[] { "When", "CPR", "WSA:Action", "WSA:To" }, "Search Result", new ResourceTable.ResourceCreator() {
            public void newResource() {
            }
        }, false);

        for (int i = 0; i < entries.length; i++) {
            AuditLogEntryPanel entryPanel = new AuditLogEntryPanel(context, null, entries[i], false);
            resTbl.addResource(entryPanel);
        }

        panel.add(resTbl);
    }

    private TextBox createPickableDateField() {
        final TextBox dateField = new TextBox();
        dateField.addClickListener(new ClickListener() {

            public void onClick(Widget sender) {
                final boolean[] done = new boolean[] { false }; 
                
                final DatePicker datePicker = new DatePicker() {
                    public void onClick(Widget sender) {
                        if (sender instanceof DatePickerCell) {

                            DatePickerCell cell = (DatePickerCell)sender;
                            if (cell.type() == 0) {
                                done[0] = true;
                            }
                        }
                        
                        super.onClick(sender);
                    }
                };
                datePicker.setStylePrimaryName("goog-date-picker");

                String current = dateField.getText();
                if (current.trim().length() > 0) {
                    try {
                        Date date = dateFormat.parse(current);
                        datePicker.setFullDate(date);
                    } catch (Exception ex) {
                        
                    }
                }
                
                datePicker.setPopupPosition(dateField.getOffsetWidth() + 50, dateField.getOffsetHeight() + 50);
                datePicker.show();
                datePicker.addChangeListener(new ChangeListener() {
                    public void onChange(Widget sender) {
                        if (done[0]) {
                            dateField.setText(dateFormat.format(datePicker.selectedDate()));
                            datePicker.removeFromParent();
                        }
                    }
                });
            }
        });
        return dateField;
    }
    public void addToNavTree() {
        super.addToNavTree();     
    }

    public void openInEditor() {
        init();
        super.openInEditor();
    }
}
