/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.client.panels;


import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Widget;
import com.trifork.sosigw.console.client.Context;
import com.trifork.sosigw.console.client.util.DefaultCallback;
import com.trifork.sosigw.console.client.widget.ResourceTable;
import com.trifork.sosigw.console.datamodel.ConfigCategory;
import com.trifork.sosigw.console.datamodel.ConfigItem;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
///CLOVER:OFF
public class CategoryFolder extends AbstractResourceFolder implements Resource {

    protected static final boolean CATEGORIES_ARE_MUTABLE = false;

    protected ConfigCategory category;
    protected boolean doInitNotDone = true;
    
    
    public CategoryFolder(Context context, ResourceFolder parent, String name, boolean create) {
        super(context, parent, name);
     
//        icon = "images/icons/config_folder.gif";
        icon = "images/icons/folder_database.png";

        title = name;
        guide = "Manage the configuration of the SOSI-GW cluster.";

        addButtonPanel(create);
    }
    
    private void addButtonPanel(boolean create) {
        buttonPanel = new ButtonPanel();

        panel.add(buttonPanel);

//        buttonPanel.addButton("Save", new ClickListener() {
//            public void onClick(Widget sender) {
//                doSave();
//            }
//        });

        if (CATEGORIES_ARE_MUTABLE) {
            buttonPanel.addButton("Delete", new ClickListener() {
                public void onClick(Widget sender) {
                    doDelete();
                }
            });
        }

    }

    public void doDelete() {
        context.getConsoleService().removeConfigCategory(category.getName(), new DefaultCallback() {

            public void onSuccess(Object result) {
            }
        });
    }

    protected void doSave() {
        context.getConsoleService().saveConfigCategory(category, new DefaultCallback() {

            public void onSuccess(Object result) {
            }
        });
    }

    protected void doInit() {

        if (category != null && doInitNotDone) {
            String[] columns = category.getColumns();
            String[] tableColumns = new String[columns.length + 1];
            tableColumns[0] = "Key";
            for (int i = 1; i < tableColumns.length; i++) {
                tableColumns[i] = columns[i - 1];
            }

            setGuide(category.getDescription());
            
            resTbl = new ResourceTable(tableColumns, "Current Configuration Entries", new ResourceTable.ResourceCreator() {

                public void newResource() {
                    String initialName = "New Configuration Entry";
                    ConfigEntry entry = new ConfigEntry(context, CategoryFolder.this, initialName, null, true);
                    entry.setParentFolder(CategoryFolder.this);

                    entry.openInEditor();
                }
            });

            panel.add(resTbl);
            
            ConfigItem[] configItems = category.getConfigItems();
            
            for (int i = 0; i < configItems.length; i++) {
                ConfigEntry entry = new ConfigEntry(context, CategoryFolder.this, configItems[i].getKey(), configItems[i], false);

                addSubNode(entry);
                resTbl.addResource(entry);
            }

            doInitNotDone = true;
        }
    }
    
    protected void fetch() {
        context.getConsoleService().getConfigCategory(getName(), new DefaultCallback() {

            public void onSuccess(Object result) {
                category = (ConfigCategory) result;

                if (initialized) {
                    // Don't call doInit yet if AbstractResourceFolder.init not yet called
                    doInit();
                }
            }
        });
    }

    public Resource createResource(Context context, String name, boolean create) {

        return new ConfigEntry(context, this, name, null, create);
    }
    
    public void addResource(Resource resource) {
        super.addResource(resource);
        
        ConfigEntry entry = (ConfigEntry)resource;
        category.addItem(entry.configItem);
    }

    public String[] getColumnText() {
        return new String[] { name, category != null ? category.getDescription() : ""};
    }

    public void setParentFolder(ResourceFolder parentFolder) {
        this.parentFolder = parentFolder;
    }

}
