/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.client.panels;

import com.google.gwt.user.client.ui.KeyboardListener;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.Widget;
import com.trifork.sosigw.console.client.Context;
import com.trifork.sosigw.console.client.util.DefaultCallback;
import com.trifork.sosigw.console.datamodel.ConfigCategory;
import com.trifork.sosigw.console.datamodel.ConfigItem;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
///CLOVER:OFF
public class ConfigEntry extends AbstractResource {

    private static final String CONFIGURATION_ENTRY_TYPE = "ConfigurationEntry";

    private TextBox[] valueFields;

    protected ConfigItem configItem;
    

    public ConfigEntry(Context context, Folder parentFolder, String name, ConfigItem item, boolean create) {

        super(context, parentFolder, CONFIGURATION_ENTRY_TYPE, name, create, 6);
        this.configItem = item;

//        if (saveButton != null) {
//            saveButton.setHTML("Update");
//        }
        
        icon = "images/icons/page_edit.png";

        if (configItem != null) {
            String[] values = configItem.getValues();
            for (int i = 0; i < values.length; i++) {
                valueFields[i].setText(values[i]);
            }
        }
    }


    protected void loadAndInit() {
        // Already loaded with the ConfigCategory
    }

    protected void initSettingsTbl() {
        super.initSettingsTbl();
        
//        ChangeListener changeListener = new ChangeListener() {
//            public void onChange(Widget sender) {
//                if (configItem != null) {
//                    setModified(true);
////                    context.getTabbedEditor().updateNeedsSaving();
//                }
//            }
//        };

        KeyboardListener keyboardListener = new KeyboardListener() {

            public void onKeyDown(Widget sender, char keyCode, int modifiers) {
                if (configItem != null) {
                    setModified(true);
                }
            }

            public void onKeyPress(Widget sender, char keyCode, int modifiers) {
            }

            public void onKeyUp(Widget sender, char keyCode, int modifiers) {
            }
        };
        String[] columns = ((CategoryFolder)parentFolder).category.getColumns();
        
        valueFields = new TextBox[columns.length];
        
        for (int i = 0; i < columns.length; i++) {
            TextBox box = addTextField(i + 2, columns[i]);
//            box.addChangeListener(changeListener);
            box.addKeyboardListener(keyboardListener);
            valueFields[i] = box;
        }
        
        settingsTbl.setWidth("100%");
    }


    public String[] getColumnText() {
        String[] result = new String[valueFields.length + 1];
        
        result[0] = name;
        for (int i = 0; i < valueFields.length; i++) {
            result[i + 1] = valueFields[i].getText();
        }
        
        return result;
    }

    protected void doSave() {
        updateFromGui();

        context.getConsoleService().saveConfigItem(configItem, new DefaultCallback() {

            public void onSuccess(Object result) {
                setModified(false);
                
                resourceFolder.updateResource(ConfigEntry.this);
            }
        });

        
    }

    protected void doCreate() {
        updateFromGui();

        context.getConsoleService().saveConfigItem(configItem, new DefaultCallback() {

            public void onSuccess(Object result) {
//                setModified(false);
//                
                closeInEditor();
                
                // Add to tree view
                Resource resource = new ConfigEntry(context, resourceFolder, name, configItem, false);
                resource.setParentFolder(resourceFolder);
                resourceFolder.addResource(resource);
                resource.addToNavTree();
                
                parentFolder.openInEditor();
            }
        });

        
        
    }


    public void doDelete() {
        final ConfigCategory category = ((CategoryFolder)parentFolder).category;

        context.getConsoleService().removeConfigItem(configItem.getCategory().getName(), configItem.getKey(), new DefaultCallback() {

            public void onSuccess(Object result) {
                category.removeItem(configItem);
                resourceFolder.updateResource(ConfigEntry.this);

                resourceDeleted();
            }
        });


    }

    private void updateFromGui() {
        
        String[] values = new String[valueFields.length];
        for (int i = 0; i < valueFields.length; i++) {
            values[i] = valueFields[i].getText();
        }
        if (configItem == null) {
            configItem = new ConfigItem(((CategoryFolder)parentFolder).category, name, values);
        } else {
            configItem.setValues(values);
        }
    }

}
