/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.client.panels;


import com.google.gwt.user.client.ui.TextBox;
import com.trifork.sosigw.console.client.Context;
import com.trifork.sosigw.console.client.util.DefaultCallback;
import com.trifork.sosigw.console.datamodel.IDCard;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
///CLOVER:OFF
public class IDCardPanel extends AbstractResource {

    private static final String IDCARD_TYPE = "IDCard";

    protected IDCard idCard;

    private TextBox givenNameField;
    private TextBox surNameField;
    private TextBox emailField;
    private TextBox occupationField;
    private TextBox roleField;
    private TextBox authCodeField;
    private TextBox stateField;
    
    public IDCardPanel(Context context, Folder parentFolder, IDCard idCard, boolean create) {

        super(context, parentFolder, IDCARD_TYPE, idCard.getCpr(), create, 7);

        this.idCard = idCard;
        
        givenNameField.setText(idCard.getGivenName());
        surNameField.setText(idCard.getSurName());
        emailField.setText(idCard.getEmail());
        occupationField.setText(idCard.getOccupation());
        roleField.setText(idCard.getRole());
        authCodeField.setText(idCard.getAuthorizationCode());
        stateField.setText(idCard.getState());
        
        icon = "images/icons/find.png";
    }


    protected void loadAndInit() {
        // Preloaded in constructor
    }

    protected void initSettingsTbl() {
        super.initSettingsTbl();
        
        givenNameField = addTextField(2, "Given Name");
        givenNameField.setEnabled(false);
        
        surNameField = addTextField(3, "Surname");
        surNameField.setEnabled(false);

        emailField = addTextField(4, "Email");
        emailField.setEnabled(false);
        
        occupationField = addTextField(5, "Occupation");
        occupationField.setEnabled(false);
        
        roleField = addTextField(5, "Role");
        roleField.setEnabled(false);

        authCodeField = addTextField(5, "Authorization Code");
        authCodeField.setEnabled(false);
        
        stateField = addTextField(6, "State");
        stateField.setEnabled(false);
        
        settingsTbl.setWidth("100%");
    }

    public String[] getColumnText() {
        return new String[] { idCard.getCpr(), idCard.getGivenName() + " " + idCard.getSurName(), idCard.getState() };
    }

    public void doDelete() {
        context.getConsoleService().revokeIdCard(idCard.getNameId(), new DefaultCallback() {

            public void onSuccess(Object result) {
                // TODO Auto-generated method stub
                
            }
        });
    }

    protected void doCreate() {
        // TODO Auto-generated method stub
        
    }

    protected void doSave() {
        // TODO Auto-generated method stub
        
    }
}
