/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.client.panels;

import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.trifork.sosigw.console.client.Context;
import com.trifork.sosigw.console.client.util.DefaultCallback;
import com.trifork.sosigw.console.client.widget.ResourceTable;
import com.trifork.sosigw.console.datamodel.IDCard;
import com.trifork.sosigw.console.datamodel.IDCardQuery;
import com.trifork.sosigw.console.datamodel.SearchResult;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
///CLOVER:OFF
public class IDCardSearchPanel extends AbstractNode {
    private static final String ID_CARD_SEARCH_TYPE = "IDCardSearch";
    protected VerticalPanel panel = new VerticalPanel();
    private boolean initialized = false;
    protected String title;
    protected String guide;
    ResourceTable resTbl;
    
    public IDCardSearchPanel(Context context, Folder parentFolder, String name) {
        super(context, parentFolder, ID_CARD_SEARCH_TYPE, name);

        panel.setWidth("100%");
        icon = "images/icons/find.png";

        initWidget(panel);
    }

    protected void init() {
        if (!initialized ) {
            
            title = "Search ID Card";
            guide = "Enter a name ID and press the Search button.";
            
            panel.add(new HTML("<h1>" + title + "</h1>"));
            if (guide != null && guide.length() > 0) {
                panel.add(new HTML("<hr>"));
                panel.add(new HTML(guide));
            }
            panel.add(new HTML("<hr>"));
            
            doInit();

            panel.add(new HTML("<hr>"));

            initialized = true;
        }
    }

    public void doInit() {
        final TextBox newKeyField = new TextBox();
        //newKeyField.setWidth("25em");

        HorizontalPanel hPanel = new HorizontalPanel();
        panel.add(hPanel);
        
        Button newKeyButton = new Button("Search");
        newKeyButton.addClickListener(new ClickListener() {
            public void onClick(Widget sender) {
                IDCardQuery query = new IDCardQuery();
                query.setCpr(newKeyField.getText());
                
                context.getConsoleService().searchIDCard(query, new DefaultCallback() {

                    public void onSuccess(Object searchResult) {
                        SearchResult result = (SearchResult)searchResult;

                        presentResult(result);
                    }
                    
                });
            }
        });

        hPanel.add(newKeyButton);
        
        hPanel.add(newKeyField);

    }
    
    protected void presentResult(SearchResult result) {
        if (resTbl != null) {
            panel.remove(resTbl);
        }

        resTbl = new ResourceTable(new String[] { "CPR", "Name", "Status" }, "Search Result", new ResourceTable.ResourceCreator() {
            public void newResource() {
            }
        });

        IDCard[] idCards = result.getIDCards();
        for (int i = 0; i < idCards.length; i++) {
            IDCardPanel idCardPanel = new IDCardPanel(context, null, idCards[i], false);
            resTbl.addResource(idCardPanel);
        }

        panel.add(resTbl);
    }
    
    public void addToNavTree() {
        super.addToNavTree();     
    }

    public void openInEditor() {
        init();
        super.openInEditor();
    }
}
