/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.client.util;



/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
public class Queue {
    private Object[] rep;
    private int put, get;
    private int size;
    private int capacity;

    public Queue(int capacity) {
        rep = new Object[capacity];
        this.size = 0;
        this.capacity = capacity;

        put = 0;
        get = 0;
    }
    
    public void put(Object o) {
        if (size == capacity) {
            throw new IllegalStateException("Queue is full");
        }

        rep[put] = o;
        incrPut();

        size++;
    }
    
    public Object get() {
        if (size == 0) {
            return null;
        }
        
        size--;

        Object result = rep[get];
        incrGet();
        
        return result;
    }

    public Object peep() {
        if (size == 0) {
            return null;
        }
        
        return rep[get];
    }

    public void refresh(Object o) {
        // Locate object

        
        for (int i = get, cnt = 0; cnt < size; i = incr(i), cnt++) {
            if (rep[i] == o) {
                // Shift all objects 'before' o by one
                int j = i;
                for (; cnt < size - 1; j = incr(j), cnt++) {
                    rep[j] = rep[incr(j)];
                }
                // Add o to the end
                rep[j] = o;
                break;
            }
        }
    }
    
    public void remove(Object o) {
        // Locate object

        for (int i = get, cnt = 0; cnt < size; i = incr(i), cnt++) {
            if (rep[i] == o) {
                // Shift all objects 'before' o by one
                int j = i;
                for (; cnt < size - 1; j = incr(j), cnt++) {
                    rep[j] = rep[incr(j)];
                }
                // Remove last
                decrPut();
                size--;
                break;
            }
        }
    }

    
    public int size() {
        return size;
    }
    

    public boolean isFull() {
        return size == capacity;
    }
    
    public boolean isEmpty() {
        return size == 0;
    }

    private final void incrGet() {
        if (++get == capacity) {
            get = 0;
        }
    }

    private final void incrPut() {
        if (++put == capacity) {
            put = 0;
        }
    }

    private final void decrPut() {
        if (--put < 0) {
            put = capacity - 1;
        }
    }
    
    private final int incr(int i) {
        if (++i == capacity) {
            return 0;
        }
        return i;
    }
}