/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.client.widget;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Iterator;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.CheckBox;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Hyperlink;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.Widget;
import com.trifork.sosigw.console.client.panels.AbstractNode;
import com.trifork.sosigw.console.client.panels.Resource;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
///CLOVER:OFF
public class ResourceTable extends Panel {

    class ResourceEntry {
        final Resource resource;
        int rowCount;
        CheckBox checkBox; 
        Hyperlink resourceLink;
        Element trElem;
        
        ResourceEntry (Resource resource) {
            this.resource = resource;
        }

        Element createTrElem() {
            String[] rowText = resource.getColumnText();

            if (rowText.length != columnHeaders.length) {
                throw new IndexOutOfBoundsException("Unexpected column count " + rowText.length
                        + " (expected " + columnHeaders.length + ")");
            }
            
            Element newTrElem = DOM.createTR();

            if (isMutable) {
                checkBox = new CheckBox();
                Element checkBoxTdElem = DOM.createTD();

                adopt(checkBox, checkBoxTdElem);
                DOM.appendChild(newTrElem, checkBoxTdElem);
                
                checkBox.addClickListener(new ClickListener() {

                    public void onClick(Widget sender) {
                        if (checkBox.isChecked()) {
                            delButton.setEnabled(true);
                        } else {
                            checkDeleteButton();
                        }
                    }
                });
            }

            for (int c = 0; c < rowText.length; c++) {
                Element tdElem = DOM.createTD();

                if (c == sortColumn) {
//                    DOM.setElementAttribute(tdElem, "className", "TblTdSrt");
                    setStyleName(tdElem, "TblTdSrt");
                }

                if (c == 0) {
                    // Make links to resources in 'Name' column
                    resourceLink = new Hyperlink(rowText[c], true, "");
                    
                    resourceLink.addClickListener(new ClickListener() {
                        public void onClick(Widget sender) {
                            resource.openInEditor();
                        }
                    });

                    DOM.appendChild(tdElem, resourceLink.getElement());
                    adopt(resourceLink, tdElem);
                    
                } else {
                    DOM.setInnerHTML(tdElem, rowText[c]);
                }
                
                DOM.appendChild(newTrElem, tdElem);
            }

            return newTrElem;
        }
        
        void addToTable() {
            trElem = createTrElem();
            DOM.appendChild(bodyElem, trElem);
        }

        void removeFromTable() {
            if (trElem != null) {
                DOM.removeChild(bodyElem, trElem);
                if (checkBox != null) {
                    disown(checkBox);
                }
                disown(resourceLink);
            }
        }
        
        void updateInTable() {
            if (trElem != null) {
                int idx = DOM.getChildIndex(bodyElem, trElem);
                removeFromTable();

                Element newTrElem = createTrElem();
                DOM.insertChild(bodyElem, newTrElem, idx);

                trElem = newTrElem;
            }
        }
        
        public boolean isChecked() {
            return checkBox.isChecked();
        }

        public void deleteResource() {
            resource.doDelete();
        }
    }
    
    public interface ResourceCreator {
        public void newResource();
    }
    
    private final String[] columnHeaders;
    private Element tableElem;
    private Element bodyElem;
    private Element captionElem;
    private int sortColumn = -1;
    private int sortDirection = 1;
    private Button newButton;
    private Button delButton;
    private ArrayList widgets = new ArrayList();
    private ArrayList resources = new ArrayList();
    private Hyperlink currentSortByLink;
    protected ResourceCreator resourceCreator;
    private final boolean isMutable;

    
    private Comparator resourceComparator = new Comparator() {

        public int compare(Object o1, Object o2) {
            ResourceEntry res1 = (ResourceEntry) o1;
            ResourceEntry res2 = (ResourceEntry) o2;

            String s1 = res1.resource.getColumnText()[sortColumn];
            String s2 = res2.resource.getColumnText()[sortColumn];

            if (sortDirection == 1) {
                return s1.compareTo(s2);
            } else {
                return -s1.compareTo(s2);
            }
        }
    };
    public ResourceTable(String[] columnHeaders, String caption, final ResourceCreator resourceCreator) {
        this(columnHeaders, caption, resourceCreator, true);
    }    

    public ResourceTable(String[] columnHeaders, String caption, final ResourceCreator resourceCreator, boolean isMutable) {
        this.columnHeaders = columnHeaders;
        this.resourceCreator = resourceCreator;
        this.isMutable = isMutable;

        tableElem = DOM.createTable();
        setStyleName(tableElem, "Tbl");
        
        captionElem = DOM.createCaption();
        DOM.setInnerHTML(captionElem, caption);
        setStyleName(captionElem, "TblTtlTxt");
        DOM.appendChild(tableElem, captionElem);
        
        bodyElem = DOM.createTBody();
        DOM.appendChild(tableElem, bodyElem);

        if (isMutable) {
            Element buttonTR = DOM.createTR();
            Element buttonTD = DOM.createTD();

            DOM.setElementAttribute(buttonTD, "colspan", "" + (columnHeaders.length + 1));
            setStyleName(buttonTD, "TblActTd");

            newButton = new TslButton("New", new ClickListener() {

                public void onClick(Widget sender) {
                    resourceCreator.newResource();
                }
            });
            
            adopt(newButton, buttonTD);
            
            delButton = new TslButton("Delete", new ClickListener() {

                public void onClick(Widget sender) {
                    deleteResources();
                }
            });

            delButton.setEnabled(false);
            
            adopt(delButton, buttonTD);

            DOM.appendChild(buttonTR, buttonTD);
            DOM.appendChild(bodyElem, buttonTR);
        }
        
        Element trElem = DOM.createTR();
        if (isMutable) {
            Element blankTd = DOM.createTD();
            DOM.appendChild(trElem, blankTd);
        }
        for (int c = 0; c < columnHeaders.length; c++) {
            final int column = c;
            Element tdElem = DOM.createTD();
            
            String linkHTML;
            linkHTML = columnHeaders[c];

            Hyperlink headerLink = new Hyperlink(linkHTML, true, "");
            
            headerLink.addClickListener(new ClickListener() {
                public void onClick(Widget sender) {
                    sortByColumn(sender, column);
                }
            });

            // Set style of <a> element within the Hyperlink's <div> 
            Element aElem = DOM.getChild(headerLink.getElement(), 0);

//            DOM.setElementAttribute(aElem, "className", "TblHdrLnk"); 
//            DOM.setElementAttribute(aElem, "class", "TblHdrLnk"); 

            setStyleName(aElem, "TblHdrLnk");
            
            widgets.add(headerLink);

            DOM.appendChild(tdElem, headerLink.getElement());
            DOM.appendChild(trElem, tdElem);
        }
        
        DOM.appendChild(bodyElem, trElem);
        
        setElement(tableElem);
        sinkEvents(Event.ONCLICK);
    }
    
    protected void checkDeleteButton() {
        boolean lastUnchecked = true;
        Iterator itor = resources.iterator();
        while (itor.hasNext()) {
            ResourceEntry entry = (ResourceEntry) itor.next();
            if (entry.isChecked()) {
                lastUnchecked = false;
            }
        }
        
        if (lastUnchecked) {
            delButton.setEnabled(false);
        }
    }

    protected void sortByColumn(Widget sender, int column) {
        Hyperlink link = (Hyperlink) sender;

        if (column == sortColumn) {
            sortDirection = -sortDirection;
        } else {
            sortDirection = 1;
            if (currentSortByLink != null) {
                // Remove up/down arrow from old sort column 
                String linkHTML = columnHeaders[sortColumn];
                currentSortByLink.setHTML(linkHTML);
            }

            sortColumn = column;
        }
            
        String linkHTML;
        if (sortDirection == 1) {
            linkHTML = AbstractNode.imageItemHTML("images/upArrow.gif", columnHeaders[column]);
        } else {
            linkHTML = AbstractNode.imageItemHTML("images/downArrow.gif", columnHeaders[column]);
        }
        link.setHTML(linkHTML);
        currentSortByLink = link;
            
        Object[] resourceEntries = resources.toArray();
        Arrays.sort(resourceEntries, resourceComparator);
        clearResourceEntries();
        for (int r = 0; r < resourceEntries.length; r++) {
            ((ResourceEntry)resourceEntries[r]).addToTable();
        }
    }

    private void clearResourceEntries() {
        Iterator i = resources.iterator();
        while (i.hasNext()) {
            ResourceEntry entry = (ResourceEntry) i.next();
            entry.removeFromTable();
        }
    }

    protected void deleteResources() {
        Iterator i1 = resources.iterator();
        ArrayList l  = new ArrayList();

        // First collect, then delete (avoid ConcurrentModificationException)
        while (i1.hasNext()) {
            ResourceEntry entry = (ResourceEntry) i1.next();
            if (entry.isChecked()) {
                l.add(entry);
            }
        }
        Iterator i2 = l.iterator();
        while (i2.hasNext()) {
            ResourceEntry entry = (ResourceEntry) i2.next();
            if (entry.isChecked()) {
                entry.deleteResource();
            }

        }
    }

    public Iterator iterator() {
        return widgets.iterator();
    }

    public boolean remove(Widget w) {
        // TODO Auto-generated method stub
        return false;
    }

    public void addResource(Resource resource) {
        
        ResourceEntry entry = new ResourceEntry(resource);
        resources.add(entry);
        
        entry.addToTable();
    }
    
    public void updateResource(Resource resource) {
        for (int i = 0; i < resources.size(); i++) {
            ResourceEntry entry = (ResourceEntry) resources.get(i);
            if (entry.resource == resource) {
                entry.updateInTable();
                break;
            }
        }
    }

    public void removeResource(Resource resource) {

        for (int i = 0; i < resources.size(); i++) {
            ResourceEntry entry = (ResourceEntry) resources.get(i);
            if (entry.resource == resource) {
                entry.removeFromTable();
                resources.remove(i);
                break;
            }
        }
        
        checkDeleteButton();
    }
    
    protected void adopt(Widget w, Element container) {
        super.adopt(w, container);
        widgets.add(w);
    }

    protected void disown(Widget w) {
        widgets.remove(w);
        super.disown(w);
    }
    
    public void refreshResources() {
        clearResourceEntries();
        
        Iterator itor = resources.iterator();
        while (itor.hasNext()) {
            ResourceEntry entry = (ResourceEntry) itor.next();
            entry.addToTable();
        }
    }

}
