/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.server;

import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;

import org.apache.log4j.Logger;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;
import com.trifork.sosigw.Factory;
import com.trifork.sosigw.api.IDCardCache;
import com.trifork.sosigw.api.IDCardCacheEntry;
import com.trifork.sosigw.api.IDCardState;
import com.trifork.sosigw.auditlog.AuditLog;
import com.trifork.sosigw.cluster.IDCardClusterCacheEntry;
import com.trifork.sosigw.config.RuntimeConfigStorageType;
import com.trifork.sosigw.config.StaticConfigImpl;
import com.trifork.sosigw.config.RuntimeConfig;
import com.trifork.sosigw.config.RuntimeConfigEditState;
import com.trifork.sosigw.config.RuntimeConfigState;
import com.trifork.sosigw.console.datamodel.AuditLogEntry;
import com.trifork.sosigw.console.datamodel.AuditLogQuery;
import com.trifork.sosigw.console.datamodel.ConfigCategory;
import com.trifork.sosigw.console.datamodel.ConfigItem;
import com.trifork.sosigw.console.datamodel.IDCard;
import com.trifork.sosigw.console.datamodel.IDCardQuery;
import com.trifork.sosigw.console.datamodel.SearchException;
import com.trifork.sosigw.console.datamodel.SearchResult;
import com.trifork.sosigw.console.service.ConsoleFacade;

import dk.sosi.seal.model.UserInfo;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
public class ConsoleFacadeImpl extends RemoteServiceServlet implements ConsoleFacade {
    private static final Logger log = Logger.getLogger(ConsoleFacadeImpl.class);

    public void init(ServletConfig config) throws ServletException {
        super.init(config);

        StaticConfigImpl.initSettings();
    }
    
    public Boolean runtimeConfigIsInPropertiesMode() {
        return new Boolean(StaticConfigImpl.getInstance().getRuntimeconfigStorage() == RuntimeConfigStorageType.PROPERTIES);
    }

    public ConfigCategory getConfigCategory(String categoryName) {
        RuntimeConfigState runtimeConfigState = RuntimeConfig.getRuntimeConfigState();

        ConfigCategory category = new ConfigCategory(categoryName);

        String[] config = runtimeConfigState.getValue(RuntimeConfig.CATEGORY_METAINFO, categoryName);
        categoryFromMetaConfig(config, category);
        
        Map<String, String[]> categoryMap = runtimeConfigState.getCategoryMap(categoryName);

        for (Map.Entry entry: categoryMap.entrySet()) {

            String key = (String)entry.getKey();
            String[] values = (String[])entry.getValue();

            ConfigItem item = new ConfigItem(category, key, values);
            category.addItem(item);
        }

        return category;
    }

    public String[] getConfigCategoryNames() {
        RuntimeConfigState runtimeConfigState = RuntimeConfig.getRuntimeConfigState();

        Map<String, String[]> categoryMap = runtimeConfigState.getCategoryMap(RuntimeConfig.CATEGORY_METAINFO);
        Set<String> keySet = categoryMap.keySet();


        String[] result = keySet.toArray(new String[0]);
        
        return result;
    }

    public void saveConfigCategory(ConfigCategory category) {
        RuntimeConfigState runtimeConfigState = RuntimeConfig.getRuntimeConfigState();

        RuntimeConfigEditState editableVersion = runtimeConfigState.getEditableVersion();

        String[] config = categoryToMetaConfig(category);
        
        editableVersion.addEntry(RuntimeConfig.CATEGORY_METAINFO, category.getName(), config);
        
        for (ConfigItem item: category.getConfigItems()) {
            editableVersion.addEntry(category.getName(), item.getKey(), item.getValues());
        }
        
        RuntimeConfig.setRuntimeConfigState(editableVersion.getResult());
        saveConfig();
    }

    public void saveConfigItem(ConfigItem item) {
        RuntimeConfigState runtimeConfigState = RuntimeConfig.getRuntimeConfigState();

        RuntimeConfigEditState editableVersion = runtimeConfigState.getEditableVersion();

        ConfigCategory category = item.getCategory();
        
        String[] config = categoryToMetaConfig(category);
        
        editableVersion.addEntry(RuntimeConfig.CATEGORY_METAINFO, category.getName(), config);
        
        editableVersion.addEntry(category.getName(), item.getKey(), item.getValues());
        
        RuntimeConfig.setRuntimeConfigState(editableVersion.getResult());
        saveConfig();
    }

    private String[] categoryToMetaConfig(ConfigCategory category) {
        String[] columns = category.getColumns();
        
        String[] config = new String[columns.length + 1];
        System.arraycopy(columns, 0, config, 0, columns.length);
        config[columns.length] = category.getDescription();

        return config;
    }
    
    private void categoryFromMetaConfig(String[] config, ConfigCategory category) {
        String[] columns = new String[config.length - 1];
        System.arraycopy(config, 0, columns, 0, config.length - 1);
        
        String description = config[config.length - 1];
        
        category.setColumns(columns);
        category.setDescription(description);
    }

    public void removeConfigCategory(String categoryName) {
        RuntimeConfigState runtimeConfigState = RuntimeConfig.getRuntimeConfigState();

        RuntimeConfigEditState editableVersion = runtimeConfigState.getEditableVersion();

        editableVersion.removeCategoryMap(categoryName);

        RuntimeConfig.setRuntimeConfigState(editableVersion.getResult());
        saveConfig();
    }

    public void removeConfigItem(String categoryName, String itemName) {
        RuntimeConfigState runtimeConfigState = RuntimeConfig.getRuntimeConfigState();

        RuntimeConfigEditState editableVersion = runtimeConfigState.getEditableVersion();

        Map<String, String[]> categoryMap = editableVersion.getCategoryMap(categoryName);
        categoryMap.remove(itemName);
        
        RuntimeConfig.setRuntimeConfigState(editableVersion.getResult());
        saveConfig();
    }
    
    /**
     * Save config to database and distribute to other members of cluster. 
     */
    private void saveConfig() {
        RuntimeConfig.saveCurrentState();
        Factory.getIntercom().configUpdatedLocally();
    }

    public SearchResult searchIDCard(IDCardQuery query) throws SearchException {
        final IDCardCache idCardCache = Factory.getIDCardCache();

        final IDCardCacheEntry cachedIDCardEntry;
        
        if (query.getCpr() != null && query.getCpr().length() > 0) {
            cachedIDCardEntry = idCardCache.findIDCardOrNull(query.getCpr());
            
        } else {
            cachedIDCardEntry = null;
//            UserInfo userInfo = new UserInfo(query.getCpr(), query.getGivenName(), query.getSurName(), query.getEmail(),
//                    query.getOccupation(), query.getRole(), null);
//TODO: Extension point: If we ever need this, implement it... 
//            cachedIDCardEntry = idCardCache.findIDCardByUserInfo(userInfo);
            
        }

        SearchResult result = new SearchResult();

        if (cachedIDCardEntry != null) { 

            if (cachedIDCardEntry.getIdCard() != null) {
                UserInfo cachedUserInfo = cachedIDCardEntry.getIdCard().getUserInfo();

                IDCard idCard = new IDCard(cachedIDCardEntry.getCacheKey(), cachedUserInfo.getCPR(), cachedUserInfo.getGivenName(), cachedUserInfo
                        .getSurName(), cachedUserInfo.getEmail(), cachedUserInfo.getOccupation(), cachedUserInfo
                        .getRole(), cachedUserInfo.getAuthorizationCode());

                idCard.setState(cachedIDCardEntry.getState().toString());
                
                result.setIDCards(new IDCard[] { idCard });
            }
        }
        
        if (result.getIDCards() == null) {
            result.setIDCards(new IDCard[0]);
        }
        return result;
    }

    public IDCard getIDCard(String name) {
        return null;
    }

    public void revokeIdCard(String cacheKey) {
        final IDCardCache idCardCache = Factory.getIDCardCache();

        final IDCardClusterCacheEntry cachedIDCardEntry = idCardCache.findIDCardOrNull(cacheKey);
            
        if (cachedIDCardEntry != null) { 
            idCardCache.updateEntryState(cachedIDCardEntry, IDCardState.REVOKED);

            idCardCache.entryUpdatedLocally(cachedIDCardEntry);
            log.debug("Entry revoked: " + cachedIDCardEntry.toString());
            AuditLog.logIdcardChange(cachedIDCardEntry, getThreadLocalRequest());
        }
    }

    public AuditLogEntry[] searchAuditLog(AuditLogQuery query, int offset, int count) {

        List<AuditLogEntry> entries = AuditLog.search(query);
        AuditLogEntry[] result = new AuditLogEntry[entries.size()];
        
        entries.toArray(result);
        
        return result;
    }
}
