/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.proxy;

import com.trifork.sosigw.cluster.ManageIDCardClusterCache;
import com.trifork.sosigw.config.StaticConfigImpl;
import com.trifork.sosigw.util.Util;
import com.trifork.sosigw.util.timing.Timer;
import dk.sdsd.nsp.slalog.api.SLALogException;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import org.apache.log4j.Logger;

import javax.management.InstanceAlreadyExistsException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.NotCompliantMBeanException;
import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.lang.management.ManagementFactory;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
public class ProxyRequestServlet extends HttpServlet {
    public static final Logger log = Logger.getLogger(ProxyRequestServlet.class); 


    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        doPost(req, resp);
    }

    @Override
    public void init() throws ServletException {
        super.init();
        try {
            MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
            ObjectName mbeanName = new ObjectName("com.trifork.sosigw:type=ManageIDCardClusterCache");
            if (mbs.isRegistered(mbeanName)) {
                // Unregister existing bean
                try {
                    mbs.unregisterMBean(mbeanName);
                } catch (InstanceNotFoundException e) {
                }
            }
            ManageIDCardClusterCache mbean = new ManageIDCardClusterCache();
            mbs.registerMBean(mbean, mbeanName);
        } catch (MalformedObjectNameException e) {
            throw new ServletException("Error during MBean registration", e);
        } catch (InstanceAlreadyExistsException e) {
            throw new ServletException("Error during MBean registration", e);
        } catch (MBeanRegistrationException e) {
            throw new ServletException("Error during MBean registration", e);
        } catch (NotCompliantMBeanException e) {
            throw new ServletException("Error during MBean registration", e);
        }
    }

    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        String timingOperationName = "handleRequest";
        try {
            Timer.start(ProxyRequestServlet.class, timingOperationName);
            handleRequest(req, resp);
        } finally {
            Timer.stop(ProxyRequestServlet.class, timingOperationName);
        }
    }

    private void handleRequest(HttpServletRequest req, HttpServletResponse resp) {
        try {
            Util.setActiveHttpRequest(req);
            
            SLALogItem slaLogItem = Util.getSLALogger().createLogItem("SOSIGWProxyRequestServlet.handleRequest()", "1");
            slaLogItem.setClientIP(req);
            Util.setSLALogItem(slaLogItem);
            log.info("SLA Log Item: "+slaLogItem);
            
            if (StaticConfigImpl.getInstance().isBootstrapRestrictedMode()) {
                throw new IllegalStateException(
                        "SOSIGW running in BootstrapRestrictedMode. No webservice access allowed");
            }

            new ProxyRequestHandler(req, resp).proxyRequest();
        } finally {
            Util.setActiveHttpRequest(null);
            try {
                SLALogItem slaLogItem = Util.getSLALogItem();
                log.info("SLA Log Item: "+slaLogItem);
                slaLogItem.store();
            } catch (SLALogException e) {
                log.error("Cannot store SLA Log Item",e);
            }
            Util.setSLALogItem(null);
        }
    }

}
