/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.util;

import java.util.Properties;

import com.trifork.sosigw.api.Constants;
import com.trifork.sosigw.config.StaticConfigApi;
import com.trifork.sosigw.config.StaticConfigImpl;
import com.trifork.sosigw.config.RuntimeConfig;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.AuthenticationLevel;
import dk.sosi.seal.model.UserIDCard;
import dk.sosi.seal.pki.Federation;
import dk.sosi.seal.pki.SOSIFederation;
import dk.sosi.seal.pki.SOSITestFederation;
import dk.sosi.seal.vault.CredentialVault;
import dk.sosi.seal.vault.EmptyCredentialVault;

/**
 * 
 * 
 * @author Jeppe Sommer <jso at trifork.com>
 * @author Morten Grouleff <mgr at trifork.com>
 */
public class SosiSeal {

    private static final SOSIFactory factory = makeSosiSealFactory();
    private static final String sosigwITSystemName = calcSosigwITSystemName();
    private static final StaticConfigApi STATIC_CONF = StaticConfigImpl.getInstance();

    private static final SOSIFactory makeSosiSealFactory() {
        CredentialVault emptyCredentialVault = new EmptyCredentialVault();
        Properties props = new Properties();
        String issuer = RuntimeConfig.getConsoleCVRName();
        props.setProperty(SOSIFactory.PROPERTYNAME_SOSI_ISSUER, issuer);
        boolean isProduction = StaticConfigImpl.getInstance().isProductionMode();
        Federation federation;
        if (isProduction) {
            federation = new SOSIFederation(props);

        } else {
            // All modes but production use the test-sts.
            federation = new SOSITestFederation(props);
        }
        SOSIFactory sosiFactory = new SOSIFactory(federation, emptyCredentialVault, props);
        return sosiFactory;
    }

    private static String calcSosigwITSystemName() {
        boolean isProduction = StaticConfigImpl.getInstance().isProductionMode();
        if (isProduction) {
            return Constants.SOSIGW_IT_SYSTEM_PRODUCTION_NAME;
        } else {
            return Constants.SOSIGW_IT_SYSTEM_TESTNAME;
        }
    }

    public static SOSIFactory getSosiSealFactory() {
        return factory;
    }

    public static String getSosigwITSystemName() {
        return sosigwITSystemName;
    }

    public static UserIDCard MakeNewIdenticalLevel4Card(UserIDCard card) {
        return new UserIDCard(STATIC_CONF.getSupportedDGWSVersion(), AuthenticationLevel.MOCES_TRUSTED_USER, card
                .getIssuer(), card.getSystemInfo(), card.getUserInfo(), null, null, null, null);
    }
}
