/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.util;

import org.apache.log4j.LogManager;
import org.apache.log4j.helpers.FileWatchdog;
import org.apache.log4j.xml.DOMConfigurator;

/**
 * The default FileWatchdog in log4j leaks memory when redeploying, as it never quits, thus keeping the classloader alive.
 * This one should fix that and otherwise behave identically.
 * 
 * @author Morten Grouleff <mgr@trifork.com>
 */
public class XMLWatchdog extends FileWatchdog {
    private boolean killed;
    public XMLWatchdog(String filename, long delay) {
        super(filename);
        setName("log4J watcher for " + filename);
        setDelay(delay);
    }

    public void doOnChange() {
        (new DOMConfigurator()).doConfigure(super.filename, LogManager.getLoggerRepository());
    }
    
    public void run() {
        while (!killed) {
            try {
                Thread.currentThread();
                Thread.sleep(delay);
                checkAndConfigure();
            } catch(InterruptedException interruptedexception) { }
        }
    }
    public void kill() {
        killed = true;
        interrupt();
    }
}
