/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.util.timing;

import org.apache.log4j.Logger;

/**
 * Records timing events using log4j. 
 */
public class Log4jTimeSink implements TimeSink {

    private Logger log; 

	
	/**
	 * Creates an instance recording timing events to TIMING log.
	 */
	public Log4jTimeSink() {
		this(Logger.getLogger(Log4jTimeSink.class));
	}

	/**
	 * Creates an instance recording timing events to the given log.
	 * @param log
	 */
	public Log4jTimeSink(Logger log) {
		this.log = log;
	}

	public void enter(TimerEvent e) {
		if (log.isDebugEnabled()) {
			log.debug("ENTER " + elementInfo(e));
		}
	}

	public void exit(TimerEvent e, long duration) {
		if (log.isInfoEnabled()) {
			log.info("EXIT " + elementInfo(e) + " duration " + duration + " childtime " + e.childtime);
		}
	}

	public void discard(TimerEvent e, String reason) {
		log.warn("DISCARD " + elementInfo(e) + " reason " + reason);
	}
	
    private String elementInfo(TimerEvent e) {
		return "(" + e.clazz.getSimpleName() + ", " + e.operation + ")";
	}
}
