/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.ws;

import com.trifork.sosigw.api.Constants;
import com.trifork.sosigw.util.Util;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sosi.seal.model.constants.MedComTags;
import org.w3c.dom.Element;

import javax.servlet.http.HttpServletRequest;
import javax.xml.soap.SOAPHeader;

/**
 * This class contains utility methods for making it simpler to slalog values from the SOAPheader.
 */
public class SlaLogItemUtil {

    /**
     * This method will add available information about the request and in cases where a SOAPHeader is present information about the MessageId and SoapAction will also be added to the slaLogItem. 
     * @param slaLogItem The item to append extra information about the request to.
     */
    public static void setSLALogValues(SLALogItem slaLogItem) {
        HttpServletRequest httpServletRequest = Util.getHttpServletRequest();
        SOAPHeader soapHeader = DGWSHandler.getSOAPHeader();
        if (httpServletRequest != null) {
            slaLogItem.setRequestSize(httpServletRequest.getContentLength());
            slaLogItem.setClientIP(httpServletRequest);
        }
        if (soapHeader != null) {
            Element messageIdNode = DGWSHandler.getElm(soapHeader, Constants.URL_MEDCOM, MedComTags.MESSAGE_ID);
            if (messageIdNode != null) {
                slaLogItem.setMessageId(messageIdNode.getTextContent());
            }
            Element soapActionNode = DGWSHandler.getElm(soapHeader, "http://www.w3.org/2005/08/addressing","Action");
            if (soapActionNode != null) {
                slaLogItem.setSourceSOAPAction(soapActionNode.getTextContent());
            }
        }
    }

}
