/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.ws;

import com.trifork.sosigw.api.Constants;
import com.trifork.sosigw.config.StaticConfigImpl;
import com.trifork.sosigw.util.Util;
import com.trifork.sosigw.util.timing.Timer;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sosi.gw._2007_09.DGWSFault;
import dk.sosi.gw._2007_09.RequestIdCardDigestForSigningResponse;
import dk.sosi.gw._2007_09.SignIdCardRequestBody;
import dk.sosi.gw._2007_09.SosiGWFacade;
import oasis.names.tc.saml._2_0.assertion.AssertionType;
import org.apache.log4j.Logger;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;

import javax.jws.HandlerChain;
import javax.jws.WebService;

/**
 * This class is handwritten to conform to the code generated by
 * the JAX-WS RI.
 * 
 */
@WebService(endpointInterface = "dk.sosi.gw._2007_09.SosiGWFacade")
@HandlerChain(file = "SosiGWFacade_handler.xml")
public class SosiGWFacadeImpl implements SosiGWFacade {
    private static final Logger log = Logger.getLogger(SosiGWFacadeImpl.class); 

    public AssertionType getValidIdCard(Security wsseHeader, String sosigwGetValidIdCardRequestBody) throws DGWSFault {
        if (StaticConfigImpl.getInstance().isBootstrapRestrictedMode()) {
            throw new DGWSFault("SOSIGW running in BootstrapRestrictedMode. No webservice access allowed", Constants.FAULTCODE_INTERNAL_ERROR);
        }
        String timingOperationName = "getValidIdCard";
        Timer.start(SosiGWFacadeImpl.class, timingOperationName);
        
        SLALogItem slaLogItem = Util.getSLALogger().createLogItem("SOSIGWFacade."+timingOperationName, "10");
        Util.setSLALogItem(slaLogItem);
        SlaLogItemUtil.setSLALogValues(slaLogItem);
        try {
            AssertionType result = SosiGWFacadeBusiness.getValidIdCardImpl(wsseHeader);
            slaLogItem.setCallResultOk();
            return result;
        } catch (DGWSFault e) {
            slaLogItem.setCallResultError("DGWS error: "+e.getMessage());
            throw e;
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            slaLogItem.setCallResultError("Internal error: "+e.getMessage());
            throw new DGWSFault(e.getMessage(), Constants.FAULTCODE_INTERNAL_ERROR, e);
        } finally {
            try {
                slaLogItem.store();
            } catch (Exception e) {
                log.error("Caught exception while storing SLA Log item " + slaLogItem, e);
            }
            Timer.stop(SosiGWFacadeImpl.class, timingOperationName);
        }
    }

    public void logout(Security wsseHeader, String sosigwLogoutRequestBody) {
        restrictIllegalState();
        String timingOperationName = "logout";
        Timer.start(SosiGWFacadeImpl.class, timingOperationName);
        SLALogItem slaLogItem = Util.getSLALogger().createLogItem("SOSIGWFacade."+timingOperationName, "11");
        Util.setSLALogItem(slaLogItem);
        SlaLogItemUtil.setSLALogValues(slaLogItem);
        try {
            SosiGWFacadeBusiness.logoutImpl(wsseHeader);
            slaLogItem.setCallResultOk();
        } catch (Exception e) {
            // For logout, we just ignore it.
            log.warn("Failed to logout; ignoring.", e);
            slaLogItem.setCallResultError("Internal error: "+e.getMessage());
        } finally {
            storeSlaLogItem(slaLogItem);
            Timer.stop(SosiGWFacadeImpl.class, timingOperationName);
        }
    }

    public String logoutWithResponse(Security wsseHeader, String logoutRequestBody) throws DGWSFault {
        restrictIllegalState();
        String timingOperationName = "logoutWithResponse";
        Timer.start(SosiGWFacadeImpl.class, timingOperationName);
        SLALogItem slaLogItem = Util.getSLALogger().createLogItem("SOSIGWFacade."+timingOperationName, "14");
        Util.setSLALogItem(slaLogItem);
        SlaLogItemUtil.setSLALogValues(slaLogItem);
        try {
            SosiGWFacadeBusiness.logoutImpl(wsseHeader);
            slaLogItem.setCallResultOk();
            return "ok";
        } catch (DGWSFault e) {
            slaLogItem.setCallResultError("DGWS error: "+e.getMessage());
            throw e;
        } catch (Exception e) {
            // For logout, we just ignore it.
            log.warn("Failed to logout", e);
            slaLogItem.setCallResultError("Internal error: "+e.getMessage());
            throw new DGWSFault(e.getMessage(), Constants.FAULTCODE_INTERNAL_ERROR, e);
        } finally {
            storeSlaLogItem(slaLogItem);
            Timer.stop(SosiGWFacadeImpl.class, timingOperationName);
        }
    }

    public RequestIdCardDigestForSigningResponse requestIdCardDigestForSigning(Security wsseHeader, String sosigwRequestIdCardDigestForSigningRequestBody) throws DGWSFault {
        if (StaticConfigImpl.getInstance().isBootstrapRestrictedMode()) {
            throw new DGWSFault("SOSIGW running in BootstrapRestrictedMode. No webservice access allowed", Constants.FAULTCODE_INTERNAL_ERROR);
        }
        String timingOperationName = "requestIdCardDigestForSigning";
        Timer.start(SosiGWFacadeImpl.class, timingOperationName);
        SLALogItem slaLogItem = Util.getSLALogger().createLogItem("SOSIGWFacade."+timingOperationName, "12");
        Util.setSLALogItem(slaLogItem);
        SlaLogItemUtil.setSLALogValues(slaLogItem);
        try {
            RequestIdCardDigestForSigningResponse result = SosiGWFacadeBusiness.requestIdCardDigestForSigningImpl(wsseHeader);
            slaLogItem.setCallResultOk();
            return result;
        } catch (DGWSFault e) {
            slaLogItem.setCallResultError("DGWS error: "+e.getMessage());
            throw e;
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            slaLogItem.setCallResultError("Internal error: "+e.getMessage());
            throw new DGWSFault(e.getMessage(), Constants.FAULTCODE_INTERNAL_ERROR, e);
        } finally {
            storeSlaLogItem(slaLogItem);
            Timer.stop(SosiGWFacadeImpl.class, timingOperationName);
        }
    }

    public String signIdCard(Security wsseHeader, SignIdCardRequestBody sosigwSignIdCardRequestBody) throws DGWSFault {
        if (StaticConfigImpl.getInstance().isBootstrapRestrictedMode()) {
            throw new DGWSFault("SOSIGW running in BootstrapRestrictedMode. No webservice access allowed", Constants.FAULTCODE_INTERNAL_ERROR);
        }
        String timingOperationName = "signIdCard";
        Timer.start(SosiGWFacadeImpl.class, timingOperationName);
        SLALogItem slaLogItem = Util.getSLALogger().createLogItem("SOSIGWFacade."+timingOperationName, "13");
        Util.setSLALogItem(slaLogItem);
        SlaLogItemUtil.setSLALogValues(slaLogItem);
        try {
            String res = SosiGWFacadeBusiness.signIdCardImpl(wsseHeader, sosigwSignIdCardRequestBody);
            slaLogItem.setCallResultOk();
            return res;
        } catch (DGWSFault e) {
            slaLogItem.setCallResultError("DGWS error: "+e.getMessage());
            throw e;
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            slaLogItem.setCallResultError("Internal error: "+e.getMessage());
            throw new DGWSFault(e.getMessage(), Constants.FAULTCODE_INTERNAL_ERROR, e);
        } finally {
            storeSlaLogItem(slaLogItem);
            Timer.stop(SosiGWFacadeImpl.class, timingOperationName);
        }
    }


    private void restrictIllegalState() {
        if (StaticConfigImpl.getInstance().isBootstrapRestrictedMode()) {
            throw new IllegalStateException("SOSIGW running in BootstrapRestrictedMode. No webservice access allowed");
        }
    }

    private void storeSlaLogItem(SLALogItem slaLogItem) {
        try {
            slaLogItem.store();
        } catch (Exception e) {
            log.error("Caught exception while storing SLA Log item",e);
        }
    }
}
