/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package dk.sdsd.sosigw.utils;

import com.trifork.sosigw.BootStrapRejectorFilter;
import com.trifork.sosigw.config.RuntimeConfig;
import com.trifork.sosigw.config.StaticConfigApi;
import com.trifork.sosigw.config.StaticConfigImpl;
import dk.sdsd.nsp.monitor.HttpGetMonitoredObject;
import dk.sdsd.nsp.monitor.Monitor;
import dk.sdsd.nsp.monitor.MonitoredResultsHolder;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Date;

/**
 * 
 * Crude helper class for producing status output to clients monitoring the service
 *
 */
public class IsAliveUtils {
    public static String getCheckGwText(HttpServletRequest request, HttpServletResponse response) {
        if (serviceIsDown()) {
            return serviceDownText(response);
        }

        StringBuilder sb = new StringBuilder("NSP GW check\n");
        String hostName = "";
        try {
            hostName = InetAddress.getLocalHost().getHostName();
        } catch (UnknownHostException ex) {
            hostName = "unknown";
        }
        sb.append("GW running on: ").append(hostName).append("\n");
        sb.append(new Date());
        return sb.toString();
    }

    public static String getCheckAllText(HttpServletRequest request, HttpServletResponse response) {
        if (serviceIsDown()) {
            return serviceDownText(response);
        }

        StringBuilder header = new StringBuilder("NSP-GW=");
        StringBuilder body = new StringBuilder();

        Monitor m = new Monitor();
        m.addMonitoredObject(new HttpGetMonitoredObject("FMK", StaticConfigImpl.getInstance().getFMKIsAliveURL()));
        m.addMonitoredObject(new HttpGetMonitoredObject("STS", RuntimeConfig.getSecurityTokenServiceUrlList()));

        MonitoredResultsHolder monHolder = m.checkMonitoredObjects();

        body.append(Monitor.SEPARATOR);
        body.append("GW-settings:\n");
        body.append(getSOSIGWSetup(request));
        body.append(getSOSIGWStaticSetup());
        
        header.append(monHolder.getResultsHeader()).append("\n").append(body).append(monHolder.getResultsBody());
        return header.toString();
    } 

    public static String getSOSIGWSetup(HttpServletRequest request) {
        StringBuilder sb = new StringBuilder("Request servername: ").append(request.getServerName()).append("\n");
        
        String hostName = "";
        try {
            hostName = InetAddress.getLocalHost().getHostName();
        } catch (UnknownHostException ex) {
            hostName = "unknown";
        }
        sb.append("Hostname=").append(hostName);
        sb.append("\n");
        sb.append("STS=").append(RuntimeConfig.getSecurityTokenServiceUrlList());
        sb.append("\n");
        sb.append("Partitioning=").append(RuntimeConfig.isCachePartitioningEnabled());
        sb.append("\n");
        if (RuntimeConfig.isCachePartitioningEnabled()) {
            sb.append("Partition Header=").append(RuntimeConfig.getCachePartitioningHttpHeaderName());
            sb.append("\n");
            sb.append("DCC=").append(RuntimeConfig.getDecouplingUrl());
            sb.append("\n");
        }
        sb.append("PassThroughOnSignature=").append(RuntimeConfig.isPassThroughOnSignatureEnabled());
        sb.append("\n");
        sb.append("GlobalDB=").append(RuntimeConfig.getGlobalDatabaseJDBCUrl());
        sb.append("\n");
        sb.append("GlobalDBInterval=").append(RuntimeConfig.getGlobalDatabaseJDBCInterval());
        sb.append("\n");
        sb.append("ConsoleCVR=").append(RuntimeConfig.getConsoleCVRNumber());
        sb.append("\n");
        sb.append("ConsoleCVRName=").append(RuntimeConfig.getConsoleCVRName());
        sb.append("\n");
        sb.append("IdCardSigningTimeout=").append(RuntimeConfig.getIdCardSigningTimeout());
        sb.append("\n");
        sb.append("BrowserTimingTimeout=").append(RuntimeConfig.getBrowserSigningTimeout());
        sb.append("\n");
        return sb.toString();
    }
      
    public static String getSOSIGWStaticSetup() {
        StringBuilder sb = new StringBuilder();

        StaticConfigApi conf = StaticConfigImpl.getInstance();
        sb.append("Mode=").append(conf.getMode());
        sb.append("\n");
        sb.append("SLA log=").append(conf.isSLALogEnabled());
        sb.append("\n");
        sb.append("DGWS version=").append(conf.getSupportedDGWSVersion());
        sb.append("\n");
        InetAddress inet = null;
        try {
            inet = conf.getMulticastIpAddress();
        } catch (UnknownHostException ex) {
            // ignore
        }
        sb.append("Multicast IP=").append((inet != null) ? inet.getHostAddress() : "null");
        sb.append("\n");
        sb.append("Multicastport=").append(conf.getMulticastPort());
        sb.append("\n");
        sb.append("ClusterIpPrefix=").append(conf.getClusterIpPrefix());
        sb.append("\n");
        sb.append("KeepAliveEnabled=").append(conf.isKeepAliveEnabled());
        sb.append("\n");
        sb.append("FMKIsAliveURL=").append(conf.getFMKIsAliveURL());
        sb.append("\n");
        return sb.toString();
     }

    private static boolean serviceIsDown() {
        return (StaticConfigImpl.getInstance().isBootstrapRestrictedMode() || BootStrapRejectorFilter.requestDisallowed());
    }

    private static String serviceDownText(HttpServletResponse response) {
        response.setStatus(500);
        return "SOSI-GW is not accepting proxy requests";
    }
}
