/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw;

import com.trifork.sosigw.api.Intercom;
import org.junit.Before;
import org.junit.Test;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.io.StringWriter;

import static org.mockito.Mockito.*;

public class BootStrapRejectorFilterTest {
    
    private BootStrapRejectorFilter filter;
    private HttpServletRequest req;
    private HttpServletResponse res;
    private FilterChain fc;
    
    @Before
    public void setup() {
        filter = new BootStrapRejectorFilter();
        
        req = mock(HttpServletRequest.class);
        res = mock(HttpServletResponse.class);
        fc = mock(FilterChain.class);
        
        BootStrapRejectorFilter.start();
    }

    @Test
    public void testFilter() throws Exception {
        Intercom ic = mock(Intercom.class);
        when(ic.isBootstrapped()).thenReturn(true);
        
        Factory.setIntercom(ic);
        filter.doFilter(req, res, fc);
        
        verify(fc).doFilter(req, res);
    }
    
    @Test
    public void blockWhenNotBootstrapped() throws Exception {
        Factory.setIntercom(null);
        when(res.getWriter()).thenReturn(new PrintWriter(new StringWriter()));
        filter.doFilter(req, res, fc);
        

        verifyZeroInteractions(fc);
        verify(res).setStatus(404);
    }

}
