/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.audit;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.util.List;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.trifork.sosigw.Factory;
import com.trifork.sosigw.api.Intercom;
import com.trifork.sosigw.auditlog.AuditLogGlobalPostgresql;
import com.trifork.sosigw.auditlog.AuditLogItem;
import com.trifork.sosigw.auditlog.AuditLogPostgresql;
import com.trifork.sosigw.auditlog.SqlTests;
import com.trifork.sosigw.config.RuntimeConfig;
import com.trifork.sosigw.config.RuntimeConfigItem;
import com.trifork.sosigw.config.RuntimeConfigState;
import com.trifork.sosigw.console.datamodel.AuditLogEntry;
import com.trifork.sosigw.console.datamodel.AuditLogQuery;


public class AuditLogSearchTest extends SqlTests {

    @Before
    public void setup() {
        RuntimeConfig.setRuntimeConfigState(new RuntimeConfigState(1L, new RuntimeConfigItem[] {
                new RuntimeConfigItem(RuntimeConfig.CATEGORY_PROPERTIES, RuntimeConfig.KEY_GLOBALJDBCINTERVAL, new String[] { "100" }),
                new RuntimeConfigItem(RuntimeConfig.CATEGORY_PROPERTIES, RuntimeConfig.KEY_GLOBALJDBCURL, new String[] { "jdbc:hsqldb:mem:test" }),
                new RuntimeConfigItem(RuntimeConfig.CATEGORY_PROPERTIES, RuntimeConfig.KEY_GLOBALJDBCUSERNAME, new String[] { "sa" }),
                new RuntimeConfigItem(RuntimeConfig.CATEGORY_PROPERTIES, RuntimeConfig.KEY_GLOBALJDBCPASSWORD, new String[] { "" })
        }));

        Intercom ic = mock(Intercom.class);
        Factory.setIntercom(ic);
        
        when(ic.getUID()).thenReturn("uid");
    }
    
    @After
    public void shutdown() {
        AuditLogGlobalPostgresql.stop();
    }
    
    @Test
    public void testSearch() throws Exception {
        AuditLogGlobalPostgresql globalPostgresql = new AuditLogGlobalPostgresql();
        List<AuditLogEntry> list = globalPostgresql.search(new AuditLogQuery());
        int count = list.size();
        
        AuditLogGlobalPostgresql.start();

        AuditLogPostgresql al = new AuditLogPostgresql();
        al.log(new AuditLogItem("test", "1111111118", System.currentTimeMillis(), "http://localhost", "urn:action", "127.0.0.1"));
        al.log(new AuditLogItem("test", "3333333333", System.currentTimeMillis(), "http://localhost", "urn:action", "127.0.0.1"));

        Thread.sleep(200);
        
        list = globalPostgresql.search(new AuditLogQuery());
        assertEquals(count + 2, list.size());
        
        AuditLogQuery q = new AuditLogQuery();
        q.setCpr("1111111118");
        list = globalPostgresql.search(q);
        assertEquals(count + 1, list.size());
        assertEquals("1111111118", list.get(0).getCpr());
    }
}
