/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.auditlog;

import java.io.IOException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.hsqldb.jdbc.jdbcDataSource;
import org.junit.BeforeClass;

import com.trifork.sosigw.config.StaticConfigImpl;
import com.trifork.sosigw.config.StaticConfigTestImpl;

public abstract class SqlTests {
    private static long seq = 0;
    private static DataSource ds;

    @BeforeClass
    public static void initSqlTests() {
        try {
            StaticConfigImpl.setInstance(new StaticConfigTestImpl());

            initDB();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private static void initDB() throws ClassNotFoundException, NamingException, SQLException, IOException {
        if (ds == null) {
            Class.forName("org.hsqldb.jdbcDriver");

            // Create initial context
            System.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.apache.naming.java.javaURLContextFactory");
            System.setProperty(Context.URL_PKG_PREFIXES, "org.apache.naming");

            InitialContext ic = new InitialContext();

            try {
                ds = (DataSource)ic.lookup("java:comp/env/jdbc/postgres");
                if(ds == null || ! (ds instanceof jdbcDataSource)) {
                    ds = createMemDatabase();
                    ic.rebind("java:comp/env/jdbc/postgres", ds);
                }
            } catch (NamingException nex) {
                createSubcontext(ic, "java:");
                createSubcontext(ic, "java:comp");
                createSubcontext(ic, "java:comp/env");
                createSubcontext(ic, "java:comp/env/jdbc");
           
                ds = createMemDatabase();
                ic.bind("java:comp/env/jdbc/postgres", ds);
            }
            
            initData();
        }
        clearData();
    }
    
    private static void createSubcontext(InitialContext ic, String ctx) {
        try {
            ic.createSubcontext(ctx);
        } catch (NamingException e) {
            //Do nothing - continue
        }
    }
    
    public static DataSource createMemDatabase() {
        return createMemDatabase("test");
    }
    public static DataSource createMemDatabase(String db) {
        return createMemDatabase(db, "sa", "");
    }
    public static DataSource createMemDatabase(String db, String user, String pass) {
        // Construct DataSource
        jdbcDataSource jdbcDataSource = new jdbcDataSource();
        jdbcDataSource.setDatabase("jdbc:hsqldb:mem:"+db);
        jdbcDataSource.setUser(user);
        jdbcDataSource.setPassword(pass);
        return jdbcDataSource;
    }

    private static void clearData() throws SQLException, IOException {
        SqlTestsUtil.executeSql(ds, "truncate.sql");
    }
    
    private static void initData() throws SQLException, IOException {
        SqlTestsUtil.executeSql(ds, "create.sql");
    }

    protected int count(String table) throws SQLException {
        PreparedStatement ps = ds.getConnection().prepareStatement("SELECT count(*) FROM " + table);
        ResultSet rs = ps.executeQuery();
        rs.next();
        return rs.getInt(1);
    }

    public static long nextval(String s) {
        return ++seq;
    }

}
