/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.cluster.intercom;

import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyNoMoreInteractions;
import static org.mockito.Mockito.verifyZeroInteractions;

import org.junit.Test;
import org.mockito.ArgumentCaptor;

import com.trifork.sosigw.config.RuntimeConfig;

public class ProtocolReceiverAliveTest extends ProtocolReceiverTest {

    @Test
    public void testReceiveAliveFromSelf() throws Exception {
        pr.receive(getStream(Protocol.encodeVersion1AliveAsBytes(1, 1, "uid")));
        assertTrue(state.getLastSenderPing().isEmpty());
    }
    
    @Test
    public void testReceiveAliveFromNewOther() throws Exception {
        pr.receive(getStream(Protocol.encodeVersion1AliveAsBytes(1, 0, "other")));
        
        assertTrue(state.getLastSenderPing().containsKey("other"));
        assertTrue("sync request should be started", state.getSyncRequestedAt().containsKey("other"));
        
        verifyZeroInteractions(handler);
    }

    @Test
    public void receiveAliveWithNewerConfigShouldRequestNewConfig() throws Exception {
        pr.receive(getStream(Protocol.encodeVersion1AliveAsBytes(1, System.currentTimeMillis(), "other")));

        ArgumentCaptor<byte[]> arg = ArgumentCaptor.forClass(byte[].class);
        verify(handler).sendMessage(arg.capture());
        verifyNoMoreInteractions(handler);
        
        ProtocolVerifier pv = new ProtocolVerifier(arg.getValue());
        pv.protocolVersion1()
            .then("the operation", Protocol.PROTOCOL_V1_CONFIGREQUEST);
    }
    
    @Test
    public void receiveAliveWithOlderConfigShouldSendNew() throws Exception {
        pr.receive(getStream(Protocol.encodeVersion1AliveAsBytes(1, RuntimeConfig.getConfigTimestamp() - 10000L, "other")));
        
        ArgumentCaptor<byte[]> arg = ArgumentCaptor.forClass(byte[].class);
        verify(handler).sendMessage(arg.capture());
        verifyNoMoreInteractions(handler);
        
        ProtocolVerifier pv = new ProtocolVerifier(arg.getValue());
        pv.protocolVersion1()
            .then("the message type", Protocol.PROTOCOL_V1_CONFIGSTATE);
    }

}
