/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.cluster.intercom;

import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyZeroInteractions;

import org.junit.Test;
import org.mockito.ArgumentCaptor;

import com.trifork.sosigw.config.RuntimeConfig;

public class ProtocolReceiverConfigRequestTest extends ProtocolReceiverTest {

    @Test
    public void messageFromSelfIgnored() throws Exception {
        pr.receive(getStream(Protocol.encodeVersion1ConfigRequestAsBytes(1, "uid")));
        verifyZeroInteractions(handler);
    }

    @Test
    public void sameTimestampShouldBeIgnored() throws Exception {
        pr.receive(getStream(Protocol.encodeVersion1ConfigRequestAsBytes(RuntimeConfig.getConfigTimestamp(), "other")));
        verifyZeroInteractions(handler);
    }
    
    @Test
    public void olderTimestampShouldSendNew() throws Exception {
        pr.receive(getStream(Protocol.encodeVersion1ConfigRequestAsBytes(RuntimeConfig.getConfigTimestamp() - 100L, "other")));
        
        ArgumentCaptor<byte[]> arg = ArgumentCaptor.forClass(byte[].class);
        verify(handler).sendMessage(arg.capture());
        
        ProtocolVerifier pv = new ProtocolVerifier(arg.getValue());
        pv.protocolVersion1()
            .then("the message type", Protocol.PROTOCOL_V1_CONFIGSTATE);
    }
    
    @Test
    public void newerTimestampShoulRequestConfig() throws Exception {
        pr.receive(getStream(Protocol.encodeVersion1ConfigRequestAsBytes(RuntimeConfig.getConfigTimestamp() + 100L, "other")));
        
        ArgumentCaptor<byte[]> arg = ArgumentCaptor.forClass(byte[].class);
        verify(handler).sendMessage(arg.capture());
        
        ProtocolVerifier pv = new ProtocolVerifier(arg.getValue());
        pv.protocolVersion1()
            .then("the message type", Protocol.PROTOCOL_V1_CONFIGREQUEST);
    }
}
