/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.cluster.intercom;

import static org.junit.Assert.assertEquals;

import java.io.DataInputStream;
import java.net.ServerSocket;
import java.net.Socket;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class ProtocolReceiverSyncRequestTest extends ProtocolReceiverTest {
    
    private ServerSocket socket;

    @Before
    public void configure() throws Exception {
        socket = new ServerSocket(7765);
    }
    
    @After
    public void teardown() throws Exception {
        if (socket != null) {
            socket.close();
        }
    }
    

    @Test
    public void messageForOtherIgnored() throws Exception {
        // socket should not be used
        socket.close();
        pr.receive(getStream(Protocol.encodeVersion1SyncRequestAsBytes(new SyncRequest("other", 1), socket.getLocalSocketAddress())));
    }
    
    @Test
    public void receiveSyncRequest() throws Exception {
        pr.receive(getStream(Protocol.encodeVersion1SyncRequestAsBytes(new SyncRequest("uid", 1), socket.getLocalSocketAddress())));
        
        Socket s = socket.accept();
        DataInputStream is = new DataInputStream(s.getInputStream());
        
        assertEquals("First the protocol", Protocol.PROTOCOL_VERSION_1, is.readByte());
        assertEquals("Then the number of states", 1, is.readInt());
        assertEquals("Then the uid", "uid", is.readUTF());
        assertEquals("Then the seqno", window.getTop(), is.readLong());
        assertEquals("Then the number of cards", 0, is.readLong());
        assertEquals("Then no more entries", false, is.readBoolean());
        assertEquals("And no more data", 0, is.available());
        s.close();
    }
}
