/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.cluster.intercom;

import static org.mockito.Mockito.mock;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Properties;

import org.junit.Before;
import org.junit.Test;

import com.trifork.sosigw.api.IDCardClusterCache;
import com.trifork.sosigw.config.StaticConfigImpl;

public class ProtocolReceiverTest {
    
    protected SlidingWindow<String> window;
    protected ClusterState state;
    protected NetHandler handler;
    protected ProtocolReceiver pr;
    protected IDCardClusterCache cache;

    @Before
    public final void setup() {
        StaticConfigImpl.setInstance(new StaticConfigImpl(new Properties()));
        
        window = new SlidingWindow<String>(2);
        state = new ClusterState(5, 300, 5000, true);
        cache = mock(IDCardClusterCache.class);
        handler = mock(NetHandler.class);
        pr = new ProtocolReceiver("uid", state, window, cache, 100, 5000, true, new ConfigSender("uid", handler));
    }
    
    @Test
    public void testUnknownProtocolShouldBeIgnored() throws Exception {
        pr.receive(getStream(write(99)));
    }
    
    @Test
    public void unknownOperationShouldBeIgnored() throws Exception {
        pr.receive(getStream(write(Protocol.PROTOCOL_VERSION_1, 99)));
    }
    
    
    protected ObjectInputStream getStream(byte[] bytes) throws IOException {
        return new ObjectInputStream(new ByteArrayInputStream(bytes));
    }
    
    protected byte[] write(int ... bytes) throws IOException {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ObjectOutputStream oos = new ObjectOutputStream(bos);
        for (int b : bytes) {
            oos.writeByte(b);
            
        }
        oos.close();
        return bos.toByteArray();
    }
}
