/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.cluster.intercom;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.HashMap;
import java.util.Map;

public class ProtocolVerifier {
    private ObjectInputStream is;
    
    private Map<Class<?>, Object> objs = new HashMap<Class<?>, Object>();

    public ProtocolVerifier(byte[] bytes) throws IOException {
        is = new ObjectInputStream(new ByteArrayInputStream(bytes));
    }
    
    public ProtocolVerifier protocolVersion1() throws IOException {
        assertEquals("Protocol must be 1", Protocol.PROTOCOL_VERSION_1, is.readByte());
        return this;
    }
    
    public ProtocolVerifier then(String desc, String value) throws IOException {
        assertEquals(desc + " not correct", value, is.readUTF());
        return this;
    }
    
    public ProtocolVerifier then(String desc, byte value) throws IOException {
        assertEquals(desc + " not correct ", value, is.readByte());
        return this;
    }
    
    public ProtocolVerifier then(String desc, long value) throws IOException {
        assertEquals(desc + " (long) not correct", value, is.readLong());
        return this;
    }
    
    public ProtocolVerifier then(String desc, Class<?> type) throws IOException, ClassNotFoundException {
        Object obj = is.readObject();
        assertNotNull("Expected object of type " + type + " not received", obj);
        objs.put(type, obj);
        assertTrue("Expected object of type " + type + " for " + desc, type.isAssignableFrom(obj.getClass()));
        return this;
    }
    
    public ProtocolVerifier thenALong(String desc) throws IOException {
        is.readLong();
        return this;
    }
    
    public void done() throws IOException {
        assertEquals("No more data should be available", 0, is.available());
    }
    
    @SuppressWarnings("unchecked")
    public <T> T get(Class<T> type) {
        return (T)objs.get(type);
    }
}
