/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.cluster.intercom;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.util.Properties;

import org.junit.Before;
import org.junit.Test;

import com.trifork.sosigw.cluster.IDCardClusterCacheEntry;
import com.trifork.sosigw.cluster.IDCardClusterCacheImpl;
import com.trifork.sosigw.config.StaticConfigImpl;

import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.*;

public class SyncReceiverTest {
    
    private ClusterState state;
    private ServerSocket socket;
    private NetHandler handler;
    private SyncReceiver sr;
    
    private byte[] streamContent;
    private SlidingWindow<String> window;
    private IDCardClusterCacheImpl clusterCache;

    @Before
    public void setup() throws Exception {
        state = new ClusterState(5, 1000, 5000, true);
        
        socket = new ServerSocket() {

            public Socket accept() throws IOException {
                return new Socket() {

                    public InputStream getInputStream() throws IOException {
                        return new ByteArrayInputStream(streamContent);
                    }
                    

                    public synchronized void setSoTimeout(int timeout) throws SocketException {
                    }
                };
            }
        };
        
        handler = mock(NetHandler.class);
        
        window = new SlidingWindow<String>(2);
        clusterCache = new IDCardClusterCacheImpl(null);
        ProtocolReceiver receiver = new ProtocolReceiver("uid", state, window, clusterCache, 1000, 5000, true, new ConfigSender("uid", handler));
        sr = new SyncReceiver("uid", socket, state, new StaticConfigImpl(new Properties()), receiver);
    }
    
    @Test
    public void testReceive() throws Exception {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        DataOutputStream os = new DataOutputStream(bos);

        window.add(window.reserve(), "test");
        byte[] serverState = Protocol.encodeVersion1ServerStatesAsBytes("other", window, state.getLastSenderSeqno());
        os.write(serverState);
        
        os.writeLong(1); // number of cards
        IDCardClusterCacheEntry entry = new IDCardClusterCacheEntry("test");
        entry.setLastModified(System.currentTimeMillis());
        byte[] card = Protocol.encodeVersion1IDCardEntryAsBytes(entry, window.getTop(), "other");
        os.writeInt(card.length);
        os.write(card);
        
        os.writeBoolean(false); // end of stream marker
        os.close();
        
        streamContent = bos.toByteArray();
        sr.run0();
        
        
        assertTrue(clusterCache.getContentsAsList().contains("test"));
    }
    
    @Test
    public void ignoreUnknownProtocolVersion() throws Exception {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        DataOutputStream os = new DataOutputStream(bos);
        os.writeByte(99);
        os.close();
        
        streamContent = bos.toByteArray();
        sr.run0();
    }


}
