/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.cluster.intercom;

import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.SocketAddress;
import java.util.Properties;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;

import com.trifork.sosigw.config.StaticConfigImpl;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.mockito.Mockito.*;

public class SyncRequesterTest {

    private ClusterState state;
    private NetHandler netHandler;
    private SyncRequester sr;
    private ServerSocket socket;

    @Before
    public void setup() throws Exception {
        StaticConfigImpl.setInstance(new StaticConfigImpl(new Properties()));
        state = new ClusterState(5, 300, 5000, true);
        socket = new ServerSocket();
        socket.bind(new InetSocketAddress(6554));
        
        netHandler = mock(NetHandler.class);
        sr = new SyncRequester(state, socket, netHandler);
    }
    
    @After
    public void teardown() throws Exception {
        if (socket != null) {
            socket.close();
        }
    }
    
    @Test
    public void testNoRequest() throws Exception {
        sr.run0();
        verifyZeroInteractions(netHandler);
    }
    
    @Test
    public void testSendRequest() throws Exception {
        state.setFirstSyncCompleted(true);
        state.getSyncRequestedAt().put("test", 1L);
        state.getLastSenderSeqno().put("test", 4L);
        
        sr.run0();
        
        assertFalse(state.getSyncRequestedAt().get("test") == 1L);
        
        ArgumentCaptor<byte[]> arg = ArgumentCaptor.forClass(byte[].class);
        verify(netHandler).sendMessage(arg.capture());
        verifyNoMoreInteractions(netHandler);
        
        ProtocolVerifier pv = new ProtocolVerifier(arg.getValue());
        pv.protocolVersion1()
            .then("the operation", Protocol.PROTOCOL_V1_SYNCREQUEST)
            .then("the request itself", SyncRequest.class)
            .then("the listener address", SocketAddress.class)
            .done();
        
        assertEquals(4L, pv.get(SyncRequest.class).lastKnownGood);
        assertEquals(socket.getLocalSocketAddress(), pv.get(SocketAddress.class));
    }
}
