/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.config;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import com.trifork.sosigw.auditlog.SqlTests;
import org.junit.Before;
import org.junit.Test;

import java.util.Properties;

public class RuntimeConfigMapperAbstractDbTest extends SqlTests {
    private RuntimeConfigMapperAbstractDb mapper;
    private StaticConfigApi config;

    public void setupConfig(Properties props) {
        config = StaticConfigImpl.getInstance();
        StaticConfigImpl.setInstance(new StaticConfigImpl(props));
    }

    public void restoreConfig() {
        StaticConfigImpl.setInstance(config);
    }
    
    @Before
    public void setup() {
        RuntimeConfig.setRuntimeConfigState(new RuntimeConfigState(1L, new RuntimeConfigItem[] {
                new RuntimeConfigItem(RuntimeConfig.CATEGORY_PROPERTIES, RuntimeConfig.KEY_GLOBALJDBCINTERVAL, new String[] { "100" }),
                new RuntimeConfigItem(RuntimeConfig.CATEGORY_PROPERTIES, RuntimeConfig.KEY_GLOBALJDBCURL, new String[] { "jdbc:hsqldb:mem:test" }),
                new RuntimeConfigItem(RuntimeConfig.CATEGORY_PROPERTIES, RuntimeConfig.KEY_GLOBALJDBCUSERNAME, new String[] { "sa" }),
                new RuntimeConfigItem(RuntimeConfig.CATEGORY_PROPERTIES, RuntimeConfig.KEY_GLOBALJDBCPASSWORD, new String[] { "" })
        }));

        mapper = RuntimeConfigMapperAbstractDb.getDatabaseInstance();
    }

    @Test
    public void testSaveAndLoad() throws Exception {
        RuntimeConfigStateTest.makeClientAccessTestData();
        RuntimeConfigState runtimeConfigState = RuntimeConfig.getRuntimeConfigState();
        mapper.saveConfig(runtimeConfigState);
        
        RuntimeConfigState latestConfig = mapper.loadLatestConfig();
        
        assertEquals(latestConfig.getTimestamp(), runtimeConfigState.getTimestamp());
    }

    @Test(expected = ConfigurationException.class)
    public void shouldThrowExceptionWhenNotAbleToLoadDatabaseConfigInProductionModeBecauseOfNonexistingDatasource() {
        try {
            Properties props = new Properties();
            props.setProperty(StaticConfigImpl.KEY_SOSIGW_MODE, "production");
            props.setProperty(StaticConfigImpl.KEY_JDBC_DATASOURCE_NAME, "foobar");

            setupConfig(props);

            assertTrue("Is productionmode", StaticConfigImpl.getInstance().isProductionMode());

            mapper.loadLatestConfig();
        } finally {
            restoreConfig();
        }
    }

    @Test
    public void shouldNotThrowExceptionWhenNotAbleToLoadDatabaseConfigInProductionMode() {
        try {
            Properties props = new Properties();
            props.setProperty(StaticConfigImpl.KEY_SOSIGW_MODE, "developer");
            props.setProperty(StaticConfigImpl.KEY_JDBC_DATASOURCE_NAME, "foobar");

            setupConfig(props);

            assertTrue("Is developermode", StaticConfigImpl.getInstance().isDeveloperRestrictedMode());

            mapper.loadLatestConfig();
        } finally {
            restoreConfig();
        }
    }
}
