/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.config;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Properties;

public class StaticConfigTestImpl implements StaticConfigApi {

    private String cluserIpPrefix = null;//"192";
    private int multicastPort = 3456;
    private String multicaseIpAddress = "228.0.0.7";
    private boolean bootstrapRestrictedMode;
    private boolean developerRestrictedMode;
    private boolean productionMode;
    public static final String DEFAULT_FMK_ISALIVE_URL = "http://trifork.lms.trifork.com/services/isalive";

    public String getClusterIpPrefix() {
        return cluserIpPrefix;
    }

    public int getClusterTuningParam(String param, int def) {
        if ("max.connections.per.host".equals(param)) {
            return 20;
        }
        return def;
    }

    public InetAddress getMulticastIpAddress() throws UnknownHostException {
        return InetAddress.getByName(multicaseIpAddress);
    }

    public int getMulticastPort() {
        return multicastPort;
    }

    public String getSupportedDGWSVersion() {
        return "1.0.1";
    }

    public boolean isBootstrapRestrictedMode() {
        return bootstrapRestrictedMode;
    }

    public boolean isDeveloperRestrictedMode() {
        return developerRestrictedMode;
    }

    public boolean isProductionMode() {
        return productionMode;
    }

    public boolean useLocalTimeInSaml() {
        return false;
    }

    public void setCluserIpPrefix(String cluserIpPrefix) {
        this.cluserIpPrefix = cluserIpPrefix;
    }

    public String getMode() {
        // TODO Auto-generated method stub
        return null;
    }

    public boolean isKeepAliveEnabled() {
        return true;
    }

    public String getFMKIsAliveURL() {
        return DEFAULT_FMK_ISALIVE_URL;
    }

   public String getCentralAuditLogTablePrefix() {
        return null;
   }

   public String getJdbcDriver() {
       return "org.hsqldb.jdbcDriver";
   }

   public boolean isGlobalAuditLogEnabled() {
       return false;
   }

    public boolean isAuditLogToDbEnabled() {
        return true;
    }

    public String getDataSourceName() {
       return "java:comp/env/jdbc/postgres";
   }

   public boolean isRestrictedWebserviceEnabled() {
       return true;
   }

    public String getConfigfilePath() {
        return "n/a";
    }

    public boolean isTestMode() {
        return false;
    }
    
    public boolean isSLALogEnabled() {
        return isProductionMode();
    }

    public RuntimeConfigStorageType getRuntimeconfigStorage() {
        return RuntimeConfigStorageType.DATABASE;
    }

    public Properties getProperties() {
        throw new UnsupportedOperationException("getProperties");
    }

    @Override
    public boolean isSTSMBeansEnabled() {
        return false;
    }
}
