/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.client.util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import org.junit.Test;


public class QueueTest {

    @Test
    public void getEmptyShouldReturnNull() {
        Queue q = new Queue(1);
        assertNull(q.get());
    }
    
    @Test
    public void testPutGet() {
        Queue q = new Queue(2);
        assertEquals(0, q.size());
        
        q.put("test");
        q.put("test2");
        
        assertEquals(2, q.size());
        
        assertEquals("test", q.get());
        assertEquals("test2", q.get());
        assertNull(q.get());
    }
    
    @Test(expected=IllegalStateException.class)
    public void putOnFullShouldFail() {
        Queue q = new Queue(1);
        q.put("test");
        q.put("test");
    }
    
    @Test
    public void testPeep() {
        Queue q = new Queue(1);
        q.put("test");
        
        assertEquals("test", q.peep());
        assertEquals("test", q.peep());
        q.get();
        assertNull(q.peep());
    }
    
    @Test
    public void refreshShouldMoveElementToEnd() {
        String t = "test";
        Queue q = new Queue(3);
        q.put("test2");
        q.put(t);
        q.put("test3");
        
        q.refresh(t);
        
        assertEquals("test2", q.get());
        assertEquals("test3", q.get());
        assertEquals(t, q.get());
    }

    @Test
    public void removeShouldWork() {
        String t = "blah";
        Queue q = new Queue(2);
        q.put(t);
        q.put("test");
        
        q.remove("e");
        assertEquals(2, q.size());
        
        q.remove(t);
        assertEquals(1, q.size());
        
        assertEquals("test", q.get());
        assertNull(q.get());
    }
    
}
