/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.console.datamodel;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import org.junit.Before;
import org.junit.Test;


public class ConfigCategoryTest {
    
    private ConfigCategory cc;

    @Before
    public void setup() {
        cc = new ConfigCategory();
    }
    
    @Test
    public void testItems() {
        assertNotNull(cc.getConfigItems());
        assertEquals(0, cc.getConfigItems().length);
        
        ConfigItem item = new ConfigItem(null, "test", new String[] { "test" });
        cc.addItem(item);
        
        assertEquals(1, cc.getConfigItems().length);
        
        cc.removeItem(item);
        assertEquals(0, cc.getConfigItems().length);
    }
    
    @Test
    public void testName() {
        assertNull(cc.getName());
        cc.setName("name");
        assertEquals("name", cc.getName());
    }
    
    @Test
    public void testDescription() {
        assertNull(cc.getDescription());
        cc.setDescription("test");
        assertEquals("test", cc.getDescription());
    }
    
    @Test
    public void testColumns() {
        assertNotNull(cc.getColumns());
        assertEquals(0, cc.getColumns().length);
        
        cc.addColumn("test");
        assertNotNull(cc.getColumns());
        assertEquals(1, cc.getColumns().length);
        
        cc.addColumn("test");
        assertEquals(2, cc.getColumns().length);
        
        cc.setColumns(new String[0]);
        assertEquals(0, cc.getColumns().length);
    }

}
