/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.idcardcache;

import com.trifork.sosigw.api.Constants;
import com.trifork.sosigw.cluster.IDCardClusterCacheEntry;
import com.trifork.sosigw.cluster.IDCardClusterCacheImpl;
import com.trifork.sosigw.cluster.intercom.TimeSource;

import org.junit.After;
import org.junit.Ignore;
import org.junit.Test;

import java.util.UUID;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;


public class IdCardCacheTest {
    
    @Test
    public void testObsoletion() throws Exception {
        final IDCardClusterCacheImpl cacheImpl = new IDCardClusterCacheImpl(null);
        final String nameID = "expireUser";
        final String anotherNameID = "newUser";
        final long startTime = TimeSource.currentTimeMillis();

        cacheImpl.ensureIDCard(nameID);
        assertNotNull(cacheImpl.findIDCardOrNull(nameID));

        TimeSource.setTestTime(startTime + (Constants.MAX_IDCARD_LIFETIME_HOURS - 1) * 60 * 60 * 1000);
        cacheImpl.ensureIDCard(anotherNameID + UUID.randomUUID());
        assertNotNull(cacheImpl.findIDCardOrNull(nameID));
        
        TimeSource.setTestTime(startTime + (Constants.MAX_IDCARD_LIFETIME_HOURS + 1) * 60 * 60 * 1000);
        IDCardClusterCacheEntry entry = cacheImpl.ensureIDCard(anotherNameID + UUID.randomUUID());
        assertNull(cacheImpl.findIDCardOrNull(nameID));

        assertNotNull(cacheImpl.findIDCardOrNull(entry.getCacheKey()));
    }
    
    @After
    public void cleanup() {
        TimeSource.setTestTime(0);
    }
}
