/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.nts;
///CLOVER:OFF
import com.sun.org.apache.xerces.internal.jaxp.datatype.XMLGregorianCalendarImpl;
import com.sun.xml.ws.api.message.Headers;
import com.sun.xml.ws.developer.WSBindingProvider;
import com.trifork.sosigw.api.Constants;
import com.trifork.sosigw.ws.IdCardTestUtils;
import dk.medcom.dgws._2006._04.dgws_1_0.Header;
import dk.medcom.dgws._2006._04.dgws_1_0.Linking;
import dk.nspop.nts._2013._05.NTS;
import dk.nspop.nts._2013._05.NTSService;
import dk.sosi.gw._2007_09.RequestIdCardDigestForSigningResponse;
import dk.sosi.gw._2007_09.SignIdCardRequestBody;
import dk.sosi.gw._2007_09.SosiGWFacade;
import dk.sosi.gw._2007_09.SosiGWFacadeService;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.constants.IDValues;
import dk.sosi.seal.model.constants.MedcomAttributes;
import dk.sosi.seal.model.constants.SOSIAttributes;
import dk.sosi.seal.model.constants.SubjectIdentifierTypeValues;
import oasis.names.tc.saml._2_0.assertion.*;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;
import org.w3._2000._09.xmldsig_.KeyInfo;
import org.w3._2000._09.xmldsig_.X509Data;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.handler.MessageContext;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.charset.Charset;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.cert.Certificate;
import java.util.*;

public class NTSClient {

    private final Argument nameId = new Argument("Name Id", "n", "idcard.nameid", "MitBrugerNavnSomErMereEnd10tegnLangt");
    private final Argument issuer = new Argument("Issuer", null, "idcard.issuer", "sosigwtester");
    private final Argument userCpr = new Argument("User CPR number", "c", "idcard.user.cpr", "0501792275");
    private final Argument userGivenName = new Argument("User given name", null, "idcard.user.givenname", "Tester");
    private final Argument userSurName = new Argument("User sur name", null, "idcard.user.surname", "Blåbærgrød");
    private final Argument userEmail = new Argument("User e-mail", null, "idcard.user.email", "test@trifork.com");
    private final Argument userRole = new Argument("User role", "r", "idcard.user.role", "læge");
    private final Argument userOccupation = new Argument("User occupation", "o", "idcard.user.occupation", "Gråstenæblegrødskoger");
    private final Argument userAuthorizationCode = new Argument("User Authorization code", "a", "idcard.user.authorizationcode", "J0184");
    private final Argument itSystem = new Argument("IT system", null, "idcard.itsystem", Constants.SOSIGW_IT_SYSTEM_TESTNAME);
    private final Argument careProviderType = new Argument("Care provider type", null, "idcard.careprovider.type", SubjectIdentifierTypeValues.CVR_NUMBER);
    private final Argument careProviderId = new Argument("Care provider id", "p", "idcard.careprovider.id", "20921897");
    private final Argument careProviderName = new Argument("Care provider name", null, "idcard.careprovider.name", "Vi brækker dit hårde øre");
    private final Argument keystorePath = new Argument("Keystore path", null, "keystore.path", "etc/validMocesVault.jks");
    private final Argument keystoreAlias = new Argument("Keystore alias", null, "keystore.alias", "SOSI:ALIAS_SYSTEM");
    private final Argument keystorePassword = new Argument("Keystore password", null, "keystore.password", "Test1234");
    private final Argument sosigwHost = new Argument("SOSI GW host and port", "g", "sosigw.host", "localhost:8080");
    private final Argument ntsHost = new Argument("NTS host and port", "h", "nts.host", "localhost:8080");
    private final Argument partition = new Argument("Partition", null, "partition", "");

    public static void main(String[] args) throws Exception {
        NTSClient ntsClient = new NTSClient();
        Properties properties = new Properties();
        File propertiesFile;
        if (args.length == 1 && args[0].indexOf("help") != -1) {
            ntsClient.usage();
            System.exit(0);
        }
        if (args.length > 0 && !args[0].startsWith("-")) {
            if (!args[0].isEmpty()) {
                propertiesFile = new File(args[0]);
                if (propertiesFile.exists() && propertiesFile.canRead()) {
                    properties.load(new InputStreamReader(new FileInputStream(propertiesFile), Charset.forName("utf-8")));
                } else {
                    System.err.println("properties file " + propertiesFile.getAbsolutePath() + " not found");
                    System.exit(1);
                }
            }
            String[] newArgs = new String[args.length - 1];
            System.arraycopy(args, 1, newArgs, 0, newArgs.length);
            args = newArgs;
        }
        ntsClient.configure(args, properties);
        ntsClient.execute();
    }

    private void configure(String[] args, Properties properties) {
        // First overwrite the default values with the properties loaded from a file 
        for (Object key : properties.keySet()) {
            Argument argument = Argument.longOptions.get(key);
            if (argument != null) {
                argument.set(properties.getProperty((String)key));
            } else {
                System.err.println("Unknown property found in file: " + key);
                System.exit(1);
            }
        }
        // then overwrite values with the arguments provided on the command line
        for (int i = 0; i < args.length; i++) {
            String arg = args[i];
            Argument argument = null;
            if (arg.startsWith("--")) {
                argument = Argument.longOptions.get(arg.substring(2));
            } else if (arg.startsWith("-")) {
                argument = Argument.shortOptions.get(arg.substring(1));
            } else {
                System.err.println("Syntax error in command line near '" + arg + "'");
                System.exit(1);
            }
            if (argument == null) {
                System.err.println("Unknown option '" + arg + "'");
                System.exit(1);
            }
            if (++i < args.length && !args[i].startsWith("-")) {
                argument.set(args[i]);
            } else {
                System.err.println("Invalid value for option '" + arg + "'");
                System.exit(1);
            }
        }
    }

    private void usage() {
        System.out.println("SOSI Gateway NTS client");
        System.out.println();
        System.out.println("\tjava NTSClient [properties file] [options]");
        System.out.println();
        System.out.println("Command line options have presedence over values in the properties file.");
        System.out.println("All options can be omitted if the default values are desired.");
        System.out.println("The properties file must be in utf-8 encoding, with the long options as keys");
        System.out.println("Possible options are:");
        for (Argument argument : Argument.longOptions.values()) {
            System.out.println(argument);
        }
    }

    private void execute() throws Exception {
        SosiGWFacade sosiGWFacade = getSosiGWFacadePort();

        Security securityHeader = makeIDCardInSecurity();

        System.out.println("Requesting IdCard for signing");
        RequestIdCardDigestForSigningResponse response = sosiGWFacade.requestIdCardDigestForSigning(securityHeader, "");

        SignIdCardRequestBody signIdCardRequestBody = signIdCardDigest(response);

        System.out.println("Signing IdCard");
        sosiGWFacade.signIdCard(securityHeader, signIdCardRequestBody);

        System.out.println("Verifying IdCard in cache");
        sosiGWFacade.getValidIdCard(securityHeader, "Ignored");

        NTS nts = getNTSPort();
        Header medcomHeader = createMedcomHeader();

        System.out.println("Calling NTS through SOSI GW");
        nts.verify(securityHeader, medcomHeader, "payload");

        System.out.println("Done");
    }

    private SosiGWFacade getSosiGWFacadePort() {
        SosiGWFacadeService sosiGWFacadeService = new SosiGWFacadeService(getClass().getResource("wsdl/sosigw.wsdl"), new QName("http://sosi.dk/gw/2007.09.01", "SosiGWFacadeService"));
        SosiGWFacade sosiGWFacade = sosiGWFacadeService.getSosiGWSoapBinding();
        BindingProvider sosiGWFacadeBindingProvider = (BindingProvider)sosiGWFacade;
        sosiGWFacadeBindingProvider.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "http://" + sosigwHost.value() + "/sosigw/service/sosigw");
        if (!partition.value().trim().isEmpty()) {
            sosiGWFacadeBindingProvider.getRequestContext().put(MessageContext.HTTP_REQUEST_HEADERS, Collections.singletonMap("SOSIGW_PARTITION", Collections.singletonList(partition.value())));
        }
        return sosiGWFacade;
    }

    public Security makeIDCardInSecurity() {
        Security security = new Security();
        security.setTimestamp(IdCardTestUtils.makeTimestamp());
        security.setAssertion(makeIDCard());
        return security;
    }

    public AssertionType makeIDCard() {
        AssertionType assertion = new AssertionType();
        assertion.setIssuer(issuer.value());
        assertion.setId(IDValues.IDCARD);

        NameIDType nameID = new NameIDType();
        nameID.setValue(nameId.value());
        nameID.setFormat(SubjectIdentifierTypeValues.OTHER);
        Subject subject = new Subject();
        subject.setNameID(nameID);
        assertion.setSubject(subject);

        AttributeStatement stmt = new AttributeStatement();

        stmt.setId(IDValues.IDCARD_DATA);
        stmt.getAttribute().add(IdCardTestUtils.makeAttribute(SOSIAttributes.IDCARD_VERSION, null, "1.0.1"));
        stmt.getAttribute().add(IdCardTestUtils.makeAttribute(SOSIAttributes.IDCARD_TYPE, null, IDCard.IDCARDTYPE_USER));
        stmt.getAttribute().add(IdCardTestUtils.makeAttribute(SOSIAttributes.AUTHENTICATION_LEVEL, null, "1"));
        stmt.getAttribute().add(IdCardTestUtils.makeAttribute(SOSIAttributes.IDCARD_ID, null, "id-card-id"));
        assertion.getAttributeStatement().add(stmt);

        stmt = new AttributeStatement();
        stmt.setId(IDValues.USER_LOG);
        stmt.getAttribute().add(IdCardTestUtils.makeAttribute(MedcomAttributes.USER_CIVIL_REGISTRATION_NUMBER, null, userCpr.value()));
        stmt.getAttribute().add(IdCardTestUtils.makeAttribute(MedcomAttributes.USER_GIVEN_NAME, null, userGivenName.value()));
        stmt.getAttribute().add(IdCardTestUtils.makeAttribute(MedcomAttributes.USER_SURNAME, null, userSurName.value()));
        stmt.getAttribute().add(IdCardTestUtils.makeAttribute(MedcomAttributes.USER_EMAIL_ADDRESS, null, userEmail.value()));
        stmt.getAttribute().add(IdCardTestUtils.makeAttribute(MedcomAttributes.USER_ROLE, null, userRole.value()));
        stmt.getAttribute().add(IdCardTestUtils.makeAttribute(MedcomAttributes.USER_OCCUPATION, null, userOccupation.value()));
        stmt.getAttribute().add(IdCardTestUtils.makeAttribute(MedcomAttributes.USER_AUTHORIZATION_CODE, null, userAuthorizationCode.value()));
        assertion.getAttributeStatement().add(stmt);

        stmt = new AttributeStatement();
        stmt.setId(IDValues.SYSTEM_LOG);
        stmt.getAttribute().add(IdCardTestUtils.makeAttribute(MedcomAttributes.IT_SYSTEM_NAME, null, itSystem.value()));
        stmt.getAttribute().add(IdCardTestUtils.makeAttribute(MedcomAttributes.CARE_PROVIDER_ID, careProviderType.value(), careProviderId.value()));
        stmt.getAttribute().add(IdCardTestUtils.makeAttribute(MedcomAttributes.CARE_PROVIDER_NAME, null, careProviderName.value()));
        assertion.getAttributeStatement().add(stmt);

        assertion.setConditions(new Conditions());

        Calendar notBefore = Calendar.getInstance();
        notBefore.add(Calendar.MINUTE, -5);

        Calendar notOnOrAfter = Calendar.getInstance();
        notOnOrAfter.add(Calendar.HOUR, 8);

        notBefore.add(Calendar.MILLISECOND, notBefore.getTimeZone().getOffset(notBefore.getTimeInMillis()));
        notOnOrAfter.add(Calendar.MILLISECOND, notOnOrAfter.getTimeZone().getOffset(notOnOrAfter.getTimeInMillis()));

        assertion.getConditions().setNotBefore(new XMLGregorianCalendarImpl((GregorianCalendar)notBefore));
        assertion.getConditions().setNotOnOrAfter(new XMLGregorianCalendarImpl((GregorianCalendar)notOnOrAfter));
        return assertion;
    }

    private SignIdCardRequestBody signIdCardDigest(RequestIdCardDigestForSigningResponse response) throws GeneralSecurityException, IOException {
        byte[] digestValue = response.getDigestValue();
        SignIdCardRequestBody signIdCardRequestBody = new SignIdCardRequestBody();

        KeyStore keystore = KeyStore.getInstance("JKS");
        keystore.load(new FileInputStream(keystorePath.value()), keystorePassword.value().toCharArray());

        Certificate certificate = keystore.getCertificate(keystoreAlias.value());
        PrivateKey key = (PrivateKey)keystore.getKey(keystoreAlias.value(), keystorePassword.value().toCharArray());

        byte[] signatureValue = IdCardTestUtils.signDigest(digestValue, key);

        signIdCardRequestBody.setSignatureValue(signatureValue);

        KeyInfo keyInfo = new KeyInfo();
        X509Data x509Data = new X509Data();

        x509Data.setX509Certificate(certificate.getEncoded());

        keyInfo.setX509Data(x509Data);
        signIdCardRequestBody.setKeyInfo(keyInfo);
        return signIdCardRequestBody;
    }

    @SuppressWarnings("resource")
    private NTS getNTSPort() throws MalformedURLException {
        getClass();
        NTSService ntsService = new NTSService(new URL("file:test/etc/nts.wsdl"), new QName("http://nspop.dk/nts/2013/05", "NTSService"));

        NTS nts = ntsService.getNtsPort();
        WSBindingProvider ntsWsBindingProvider = (WSBindingProvider)nts;
        BindingProvider ntsBindingProvider = (javax.xml.ws.BindingProvider)nts;

        ntsWsBindingProvider.setAddress("http://" + ntsHost.value() + "/nts/service");

        List<com.sun.xml.ws.api.message.Header> headers = new ArrayList<com.sun.xml.ws.api.message.Header>();
        headers.add(Headers.create(new QName("http://www.w3.org/2005/08/addressing", "To"), "http://" + ntsHost.value() + "/nts/service"));
        headers.add(Headers.create(new QName("http://www.w3.org/2005/08/addressing", "Action"), "http://nspop.dk/nts/2013/05#verify"));
        headers.add(Headers.create(new QName("http://www.w3.org/2005/08/addressing", "MessageID"), "2"));

        ntsWsBindingProvider.setOutboundHeaders((com.sun.xml.ws.api.message.Header[]) headers.toArray(new com.sun.xml.ws.api.message.Header[headers.size()]));

        ntsBindingProvider.getRequestContext().put(javax.xml.ws.BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "http://" + sosigwHost.value() + "/sosigw/proxy/soap-request");
        ntsBindingProvider.getRequestContext().put("com.sun.xml.ws.connect.timeout", new Integer(5 * 1000));
        ntsBindingProvider.getRequestContext().put("com.sun.xml.ws.request.timeout", new Integer(200 * 1000));
        if (!partition.value().trim().isEmpty()) {
            ntsBindingProvider.getRequestContext().put(MessageContext.HTTP_REQUEST_HEADERS, Collections.singletonMap("SOSIGW_PARTITION", Collections.singletonList(partition.value())));
        }
        return nts;
    }

    private Header createMedcomHeader() {
        Header medcomHeader = new Header();
        Linking linking = new Linking();
        linking.setFlowID(UUID.randomUUID().toString());
        linking.setMessageID(UUID.randomUUID().toString());
        medcomHeader.setLinking(linking);
        int securityLevel = 4;
        medcomHeader.setSecurityLevel(securityLevel);
        medcomHeader.setRequireNonRepudiationReceipt("no");
        return medcomHeader;
    }

    private static class Argument {

        private static final Map<String, Argument> shortOptions = new HashMap<String, NTSClient.Argument>();
        private static final Map<String, Argument> longOptions = new HashMap<String, NTSClient.Argument>();

        private final String name;
        private final String shortOption;
        private final String longOption;

        private String value;

        public Argument(String name, String shortOption, String longOption, String defaultValue) {
            this.name = name;
            this.shortOption = shortOption;
            this.longOption = longOption;
            this.value = defaultValue;
            if (shortOption != null) {
                if (shortOptions.containsKey(shortOption)) {
                    throw new IllegalStateException(shortOption);
                }
                shortOptions.put(shortOption, this);
            }
            if (longOptions.containsKey(longOption)) {
                throw new IllegalStateException(longOption);
            }
            longOptions.put(longOption, this);
        }

        public void set(String value) {
            this.value = value;
        }

        public String value() {
            return value;
        }

        @Override
        public String toString() {
            StringBuilder sb = new StringBuilder();
            sb.append('\n').append("   ");
            if (shortOption != null) {
                sb.append('-').append(shortOption);
            } else {
                sb.append("  ");
            }
            sb.append("   ").append("--").append(longOption).append('\n');
            sb.append("           ").append(name).append(". ").append("Default value is '").append(value).append('\'');
            return sb.toString();
        }
    }
}
