/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.proxy;

import com.trifork.sosigw.api.Constants;
import com.trifork.sosigw.ws.AbstractIntegrationTests;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.MultiThreadedHttpConnectionManager;
import org.apache.commons.httpclient.methods.ByteArrayRequestEntity;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.io.IOUtils;
import org.apache.james.mime4j.MimeException;
import org.junit.Test;

import javax.servlet.ServletException;

import java.io.IOException;
import java.io.InputStream;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class ImplicitLoginIntegrationTest extends AbstractIntegrationTests {
    private static final MultiThreadedHttpConnectionManager connectionManager = new MultiThreadedHttpConnectionManager();

    @Test(timeout = 60000)
    public void testImplicitLogin() throws IOException, ServletException  {
        PostMethod postMethod = new PostMethod(SERVER_URL + "/proxy/soap-request");
        
        InputStream resourceAsStream = ImplicitLoginIntegrationTest.class.
                getClassLoader().getResourceAsStream("com/trifork/sosigw/proxy/idcard-level1-testdata.xml");
        final byte[] buffer = new byte[100000];
        final int length = resourceAsStream.read(buffer);
        final byte[] bytes = new byte[length];
        System.arraycopy(buffer, 0, bytes, 0, length);
        
        postMethod.setRequestEntity(new ByteArrayRequestEntity(bytes, "text/xml; charset=utf-8"));
        postMethod.addRequestHeader("Accept", "text/xml");
        HttpClient httpClient = new HttpClient(connectionManager);
        httpClient.executeMethod(postMethod);
        int status = postMethod.getStatusCode();
        String bodyAsString = postMethod.getResponseBodyAsString();
        assertEquals(500, status);
        assertTrue(bodyAsString.indexOf("implicitLoginHeader>") > -1);
        assertTrue(bodyAsString.indexOf("requestIdCardDigestForSigningResponse>") > -1);
        assertTrue(bodyAsString.indexOf("DigestValue>") > -1);
        assertTrue(bodyAsString.indexOf("BrowserUrl>") > -1);
        assertTrue(bodyAsString.indexOf(Constants.FAULTCODE_NO_VALID_IDCARD_IN_CACHE) > -1);
    }

    @Test(timeout = 60000)
    public void testMtomRawRequest() throws IOException, MimeException {
        PostMethod postMethod = new PostMethod(SERVER_URL + "/proxy/soap-request");
        String contentType = "multipart/related;start=\"<rootpart*c40fbefb-c8d3-4258-aa80-cf891234b14e@example.jaxws.sun.com>\";type=\"application/xop+xml\";boundary=\"uuid:c40fbefb-c8d3-4258-aa80-cf891234b14e\";start-info=\"text/xml\"";
        final InputStream request = SoapHeaderParserTest.class.
                getClassLoader().getResourceAsStream("com/trifork/sosigw/proxy/mtom-request.dat");
        postMethod.setRequestEntity(new ByteArrayRequestEntity(IOUtils.toByteArray(request), contentType));
        postMethod.addRequestHeader("Accept", "text/xml");
        HttpClient httpClient = new HttpClient(connectionManager);
        httpClient.executeMethod(postMethod);
        int status = postMethod.getStatusCode();
        String bodyAsString = postMethod.getResponseBodyAsString();
        assertEquals(500, status);

        assertTrue(bodyAsString.indexOf("implicitLoginHeader>") > -1);
        assertTrue(bodyAsString.indexOf("requestIdCardDigestForSigningResponse>") > -1);
        assertTrue(bodyAsString.indexOf("DigestValue>") > -1);
        assertTrue(bodyAsString.indexOf("BrowserUrl>") > -1);
        assertTrue(bodyAsString.indexOf(Constants.FAULTCODE_NO_VALID_IDCARD_IN_CACHE) > -1);
    }

}
