/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.proxy;

import com.trifork.sosigw.api.Constants;
import com.trifork.sosigw.config.StaticConfigImpl;
import com.trifork.sosigw.util.HeaderReadingInputStream;
import com.trifork.sosigw.util.SoapHeader;
import org.apache.james.mime4j.MimeException;
import org.junit.BeforeClass;
import org.junit.Test;
import org.w3c.dom.Document;

import javax.servlet.ServletException;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;

import static org.junit.Assert.assertTrue;

public class SoapHeaderParserTest {

    private static final String BASIC_SOAP_1_HEADER_PART_2 = " </soap:Header>";

    private static final String BASIC_SOAP_1_HEADER_PART_1 
    = Constants.XML_VERSION_1_0_ENCODING_UTF_8
    + "<soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
    + " xmlns:saml=\"urn:oasis:names:tc:SAML:2.0:assertion\" xmlns:wsse=\"http://docs.oasis-"
    + "open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd\""
    + " xmlns:ds=\"http://www.w3.org/2000/09/xmldsig\" xmlns:medcom=\"http://www.medcom.dk/dgws/2006/04/dgws-"
    + "1.0.xsd\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:wsu=\"http://docs.oasis-"
    + "open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd\" wsu:id=\"Envelope\">"
    + " <soap:Header>";

       
    private static final String SAML1 
    = BASIC_SOAP_1_HEADER_PART_1
    + "<wsse:Security>"
    + "<wsu:Timestamp>"
    + "<wsu:Created>2006-06-01T08:01:00</wsu:Created>"
    + "</wsu:Timestamp>"
    + "<saml:Assertion id=\"IDCard\" IssueInstant=\"2006-06-01T07:53:00\" Version=\"2.0\">"
    + "<saml:Issuer>LægeSystemA</saml:Issuer>"
    + "<saml:Subject>"
    + "<saml:NameID Format=\"medcom:cprnumber\">2606444917</saml:NameID>"
    + "</saml:Subject>"
    + "<saml:Conditions NotBefore=\"2006-06-01T08:00:00\" NotOnOrAfter=\"2006-07-01T07:53:00\"/>"
    + "</saml:Assertion>"
    + "</wsse:Security>"
    + BASIC_SOAP_1_HEADER_PART_2
    + "<soap:Body/>" + "</soap:Envelope>";

    @BeforeClass
    public static void setup() {
        StaticConfigImpl.initSettings();
    }

    @Test
    public void testParseHeaders0() throws IOException, ServletException, MimeException {
        final HeaderReadingInputStream hris = new HeaderReadingInputStream(makeInputStream(SAML1));
        Document soapHeaderNode = SoapHeader.getSoapEnvelopeNode(hris.getSoapHeader(), hris.getSoapNamespace());
        String newHeaders = ProxyRequestHandler.serializeNewHeader(soapHeaderNode);

        assertTrue(newHeaders.length() > 0);
    }


    private InputStream makeInputStream(String str) {
        byte[] bytes;
        try {
            bytes = str.getBytes("UTF-8");
            return new ByteArrayInputStream(bytes);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);// UTF-8 is a requirement.
        }
    }
}
