/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.proxy;

import com.sun.net.httpserver.*;
import com.trifork.sosigw.config.StaticConfigImpl;
import com.trifork.sosigw.util.Util;
import dk.sdsd.nsp.slalog.impl.SLALogItemImpl;
import dk.sdsd.nsp.slalog.impl.SLALoggerImpl;
import dk.sosi.gw._2007_09.DGWSFault;
import org.apache.catalina.core.DummyRequest;
import org.apache.catalina.core.DummyResponse;
import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.apache.commons.httpclient.params.HttpMethodParams;
import org.apache.log4j.Logger;
import org.junit.BeforeClass;
import org.junit.Test;

import javax.net.ssl.*;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.security.KeyStore;
import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import static org.junit.Assert.assertEquals;

/**
 * Tests that http-address in WsAdressing To-tag is upgraded to https address only when force-ssl-requests is true
 */
public class forceSSLTest {
    private static final Logger logger = Logger.getLogger(forceSSLTest.class);
    private static final int PORT = 25001;
    private static final String URL_STRING = "//localhost:" + PORT + "/test";
    private static final String TEST_STRING = "Greetings from test SSL server!";
    private static final String TEST_RESPONSE = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>" +
            "<soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
            "<soap:Header/>\n" +
            "<soap:Body>\n" +
            "<test>" + TEST_STRING + "</test>\n" +
            "</soap:Body>\n" +
            "</soap:Envelope>";

    @BeforeClass
    public static void configure() {
        StaticConfigImpl.initSettings();
        SLALoggerImpl slaLogger = new SLALoggerImpl();
        Util.setSLALogger(slaLogger);
        Util.setSLALogItem(new SLALogItemImpl("test", "test", slaLogger));
    }

    /**
     * Configure SSLContext to accept self signed certificate
     */
    @BeforeClass
    public static void acceptAnyCert() {
        try {
            SSLContext ctx = SSLContext.getInstance("TLS");
            ctx.init(new KeyManager[0], new TrustManager[]{new X509TrustManager() {
                @Override
                public void checkClientTrusted(X509Certificate[] arg0, String arg1) throws CertificateException {
                }

                @Override
                public void checkServerTrusted(X509Certificate[] arg0, String arg1) throws CertificateException {
                }

                @Override
                public X509Certificate[] getAcceptedIssuers() {
                    return null;
                }
            }}, new SecureRandom());
            SSLContext.setDefault(ctx);
        } catch (Exception ex) {
            logger.error("Configure SSLContext failed", ex);
        }
    }

    /**
     * Start HTTPS server on port 25001
     */
    @SuppressWarnings("restriction")
    @BeforeClass
    public static void setupHttpsServer() {
        try {
            // setup the socket address
            InetSocketAddress address = new InetSocketAddress(PORT);

            // initialise the HTTPS server
            HttpsServer httpsServer = HttpsServer.create(address, 0);
            SSLContext sslContext = SSLContext.getInstance("TLS");

            // initialise the keystore
            char[] password = "Test1234".toCharArray();
            KeyStore ks = KeyStore.getInstance("JKS");
            ks.load(ClassLoader.getSystemResourceAsStream("etc/certificate.jks"), password);

            // setup the key manager factory
            KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
            kmf.init(ks, password);

            // setup the trust manager factory
            TrustManagerFactory tmf = TrustManagerFactory.getInstance("SunX509");
            tmf.init(ks);

            // setup the HTTPS context and parameters
            sslContext.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);
            httpsServer.setHttpsConfigurator(new HttpsConfigurator(sslContext) {
                public void configure(HttpsParameters params) {
                    try {
                        // initialise the SSL context
                        SSLContext c = SSLContext.getDefault();
                        SSLEngine engine = c.createSSLEngine();
                        params.setNeedClientAuth(false);
                        params.setCipherSuites(engine.getEnabledCipherSuites());
                        params.setProtocols(engine.getEnabledProtocols());

                        // get the default parameters
                        SSLParameters defaultSSLParameters = c.getDefaultSSLParameters();
                        params.setSSLParameters(defaultSSLParameters);
                    } catch (Exception ex) {
                        logger.error("Failed to create HTTPS port", ex);
                    }
                }
            });

            httpsServer.createContext("/test", new HttpHandler() {
                @Override
                public void handle(HttpExchange httpExchange) throws IOException {
                    HttpsExchange httpsExchange = (HttpsExchange) httpExchange;
                    httpsExchange.getResponseHeaders().add("Access-Control-Allow-Origin", "*");
                    httpsExchange.getResponseHeaders().add("Content-Type", "text/xml; charset=utf-8");


                    httpsExchange.sendResponseHeaders(200, TEST_RESPONSE.length());
                    OutputStream os = httpsExchange.getResponseBody();
                    os.write(TEST_RESPONSE.getBytes());
                    os.close();
                }
            });
            httpsServer.setExecutor(null);

            httpsServer.start();
        } catch (Exception ex) {
            logger.error("Failed to create HTTPS server on port " + PORT + " of localhost", ex);
        }
    }

    /**
     * Call server through ProxyRequestHandler
     *
     * @param url      endpoint to call
     * @param forceSSL true to force HTTPS-requests
     * @return Response from test server
     * @throws Exception
     */
    private String callServer(String url, boolean forceSSL) throws Exception {
        final StringBuilder result = new StringBuilder();

        HttpServletRequest request = new DummyRequest();
        HttpServletResponse response = new DummyResponse() {
            @Override
            public ServletOutputStream getOutputStream() {
                return new ServletOutputStream() {
                    @Override
                    public void write(int b) throws IOException {
                        result.append(Character.toString((char) b));
                    }
                };
            }
        };

        WsAddressingHandler addressingHandler = new WsAddressingHandler(forceSSL);
        WSAHeaderInfo headerInfo = addressingHandler.handleWSAddressing(url, null, "test");

        PostMethod postMethod = new PostMethod(headerInfo.to);
        postMethod.getParams().setParameter(HttpMethodParams.RETRY_HANDLER, new DefaultHttpMethodRetryHandler(1, false));
        postMethod.getParams().setParameter("http.socket.timeout", 5000);

        ProxyRequestHandler proxyRequestHandler = new ProxyRequestHandler(request, response);
        proxyRequestHandler.invokeRemoteService(postMethod, new StringRequestEntity("test"), headerInfo, "test");

        return result.toString();
    }

    @Test
    public void testHttpsServerAlive() throws Exception {
        GetMethod getMethod = new GetMethod("https:" + URL_STRING);
        getMethod.getParams().setParameter(HttpMethodParams.RETRY_HANDLER, new DefaultHttpMethodRetryHandler(1, false));
        getMethod.getParams().setParameter("http.socket.timeout", 5000);

        HttpClient client = new HttpClient();
        int responseCode = client.executeMethod(getMethod);
        byte[] responseStream = getMethod.getResponseBody();

        System.out.println("Response Code: " + responseCode);
        System.out.println("Response Body: \n" + new String(responseStream));
    }

    @Test(expected = DGWSFault.class) // server does not accept plain http
    public void testHttpUntouchedWhenForceFalse() throws Exception {
        callServer("http:" + URL_STRING, false);
    }

    @Test
    public void testHttpsUntouchedWhenForceFalse() throws Exception {
        String result = callServer("https:" + URL_STRING, false);
        assertEquals("Expected correct response from SSL service call", TEST_RESPONSE, result);
    }

    @Test
    public void testHttpChangedToHttpsWhenForceTrue() throws Exception {
        String result = callServer("http:" + URL_STRING, true);
        assertEquals("Expected correct response from SSL service call", TEST_RESPONSE, result);
    }

    @Test
    public void testHttpsUntouchedWhenForceTrue() throws Exception {
        String result = callServer("https:" + URL_STRING, true);
        assertEquals("Expected correct response from SSL service call", TEST_RESPONSE, result);
    }
}
