/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.restrictedws;

import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.cert.Certificate;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;

import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;
import org.w3._2000._09.xmldsig_.KeyInfo;
import org.w3._2000._09.xmldsig_.X509Data;

import com.trifork.sosigw.ws.AbstractIntegrationTests;
import com.trifork.sosigw.ws.IdCardTestUtils;

import dk.sosi.gw._2007_09.RequestIdCardDigestForSigningResponse;
import dk.sosi.gw._2007_09.SignIdCardRequestBody;
import dk.sosi.gw._2007_09.SosiGWFacade;
import dk.sosi.gw._2007_09.SosiGWFacadeService;
import dk.sosi.gw._2007_09.SosiGWRestrictedFacade;
import dk.sosi.gw._2007_09.SosiGWRestrictedFacadeService;
import dk.sosi.seal.model.constants.MedcomAttributes;
import oasis.names.tc.saml._2_0.assertion.NameIDType;

public class AbstractIntegrationTestsRestricted extends AbstractIntegrationTests {
    
    protected SosiGWFacade getFacade() {
        // Try to fake it so that we hit the restricted endpoint with a non-restricted facade. This should fail in GetValidIdCardTest.
        SosiGWFacadeService service = new SosiGWFacadeService(getClass().getResource("wsdl/sosigw.wsdl"), new QName("http://sosi.dk/gw/2007.09.01", "SosiGWFacadeService"));
        SosiGWFacade proxy = service.getSosiGWSoapBinding();
        BindingProvider bp = (BindingProvider)proxy;
        bp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "http://localhost:8586/sosigw/service/sosigw-restricted");
        
        return proxy;
    }

    protected SosiGWRestrictedFacade getRestrictedFacade() {
        SosiGWRestrictedFacadeService service = new SosiGWRestrictedFacadeService(getClass().getResource("wsdl/sosigw-restricted.wsdl"), new QName("http://sosi.dk/gw/2007.09.01", "SosiGWRestrictedFacadeService"));
        SosiGWRestrictedFacade proxy = service.getSosiGWRestrictedSoapBinding();
        BindingProvider bp = (BindingProvider)proxy;
        bp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "http://localhost:8586/sosigw/service/sosigw-restricted");
        
        return proxy;
    }
    
    protected void createSignedIdCard() throws Exception {
        Security requestIdCardDigestForSigningRequest = IdCardTestUtils.makeIDCardInSecurity();

        SosiGWRestrictedFacade sosiGWRestrictedFacade = getRestrictedFacade();
        RequestIdCardDigestForSigningResponse response = sosiGWRestrictedFacade.requestIdCardDigestForSigning(requestIdCardDigestForSigningRequest, "");

        byte[] digestValue = response.getDigestValue();
        SignIdCardRequestBody signIdCardRequestBody = new SignIdCardRequestBody();
        
        NameIDType nameID = new NameIDType();
        nameID.setFormat(MedcomAttributes.USER_CIVIL_REGISTRATION_NUMBER);
        nameID.setValue("1234567890");
        
        String password = "Test1234";
        KeyStore keystore = KeyStore.getInstance("JKS");
        keystore.load(ClassLoader.getSystemResourceAsStream("etc/validMocesVault.jks"), password.toCharArray());

        Certificate certificate = keystore.getCertificate("SOSI:ALIAS_SYSTEM");
        PrivateKey key = (PrivateKey)keystore.getKey("SOSI:ALIAS_SYSTEM", password.toCharArray());
        
        byte[] signatureValue = IdCardTestUtils.signDigest(digestValue, key);

        signIdCardRequestBody.setSignatureValue(signatureValue);

        KeyInfo keyInfo = new KeyInfo();
        X509Data x509Data = new X509Data();

        x509Data.setX509Certificate(certificate.getEncoded());
        
        keyInfo.setX509Data(x509Data);
        signIdCardRequestBody.setKeyInfo(keyInfo);

        sosiGWRestrictedFacade.signIdCard(requestIdCardDigestForSigningRequest, signIdCardRequestBody);
    }
}

