/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.restrictedws;

import static org.junit.Assert.fail;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.ws.soap.SOAPFaultException;

import org.junit.Test;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;

import com.sun.xml.bind.api.JAXBRIContext;
import com.sun.xml.ws.api.message.Headers;
import com.sun.xml.ws.developer.WSBindingProvider;
import com.trifork.sosigw.ws.IdCardTestUtils;

import dk.sosi.gw._2007_09.DGWSFault;
import dk.sosi.gw._2007_09.SosiGWFacade;


public class GetValidIdCardTest extends AbstractIntegrationTestsRestricted {

    @Test(timeout = 60000)
    public void testRemote() throws Exception  {
        createSignedIdCard();

        SosiGWFacade sosiGWFacade = getFacade();

        Security sosigwGetValidIdCardRequestHeader = IdCardTestUtils.makeIDCardInSecurity();
        String sosigwGetValidIdCardRequestBody = "Who cares";
        
        WSBindingProvider bp = (WSBindingProvider)sosiGWFacade;
        JAXBRIContext medcomHeader;
        try {
            medcomHeader = (JAXBRIContext)JAXBContext.newInstance(IdCardTestUtils.MedcomHeader.class);
        } catch (JAXBException e) {
            e.printStackTrace();
            fail(); return;
        }
        bp.setOutboundHeaders(Headers.create(medcomHeader,new IdCardTestUtils.MedcomHeader()));
        try {
            sosiGWFacade.getValidIdCard(sosigwGetValidIdCardRequestHeader, sosigwGetValidIdCardRequestBody);
        } catch (SOAPFaultException e) {
            // We expect this failure - getValidIdCard is not allow through restricted webservice
            return;
        } catch (DGWSFault f) {
            return;
        }

        fail("getValidIdCard was expected to fail on the restricted webservice");
    }
}
