/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.restrictedws;

import static org.junit.Assert.assertNotNull;

import java.io.UnsupportedEncodingException;

import javax.xml.transform.TransformerException;

import org.junit.Test;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;

import com.trifork.sosigw.util.timing.Log4jTimeSink;
import com.trifork.sosigw.util.timing.Timer;
import com.trifork.sosigw.ws.IdCardTestUtils;

import dk.sosi.gw._2007_09.DGWSFault;
import dk.sosi.gw._2007_09.RequestIdCardDigestForSigningResponse;
import dk.sosi.gw._2007_09.SosiGWRestrictedFacade;

public class RequestIdCardDigestForSigningTest extends AbstractIntegrationTestsRestricted {
        
    static {
        Timer.setTimer(new Timer(new Log4jTimeSink(), 32));
    }

    @Test(timeout = 60000)
    public void testRemoteCall() throws TransformerException, DGWSFault, UnsupportedEncodingException {
        Security requestIdCardDigestForSigningRequest = IdCardTestUtils.makeIDCardInSecurity();
        
        RequestIdCardDigestForSigningResponse response = executeRemoteCall(requestIdCardDigestForSigningRequest);
        assertNotNull(response.getBrowserUrl());
        assertNotNull(response.getDigestValue());
        System.out.println("Browser URL: ");
        System.out.println(response.getBrowserUrl());
    }

    private RequestIdCardDigestForSigningResponse executeRemoteCall(Security requestIdCardDigestForSigningRequest) throws DGWSFault {
        SosiGWRestrictedFacade sosiGWFacade = getRestrictedFacade();
        RequestIdCardDigestForSigningResponse requestIdCardDigestForSigning = sosiGWFacade.requestIdCardDigestForSigning(requestIdCardDigestForSigningRequest, "who cares");
        return requestIdCardDigestForSigning;
    }
}
