/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.restrictedws;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.cert.Certificate;

import javax.xml.transform.TransformerException;

import org.junit.Test;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;
import org.w3._2000._09.xmldsig_.KeyInfo;
import org.w3._2000._09.xmldsig_.X509Data;

import com.trifork.sosigw.ws.IdCardTestUtils;

import dk.sosi.gw._2007_09.DGWSFault;
import dk.sosi.gw._2007_09.RequestIdCardDigestForSigningResponse;
import dk.sosi.gw._2007_09.SignIdCardRequestBody;
import dk.sosi.gw._2007_09.SosiGWRestrictedFacade;
import dk.sosi.seal.model.constants.MedcomAttributes;
import oasis.names.tc.saml._2_0.assertion.NameIDType;

public class SignIdCardRequestTest extends AbstractIntegrationTestsRestricted {

    @Test(timeout = 60000)
    public void testSignIdCard() throws TransformerException, DGWSFault, Exception {
        Security requestIdCardDigestForSigningRequest = IdCardTestUtils.makeIDCardInSecurity();

        SosiGWRestrictedFacade sosiGWFacade = getRestrictedFacade();
        RequestIdCardDigestForSigningResponse response = sosiGWFacade.requestIdCardDigestForSigning(requestIdCardDigestForSigningRequest, "");

        assertNotNull(response.getBrowserUrl());

        byte[] digestValue = response.getDigestValue();

        assertNotNull(digestValue);
        
        SignIdCardRequestBody signIdCardRequestBody = new SignIdCardRequestBody();
        
        NameIDType nameID = new NameIDType();
        nameID.setFormat(MedcomAttributes.USER_CIVIL_REGISTRATION_NUMBER);
        nameID.setValue("1234567890");
        
        String password = "Test1234";
        KeyStore keystore = KeyStore.getInstance("JKS");
        keystore.load(ClassLoader.getSystemResourceAsStream("etc/validMocesVault.jks"), password.toCharArray());

        Certificate certificate = keystore.getCertificate("SOSI:ALIAS_SYSTEM");
        PrivateKey key = (PrivateKey)keystore.getKey("SOSI:ALIAS_SYSTEM", password.toCharArray());
        
        byte[] signatureValue = IdCardTestUtils.signDigest(digestValue, key);

        signIdCardRequestBody.setSignatureValue(signatureValue);

        KeyInfo keyInfo = new KeyInfo();
        X509Data x509Data = new X509Data();

        x509Data.setX509Certificate(certificate.getEncoded());
        
        keyInfo.setX509Data(x509Data);
        signIdCardRequestBody.setKeyInfo(keyInfo);

        String validIdCard = sosiGWFacade.signIdCard(requestIdCardDigestForSigningRequest, signIdCardRequestBody);

        assertEquals("ok", validIdCard);
    }

}
