/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.sts;

import com.trifork.sosigw.config.RuntimeConfig;
import com.trifork.sosigw.config.RuntimeConfigItem;
import com.trifork.sosigw.config.RuntimeConfigState;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class STSClusterProxyCreateTest {
    @Before
    public void setup() {
        RuntimeConfig.setRuntimeConfigState(new RuntimeConfigState(1, new RuntimeConfigItem[]{}));
    }

    @Test
    public void shouldWorkForSingletonList() {
        final STSClusterProxy cluster = STSClusterProxy.create("http://my.sts.1/sts");
        Assert.assertEquals(1, cluster.getProxies().size());
    }

    @Test
    public void shouldMakeThreeProxiesForThreeUrls() {
        final STSClusterProxy cluster = STSClusterProxy.create("http://my.sts.1/sts http://my.sts.2/sts http://my.sts.3/sts");
        Assert.assertEquals(3, cluster.getProxies().size());
    }

    @Test
    public void shouldNotMakeEmptyProxiesForExtraSpaces() {
        final STSClusterProxy cluster = STSClusterProxy.create("http://my.sts.1/sts  http://my.sts.2/sts"); // note the double space between the URL's
        Assert.assertEquals(2, cluster.getProxies().size());
    }


    @Test
    public void shouldGetTheSequenceRight() {
        final STSClusterProxy cluster = STSClusterProxy.create("1 2 3");
        Assert.assertEquals("First url", "1", cluster.getProxies().get(0).getUrlString());
        Assert.assertEquals("Second url", "2", cluster.getProxies().get(1).getUrlString());
        Assert.assertEquals("Third url", "3", cluster.getProxies().get(2).getUrlString());
    }

    @Test
    public void shouldGetTheUrlStringRight() {
        final STSClusterProxy cluster = STSClusterProxy.create("1 2 3");
        Assert.assertEquals("1 2 3", cluster.getUrlString());
    }
}
