/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.sts;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;

public class STSClusterProxyTest {
    private List<STSProxy> proxies;
    private STSProxy cluster;
    private TestProxy goodProxy1;
    private TestProxy goodProxy2;
    private TestProxy badProxy1;
    private TestProxy badProxy2;
    private TestProxy badProxy3;

    @Before
    public void setup() {
        proxies = new ArrayList<STSProxy>();

        goodProxy1 = TestProxy.goodInstance("goodProxy1");
        goodProxy2 = TestProxy.goodInstance("goodProxy2");

        badProxy1 = TestProxy.failingInstance("badProxy1");
        badProxy2 = TestProxy.failingInstance("badProxy2");
        badProxy3 = TestProxy.failingInstance("badProxy3");
    }

    @Test
    public void willReturnWhatItGetsFromASingletonList() {
        proxies.add(goodProxy1);
        cluster = new STSClusterProxy(proxies);
        Assert.assertEquals("goodProxy1", cluster.callSTS(null));
    }

    @Test
    public void willOnlyCallFirstProxyWhenAllAreUp() {
        proxies.add(goodProxy1);
        proxies.add(goodProxy2);
        cluster = new STSClusterProxy(proxies);
        Assert.assertEquals("goodProxy1", cluster.callSTS(null));
        Assert.assertFalse(goodProxy2.wasCalled());
    }

    @Test
    public void willOnlyCallFirstLiveSecondaryProxyWhenPrimaryIsDown() {
        proxies.add(badProxy1);
        proxies.add(goodProxy2);
        proxies.add(goodProxy1);
        cluster = new STSClusterProxy(proxies);
        Assert.assertEquals("goodProxy2", cluster.callSTS(null));
        Assert.assertTrue("badProxy1 wasn't called", badProxy1.wasCalled()); // it shouldn't just skip to the good parts
        Assert.assertFalse("goodProxy1 called", goodProxy1.wasCalled());
    }

    @Test(expected = STSUnreachableException.class)
    public void willThrowExceptionWhenAllProxiesAreDown() {
        proxies.add(badProxy1);
        proxies.add(badProxy2);
        proxies.add(badProxy3);
        cluster = new STSClusterProxy(proxies);
        cluster.callSTS(null);
    }

    @Test
    public void willCallAllProxiesBeforeGivingUp() {
        proxies.add(badProxy1);
        proxies.add(badProxy2);
        proxies.add(badProxy3);
        cluster = new STSClusterProxy(proxies);

        try {
            cluster.callSTS(null);
        } catch (STSUnreachableException ignore) {}

        Assert.assertTrue("badProxy1 wasn't called", badProxy1.wasCalled());
        Assert.assertTrue("badProxy2 wasn't called", badProxy2.wasCalled());
        Assert.assertTrue("badProxy3 wasn't called", badProxy3.wasCalled());
    }

}
