/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.sts;

import org.w3c.dom.Document;

class TestProxy implements STSProxy {
    private final String response;
    private boolean doFail;
    private boolean wasCalled;

    public static TestProxy failingInstance(String response) {
        return new TestProxy(response, true);
    }

    public static TestProxy goodInstance(String response) {
        return new TestProxy(response, false);
    }

    TestProxy(String response, boolean doFail) {
        this.response = response;
        this.doFail = doFail;
    }

    public void turnGood() {
        doFail = false;
    }

    public void turnBad() {
        doFail = true;
    }

    public boolean wasCalled() {
        return wasCalled;
    }


    public String callSTS(Document doc) throws STSUnreachableException {
        wasCalled = true;
        if (doFail) throw new STSUnreachableException("Failing as I was instructed to");

        return response;
    }


    public String getUrlString() {
        return "testUrlString";
    }


    public String toString() {
        return "Test STS proxy " + response;
    }
}
