/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.testclient;

import com.sun.xml.bind.api.JAXBRIContext;
import com.sun.xml.ws.api.message.Headers;
import com.sun.xml.ws.developer.WSBindingProvider;
import com.trifork.sosigw.api.Constants;
import com.trifork.sosigw.config.RuntimeConfig;
import com.trifork.sosigw.test.DoInvokeExceptionResponse;
import com.trifork.sosigw.test.DoInvokeParamType;
import com.trifork.sosigw.test.StubService;
import com.trifork.sosigw.test.StubType;

import dk.medcom.dgws._2006._04.dgws_1_0.Header;

import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.namespace.QName;
import javax.xml.ws.handler.MessageContext;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;


public class ServiceStubClient {
    public static final String JAX_WS_TIMEOUT_CONNECT = "com.sun.xml.ws.connect.timeout";  // jax-ws-RI setting...
    public static final String JAX_WS_TIMEOUT_REQUEST = "com.sun.xml.ws.request.timeout";  // jax-ws-RI setting...
    
    private StubType stub;

    public ServiceStubClient(String testHostName) {
        this(testHostName, true, true, true, false, false);
    }

    public ServiceStubClient(String testHostName, boolean addWSAdressingTo, boolean addWSAdressingAction, boolean addWSAdressingReplyTo, boolean addSosiGWPassThrough, boolean addSosiGWMaxAge) {
        try {
            stub = createStub1(testHostName, addWSAdressingTo, addWSAdressingAction, addWSAdressingReplyTo, addSosiGWPassThrough, addSosiGWMaxAge);
        } catch (MalformedURLException e) {
            throw new RuntimeException(e);
        } catch (JAXBException e) {
            throw new RuntimeException(e);
        }
    }
    
    public String doInvoke(Header medcomHeader, Security securityHeader, String requestPayload, int responsePayloadSize, int responseDelay, String sosigwProxyURL, boolean gzip) throws JAXBException, DoInvokeExceptionResponse, MalformedURLException {
        DoInvokeParamType param = new DoInvokeParamType();
        param.setRequestPayloadParam(requestPayload);
        param.setRequestPayloadSizeParam(responsePayloadSize);
        param.setServiceDelay(responseDelay);

        ((javax.xml.ws.BindingProvider)stub).getRequestContext().put(javax.xml.ws.BindingProvider.ENDPOINT_ADDRESS_PROPERTY, sosigwProxyURL);
        ((javax.xml.ws.BindingProvider)stub).getRequestContext().put(JAX_WS_TIMEOUT_CONNECT, new Integer(5 * 1000));
        ((javax.xml.ws.BindingProvider)stub).getRequestContext().put(JAX_WS_TIMEOUT_REQUEST, new Integer(200 * 1000));
        Map<String, List<String>> headers = (Map<String, List<String>>)((javax.xml.ws.BindingProvider)stub).
                getRequestContext().get(MessageContext.HTTP_REQUEST_HEADERS);
        if (headers == null ) {
            headers = new HashMap<String, List<String>>();
        }
        
        headers.put("X_NSP_FLOWID", Collections.singletonList(UUID.randomUUID().toString()));
        headers.put("X_NSP_MY_HTTP_FORWARD_HEADER1", Collections.singletonList("optional_header1"));
        headers.put("X_NSP_MY_HTTP_FORWARD_HEADER2", Collections.singletonList("optional_header2"));
        if (gzip) {
            headers.put("Accept-Encoding", Collections.singletonList("gzip"));
        }
        ((javax.xml.ws.BindingProvider)stub).getRequestContext().put(MessageContext.HTTP_REQUEST_HEADERS, headers);

        return stub.doInvoke(securityHeader, medcomHeader, param);
    }

    public void addHttpHeader(String name, String value) {
        Map<String, List<String>> headers = new HashMap<String, List<String>>();
        headers.put(name, Collections.singletonList(value));
        ((javax.xml.ws.BindingProvider)stub).getRequestContext().put(MessageContext.HTTP_REQUEST_HEADERS, headers);
    }

    @SuppressWarnings("resource")
    private StubType createStub1(String testHostName, boolean addWSAdressingTo, boolean addWSAdressingAction, boolean addWSAdressingReplyTo, boolean addSosiGWPassThrough, boolean addSosiGWMaxAge) throws JAXBException, MalformedURLException {
        StubService service = new StubService(new URL("file:test/etc/ServiceStub.wsdl"), new QName("http://www.trifork.com/sosigw/test", "StubService"));

        StubType stub = service.getDoInvokePort();
        WSBindingProvider bp = (WSBindingProvider)stub;

        final String endpointAddress = "http://" + testHostName + "/stubservice/doInvoke";
        bp.setAddress(endpointAddress);
        
        List<com.sun.xml.ws.api.message.Header> headers = new ArrayList<com.sun.xml.ws.api.message.Header>();
        if (addWSAdressingTo) {
            headers.add(Headers.create(new QName("http://www.w3.org/2005/08/addressing","To"), endpointAddress));
        }
        if (addWSAdressingAction) {
            headers.add(Headers.create(new QName("http://www.w3.org/2005/08/addressing","Action"),"http://www.trifork.com/sosigw/test#doInvoke"));
        }
        if (addWSAdressingReplyTo) {
            headers.add(Headers.create((JAXBRIContext)JAXBContext.newInstance(ReplyToHeader.class),new ReplyToHeader()));
        }
        if (addSosiGWPassThrough) {
            headers.add(Headers.create(new QName(Constants.HTTP_SOSI_DK_GW_2007_09_01, Constants.PASS_THROUGH),"1"));
        }
        if (addSosiGWMaxAge) {
            headers.add(Headers.create(new QName(Constants.HTTP_SOSI_DK_GW_2007_09_01, Constants.MAX_AGE),"60"));
        }
        headers.add(Headers.create(new QName("http://www.w3.org/2005/08/addressing","MessageID"),"2"));
        
        bp.setOutboundHeaders((com.sun.xml.ws.api.message.Header[])headers.toArray(new com.sun.xml.ws.api.message.Header[headers.size()]));
      return stub;
  }

}

@XmlRootElement(namespace="http://www.w3.org/2005/08/addressing",name="ReplyTo")
class ReplyToHeader {
  @XmlElement(name="Address")
  String value = "http://www.w3.org/2005/08/addressing/anonymous";
}



