/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.util;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNotSame;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertSame;

import java.io.ByteArrayInputStream;

import javax.xml.parsers.DocumentBuilderFactory;

import org.junit.Test;
import org.w3c.dom.Document;
import org.w3c.dom.ls.LSParser;
import org.w3c.dom.ls.LSSerializer;


public class DOMUtilTest {
    
    private LSSerializer s;
    private LSParser p;
    
    @Test
    public void getDomLSSerializerShouldCacheThreadLocal() throws Exception {
        LSSerializer s1 = DOMUtil.getDomLSSerializer();
        LSSerializer s2 = DOMUtil.getDomLSSerializer();
        
        assertSame(s1, s2);
        
        Thread t = new Thread() {
            public void run() {
                s = DOMUtil.getDomLSSerializer();
            }
        };
        t.start(); t.join();
        
        assertNotSame(s1, s);
    }
    
    @Test
    public void getDomParserShouldCacheThreadLocal() throws Exception {
        LSParser p1 = DOMUtil.getDomLSParser();
        LSParser p2 = DOMUtil.getDomLSParser();
        
        assertSame(p1, p2);
        
        Thread t = new Thread() {
            public void run() {
                p = DOMUtil.getDomLSParser();
            }
        };
        t.start();t.join();
       
        assertNotSame(p1, p);
    }
    
    @Test
    public void testGetIdCardElement() throws Exception {
        String xml = "<test></test>";
        
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true);
        Document doc = dbf.newDocumentBuilder().parse(new ByteArrayInputStream(xml.getBytes()));
        
        assertNull(DOMUtil.getIDCardElement(doc));
        
        xml = "<test><saml:Assertion xmlns:saml='urn:oasis:names:tc:SAML:2.0:assertion' ID='IDCard'></saml:Assertion></test>";
        doc = dbf.newDocumentBuilder().parse(new ByteArrayInputStream(xml.getBytes()));
        assertNotNull(DOMUtil.getIDCardElement(doc));
    }

}
