/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.util;

import static org.junit.Assert.*;

import java.io.IOException;
import java.io.InputStream;
import java.util.Map;

import org.apache.commons.io.IOUtils;
import org.junit.Test;

import dk.sosi.gw._2007_09.DGWSFault;

public class MultipartHeaderFixTest {
    
    private MultipartHeaderFix multipartHeaderFix = new MultipartHeaderFix();

    // Make sure the header filter removes the attachements
    @Test
    public void testHeaderFilter() throws Exception {
        String contentType = "multipart/related;start=\"<rootpart*c40fbefb-c8d3-4258-aa80-cf891234b14e@example.jaxws.sun.com>\";type=\"application/xop+xml\";boundary=\"uuid:c40fbefb-c8d3-4258-aa80-cf891234b14e\";start-info=\"text/xml\"";
        final InputStream request = MultipartHeaderFixTest.class.getClassLoader().getResourceAsStream("com/trifork/sosigw/util/mtom-request.dat");
        multipartHeaderFix.extractAndFilterMtomHeaderAttachents(request, contentType);
        assertEquals(2380, multipartHeaderFix.getBytesRemoved());
        Map<String, String> headerAttachmentMap = multipartHeaderFix.getHeaderAttachmentMap();
        assertEquals(3, headerAttachmentMap.size());
    }

    // The filter should not remove anything if the contentType is invalid
    @Test
    public void testHeaderFilterBadContentType() throws Exception {
        String contentType = "text/xml";
        final InputStream request = MultipartHeaderFixTest.class.getClassLoader().getResourceAsStream("com/trifork/sosigw/util/mtom-request.dat");
        multipartHeaderFix.extractAndFilterMtomHeaderAttachents(request, contentType);
        assertEquals(0, multipartHeaderFix.getBytesRemoved());
    }

    @Test(expected = DGWSFault.class)
    public void testHeaderFilterWithCorruptIDCard() throws Exception {
        String contentType = "multipart/related;start=\"<rootpart*c40fbefb-c8d3-4258-aa80-cf891234b14e@example.jaxws.sun.com>\";type=\"application/xop+xml\";boundary=\"uuid:c40fbefb-c8d3-4258-aa80-cf891234b14e\";start-info=\"text/xml\"";
        final InputStream request = MultipartHeaderFixTest.class.getClassLoader().getResourceAsStream("com/trifork/sosigw/util/mtom-request-bad-idcard.dat");

        multipartHeaderFix.extractAndFilterMtomHeaderAttachents(request, contentType);
    }
    
}
