/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.ws;

import com.sun.org.apache.xerces.internal.jaxp.datatype.XMLGregorianCalendarImpl;
import com.trifork.sosigw.api.Constants;
import dk.medcom.dgws._2006._04.dgws_1_0.Header;
import dk.medcom.dgws._2006._04.dgws_1_0.Linking;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.constants.IDValues;
import dk.sosi.seal.model.constants.MedcomAttributes;
import dk.sosi.seal.model.constants.SOSIAttributes;
import dk.sosi.seal.model.constants.SubjectIdentifierTypeValues;
import oasis.names.tc.saml._2_0.assertion.*;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_utility_1_0.Timestamp;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import java.security.*;
import java.util.Calendar;
import java.util.GregorianCalendar;

public class IdCardTestUtils {

    private static final boolean USE_LOCALTIME_IN_SAML = true; // DGWS-1.0 says localtime in all timestamps. Bad stuff.

    public static Attribute makeAttribute(String name, String format, String value) {
        Attribute attribute = new Attribute();
        attribute.setName(name);
        if (format != null) {
            attribute.setNameFormat(format);
        }
        attribute.setAttributeValue(value);
        return attribute;
    }

//    public static Header makeIDCardInHeader() {
//        Header header = new Header();
//        header.setSecurity(makeIDCardInSecurity());
//        return header;
//    }

    public static Security makeIDCardInSecurity(String nameIDString) {
        Security security = new Security();
        security.setTimestamp(makeTimestamp());
        security.setAssertion(makeIDCard(nameIDString));
        return security;
    }

    public static Timestamp makeTimestamp() {
        Timestamp timestamp = new Timestamp();
        try {
            timestamp.setCreated(DatatypeFactory.newInstance().newXMLGregorianCalendar(new GregorianCalendar()));
        } catch (DatatypeConfigurationException e) {
            throw new IllegalStateException(e);
        }
        return timestamp;
    }
    
    private static AssertionType makeIDCard(String nameIDString) {
        AssertionType assertion = new AssertionType();
        assertion.setIssuer("sosigwtester");
        assertion.setId(IDValues.IDCARD);

        NameIDType nameID = new NameIDType();
        nameID.setValue(nameIDString);
        nameID.setFormat(SubjectIdentifierTypeValues.OTHER);
        Subject subject = new Subject();
        subject.setNameID(nameID );
        assertion.setSubject(subject);

        AttributeStatement stmt = new AttributeStatement();
        
        stmt.setId(IDValues.IDCARD_DATA);
        stmt.getAttribute().add(makeAttribute(SOSIAttributes.IDCARD_VERSION, null, "1.0.1"));
        stmt.getAttribute().add(makeAttribute(SOSIAttributes.IDCARD_TYPE, null, IDCard.IDCARDTYPE_USER));
        stmt.getAttribute().add(makeAttribute(SOSIAttributes.AUTHENTICATION_LEVEL, null, "1"));
        stmt.getAttribute().add(makeAttribute(SOSIAttributes.IDCARD_ID, null, "id-card-id"));
        assertion.getAttributeStatement().add(stmt);
    
        stmt = new AttributeStatement();
        stmt.setId(IDValues.USER_LOG);
        stmt.getAttribute().add(makeAttribute(MedcomAttributes.USER_CIVIL_REGISTRATION_NUMBER, null, "0501792275"));
        stmt.getAttribute().add(makeAttribute(MedcomAttributes.USER_GIVEN_NAME, null, "Tester"));
        stmt.getAttribute().add(makeAttribute(MedcomAttributes.USER_SURNAME, null, "Blåbærgrød"));
        stmt.getAttribute().add(makeAttribute(MedcomAttributes.USER_EMAIL_ADDRESS, null, "test@trifork.com"));
        stmt.getAttribute().add(makeAttribute(MedcomAttributes.USER_ROLE, null, "læge"));
        stmt.getAttribute().add(makeAttribute(MedcomAttributes.USER_OCCUPATION, null, "Gråstenæblegrødskoger"));
        stmt.getAttribute().add(makeAttribute(MedcomAttributes.USER_AUTHORIZATION_CODE, null, "J0184"));
        assertion.getAttributeStatement().add(stmt);
    
        stmt = new AttributeStatement();
        stmt.setId(IDValues.SYSTEM_LOG);
        stmt.getAttribute().add(makeAttribute(MedcomAttributes.IT_SYSTEM_NAME, null, Constants.SOSIGW_IT_SYSTEM_TESTNAME));
        stmt.getAttribute().add(makeAttribute(MedcomAttributes.CARE_PROVIDER_ID, SubjectIdentifierTypeValues.CVR_NUMBER, "20921897"));
        stmt.getAttribute().add(makeAttribute(MedcomAttributes.CARE_PROVIDER_NAME, null, "Vi brækker dit hårde øre"));
        assertion.getAttributeStatement().add(stmt);
    
        assertion.setConditions(new Conditions());
        
        Calendar notBefore = Calendar.getInstance();
        notBefore.add(Calendar.MINUTE, -5);
        
        
        Calendar notOnOrAfter = Calendar.getInstance();
        notOnOrAfter.add(Calendar.HOUR, 8);
        
        if (USE_LOCALTIME_IN_SAML) {
            notBefore.add(Calendar.MILLISECOND, notBefore.getTimeZone().getOffset(notBefore.getTimeInMillis()));
            notOnOrAfter.add(Calendar.MILLISECOND, notOnOrAfter.getTimeZone().getOffset(notOnOrAfter.getTimeInMillis()));
        }

        assertion.getConditions().setNotBefore(new XMLGregorianCalendarImpl((GregorianCalendar)notBefore));
        assertion.getConditions().setNotOnOrAfter(new XMLGregorianCalendarImpl((GregorianCalendar)notOnOrAfter));
        return assertion;
    }

    public static byte[] signDigest(byte[] digestValue, PrivateKey key) throws NoSuchAlgorithmException, InvalidKeyException,
            SignatureException {
    
        // Sign the document with user Moces signature
        byte[] siBytes = digestValue;
        Signature jceSign;
        jceSign = Signature.getInstance("SHA1withRSA");
        jceSign.initSign(key);
        jceSign.update(siBytes);

        return jceSign.sign();

    }

    @XmlRootElement(namespace="http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd", name="Header")
    public static class MedcomHeader {
      @XmlElement(namespace="http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd", name="SecurityLevel")
      String securityLevel = "1";

      @XmlElement(namespace="http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd", name="RequestPriority")
      String requestPriority = "RUTINE";

      @XmlRootElement(namespace="http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd", name="Linking")
      public static class Linking {
          @XmlElement(namespace="http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd", name="MessageID")
          String messageID = "mitMsgIdFraClient";
          @XmlElement(namespace="http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd", name="FlowID")
          String flowID = "mitFlowIdFraClient";
      }
      @XmlElement(namespace="http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd", name="Linking")
      Linking linking = new Linking();
    }

    public static Security makeIDCardInSecurity() {
        return makeIDCardInSecurity("MitBrugerNavnSomErMereEnd10tegnLangt");
    }

    public static Header makeMedcomHeader(String flowID, String messageID, String flowStatus) {
        Header hd = new Header();
        Linking ln = new Linking();
        ln.setFlowID(flowID);
        ln.setMessageID(messageID);
        hd.setLinking(ln);
        hd.setFlowStatus(flowStatus);
        
        return hd;
    }
}

