/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.ws;

import static org.junit.Assert.fail;

import javax.xml.ws.soap.SOAPFaultException;

import org.junit.Before;
import org.junit.Test;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;

import com.trifork.sosigw.testclient.ServiceStubClient;

import dk.medcom.dgws._2006._04.dgws_1_0.Header;

public class ProxyRequestTest extends AbstractIntegrationTests {

    @Before
    public void setup() throws Exception {
        createSignedIdCard();
    }
   
    @Test(timeout = 60000)
    public void testProxyRequest() throws Exception {
        int responsePayloadSize = 10;

        System.out.println("Test Client, payload size: " + responsePayloadSize);
        String requestPayload = "small payload";
        
        ServiceStubClient client = new ServiceStubClient("localhost:8586");

        Header medcomHeader = null;
        Security securityHeader = IdCardTestUtils.makeIDCardInSecurity();

        String result = client.doInvoke(medcomHeader, securityHeader, requestPayload, responsePayloadSize, 0, SERVER_URL + "/proxy/soap-request", false);
        
        System.out.println("Got: " + (result == null ? null : result.length()) + " bytes");
    }

    @Test(timeout = 60000)
    public void testProxyRequestGzip() throws Exception {
        int responsePayloadSize = 10;

        System.out.println("Test Client, payload size: " + responsePayloadSize);
        String requestPayload = "small payload";
        
        ServiceStubClient client = new ServiceStubClient("localhost:8586");

        Header medcomHeader = null;
        Security securityHeader = IdCardTestUtils.makeIDCardInSecurity();

        String result = client.doInvoke(medcomHeader, securityHeader, requestPayload, responsePayloadSize, 0, SERVER_URL + "/proxy/soap-request", true);
        
        System.out.println("Got: " + (result == null ? null : result.length()) + " bytes");
    }

    @Test(timeout = 60000)
    public void testProxyRequestWrongServiceHost() throws Exception {
        int responsePayloadSize = 10;

        System.out.println("Test Client, payload size: " + responsePayloadSize);
        String requestPayload = "small payload";
        
        ServiceStubClient client = new ServiceStubClient("NOHOST");

        Header medcomHeader = IdCardTestUtils.makeMedcomHeader("clientFlowID", "clientMessageID", "clientFlowStatus");
        
        Security securityHeader = IdCardTestUtils.makeIDCardInSecurity();

        try {
            client.doInvoke(medcomHeader, securityHeader, requestPayload, responsePayloadSize, 0, SERVER_URL + "/proxy/soap-request", false);
        } catch (SOAPFaultException ex) {
            // This is what we expected.
            return;
        }

        fail("Expected an exception here");
    }
}
