/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.ws;

import com.trifork.sosigw.Factory;
import com.trifork.sosigw.cluster.IDCardClusterCacheEntry;
import com.trifork.sosigw.cluster.IDCardClusterCacheImpl;
import com.trifork.sosigw.config.StaticConfigImpl;
import com.trifork.sosigw.util.DOMUtil;
import com.trifork.sosigw.util.SosiSeal;
import com.trifork.sosigw.util.Util;
import dk.sosi.gw._2007_09.DGWSFault;
import dk.sosi.seal.model.*;
import dk.sosi.seal.xml.XmlUtil;
import org.apache.catalina.core.DummyRequest;
import org.apache.xml.security.utils.XMLUtils;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.servlet.http.HttpServletRequest;

public class SosiGWFacadeBusinessTest {
    @Before
    public void before() {
        StaticConfigImpl.initSettings();
        HttpServletRequest httpServletRequest = new DummyRequest();
        Util.setActiveHttpRequest(httpServletRequest);
        Factory.setTheIDCardCache(new IDCardClusterCacheImpl(null));
    }

    @Test
    public void shouldSerializeIdCardIncludingBytesForSigningToCacheWhenClientRequestsIdCardForSigning() throws DGWSFault {
        // Regression test for NSP-112

        String nameId = "testuser";
        UserInfo userInfo = new UserInfo("1122334455", "Anders", "And", "anders@and.dk", "", "And", "");
        CareProvider careProvider = new CareProvider("medcom:cvrnumber", "id", "orgName");
        UserIDCard idCard = SosiSeal.getSosiSealFactory().createNewUserIDCard("itSystemName", userInfo, careProvider, AuthenticationLevel.MOCES_TRUSTED_USER, nameId, null, null, null);
        SosiGWFacadeBusiness.requestIdCardDigestForSigningImpl(idCard, nameId);

        final IDCardClusterCacheEntry idCardCacheEntry = Factory.getIDCardCache().findIDCardOrNull(nameId);
        Assert.assertNotNull(idCardCacheEntry);

        final UserIDCard idCardFromCache = idCardCacheEntry.getIdCard();
        Assert.assertNotNull(idCardFromCache);

        // As a sanity check, verify that the non-serialized id card contains a SignatureValue
        final Element domForIdcard = (Element)calcDomElementForIDCard(idCard);
        final NodeList signatureValues = domForIdcard.getElementsByTagNameNS("http://www.w3.org/2000/09/xmldsig#", "SignatureValue");
        Assert.assertEquals("# of SignatureValue nodes in non serialized id card", 1, signatureValues.getLength());

        final Element domForSerializedIdCard = (Element)idCardCacheEntry.getDOMelement();
        final NodeList signatureValuesForSerialized = domForSerializedIdCard.getElementsByTagNameNS("http://www.w3.org/2000/09/xmldsig#", "SignatureValue");
        Assert.assertEquals("# of SignatureValue nodes in serialized id card", 1, signatureValuesForSerialized.getLength());
    }

    private Node calcDomElementForIDCard(IDCard idCard) {
        SecurityTokenRequest securityTokenRequest = SosiSeal.getSosiSealFactory().createNewSecurityTokenRequest();
        securityTokenRequest.setIDCard(idCard);
        Document doc = securityTokenRequest.serialize2DOMDocument();

        Element element = DOMUtil.getIDCardElement(doc);
        return element;
    }


}
