/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.ws;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.fail;

import javax.xml.ws.soap.SOAPFaultException;

import junit.framework.Assert;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;

import com.trifork.sosigw.config.RuntimeConfig;
import com.trifork.sosigw.config.RuntimeConfigEditState;
import com.trifork.sosigw.config.RuntimeConfigState;
import com.trifork.sosigw.testclient.ServiceStubClient;

/**
 * @author $LastChangedBy: jso $ $LastChangedDate: 2015-12-10 14:30:38 +0100 (Tor, 10 Dec 2015) $
 * @version $Revision: 2036 $
 */
public class WSAdressingTest extends AbstractIntegrationTests {

    private RuntimeConfigState previous;

    @Before
    public void enableDecouplingInServer() {
        RuntimeConfigEditState editableVersion = RuntimeConfig.getRuntimeConfigState().getEditableVersion();
        previous = editableVersion.getResult();
        editableVersion.addEntry(RuntimeConfig.CATEGORY_PROPERTIES, RuntimeConfig.KEY_DECOUPLING_URL, new String[] { "http://localhost:8586/decoupling/doInvoke"} );
        RuntimeConfigState result = editableVersion.getResult();
        RuntimeConfig.setRuntimeConfigState(result);
    }
    
    @After
    public void disableDecouplingInServer() {
        RuntimeConfig.setRuntimeConfigState(previous);
    }

    @Test(timeout = 60000)
    public void testFaultWhenDecouplingUrlNotConfigured() throws Exception {
        disableDecouplingInServer();
        createSignedIdCard();
        
        Security securityHeader = IdCardTestUtils.makeIDCardInSecurity();
        ServiceStubClient client = new ServiceStubClient("localhost:8586", false, false, false, false, false);
        
        try {
            client.doInvoke(null, securityHeader, "", 0, 0, SERVER_URL + "/proxy/soap-request", false);
            fail("Expected a failure when no DCC URL was configured");
        } catch (SOAPFaultException expected) {
            
        }
    }

    @Test(timeout = 60000)
    public void testDecouplingNotCalledWhenWSAddressingToPresent() throws Exception {
        createSignedIdCard();
        
        Security securityHeader = IdCardTestUtils.makeIDCardInSecurity();
        ServiceStubClient client = new ServiceStubClient("localhost:8586", true, true, true, false, false);
        
        String response = client.doInvoke(null, securityHeader, "", 0, 0, SERVER_URL + "/proxy/soap-request", false);
        assertEquals("Incorrect response from service stub", "#", response);
        Assert.assertEquals("", ServiceCalled.TEST, lastServiceCalled);
        assertEquals("Incorrect soap action used", "http://www.trifork.com/sosigw/test#doInvoke", lastSoapActionCalled);
    }

    @Test(timeout = 60000)
    public void testDecouplingCalledWhenWSAddressingToIsMissing() throws Exception {
        createSignedIdCard();
        
        Security securityHeader = IdCardTestUtils.makeIDCardInSecurity();
        ServiceStubClient client = new ServiceStubClient("localhost:8586", false, true, true, false, false);
        
        String response = client.doInvoke(null, securityHeader, "", 0, 0, SERVER_URL + "/proxy/soap-request", false);
        assertEquals("Incorrect response from service stub", "#", response);
        Assert.assertEquals("", ServiceCalled.DCC, lastServiceCalled);
        assertEquals("Incorrect soap action used", "http://www.trifork.com/sosigw/test#doInvoke", lastSoapActionCalled);
    }

    @Test(timeout = 60000)
    public void testHTTPSoapActionUsedWhenWSAddressingActionIsMissing() throws Exception {
        createSignedIdCard();
        
        Security securityHeader = IdCardTestUtils.makeIDCardInSecurity();
        ServiceStubClient client = new ServiceStubClient("localhost:8586", false, false, true, false, false);
        
        String response = client.doInvoke(null, securityHeader, "", 0, 0, SERVER_URL + "/proxy/soap-request", false);
        assertEquals("Incorrect response from service stub", "#", response);
        Assert.assertEquals("", ServiceCalled.DCC, lastServiceCalled);
        assertEquals("Incorrect soap action used", "\"http://www.trifork.com/sosigw/test#doInvoke\"", lastSoapActionCalled);
    }
}
